@@ -1,18 +1,19 @@
 # -*- test-case-name: twisted.test.test_application,twisted.test.test_twistd -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-import sys, os, pdb, getpass, traceback, signal, warnings
+import sys, os, pdb, getpass, traceback, signal
+from operator import attrgetter
 
 from twisted.python import runtime, log, usage, failure, util, logfile
 from twisted.python.versions import Version
-from twisted.python.reflect import qual
+from twisted.python.reflect import qual, namedAny
 from twisted.python.deprecate import deprecated
 from twisted.python.log import ILogObserver
 from twisted.persisted import sob
 from twisted.application import service, reactors
 from twisted.internet import defer
-from twisted import copyright
+from twisted import copyright, plugin
 
 # Expose the new implementation of installReactor at the old location.
 from twisted.application.reactors import installReactor
@@ -174,11 +175,6 @@ class AppProfiler(object):
         saveStats = options.get("savestats", False)
         profileOutput = options.get("profile", None)
         self.profiler = options.get("profiler", "hotshot").lower()
-        if options.get("nothotshot", False):
-            warnings.warn("The --nothotshot option is deprecated. Please "
-                          "specify the profiler name using the --profiler "
-                          "option", category=DeprecationWarning)
-            self.profiler = "profile"
         if self.profiler in self.profilers:
             profiler = self.profilers[self.profiler](profileOutput, saveStats)
             self.run = profiler.run
@@ -187,32 +183,6 @@ class AppProfiler(object):
 
 
 
-def runWithProfiler(reactor, config):
-    """
-    DEPRECATED in Twisted 8.0.
-
-    Run reactor under standard profiler.
-    """
-    warnings.warn("runWithProfiler is deprecated since Twisted 8.0. "
-                  "Use ProfileRunner instead.", DeprecationWarning, 2)
-    item = AppProfiler(config)
-    return item.run(reactor)
-
-
-
-def runWithHotshot(reactor, config):
-    """
-    DEPRECATED in Twisted 8.0.
-
-    Run reactor under hotshot profiler.
-    """
-    warnings.warn("runWithHotshot is deprecated since Twisted 8.0. "
-                  "Use HotshotRunner instead.", DeprecationWarning, 2)
-    item = AppProfiler(config)
-    return item.run(reactor)
-
-
-
 class AppLogger(object):
     """
     Class managing logging faciliy of the application.
@@ -221,6 +191,9 @@ class AppLogger(object):
         default.
     @type _logfilename: C{str}
 
+    @ivar _observerFactory: Callable object that will create a log observer, or
+        None.
+
     @ivar _observer: log observer added at C{start} and removed at C{stop}.
     @type _observer: C{callable}
     """
@@ -228,20 +201,26 @@ class AppLogger(object):
 
     def __init__(self, options):
         self._logfilename = options.get("logfile", "")
+        self._observerFactory = options.get("logger") or None
 
 
     def start(self, application):
         """
         Initialize the logging system.
 
-        If an L{ILogObserver} component has been set on C{application}, then
-        it will be used as the log observer.  Otherwise a log observer will be
-        created based on the command-line options.
-        
+        If a customer logger was specified on the command line it will be
+        used. If not, and an L{ILogObserver} component has been set on
+        C{application}, then it will be used as the log observer.  Otherwise a
+        log observer will be created based on the command-line options for
+        built-in loggers (e.g. C{--logfile}).
+
         @param application: The application on which to check for an
             L{ILogObserver}.
         """
-        observer = application.getComponent(ILogObserver, None)
+        if self._observerFactory is not None:
+            observer = self._observerFactory()
+        else:
+            observer = application.getComponent(ILogObserver, None)
 
         if observer is None:
             observer = self._getLogObserver()
@@ -292,9 +271,11 @@ def fixPdb():
         reactor.callLater(0, reactor.stop)
         return 1
 
+
     def help_stop(self):
         print """stop - Continue execution, then cleanly shutdown the twisted reactor."""
 
+
     def set_quit(self):
         os._exit(0)
 
@@ -330,12 +311,6 @@ def runReactorWithLogging(config, oldstdout, oldstderr, profiler=None, reactor=N
         if config['profile']:
             if profiler is not None:
                 profiler.run(reactor)
-            else:
-                # Backward compatible code
-                if not config['nothotshot']:
-                    runWithHotshot(reactor, config)
-                else:
-                    runWithProfiler(reactor, config)
         elif config['debug']:
             sys.stdout = oldstdout
             sys.stderr = oldstderr
@@ -410,30 +385,12 @@ class ApplicationRunner(object):
         self.preApplication()
         self.application = self.createOrGetApplication()
 
-
-        getLogObserverLegacy = getattr(self, 'getLogObserver', None)
-        if getLogObserverLegacy is not None:
-            warnings.warn("Specifying a log observer with getLogObserver is "
-                          "deprecated. Please use a loggerFactory instead.",
-                          category=DeprecationWarning)
-            self.startLogging(self.getLogObserver())
-        else:
-            self.logger.start(self.application)
+        self.logger.start(self.application)
 
         self.postApplication()
         self.logger.stop()
 
 
-    def startLogging(self, observer):
-        """
-        Initialize the logging system. DEPRECATED.
-
-        @param observer: The observer to add to the logging system.
-        """
-        log.startLoggingWithObserver(observer)
-        self.logger._initialLog()
-
-
     def startReactor(self, reactor, oldstdout, oldstderr):
         """
         Run the reactor with the given configuration.  Subclasses should
@@ -498,7 +455,7 @@ class ApplicationRunner(object):
 
 def getApplication(config, passphrase):
     s = [(config[t], t)
-           for t in ['python', 'xml', 'source', 'file'] if config[t]][0]
+           for t in ['python', 'source', 'file'] if config[t]][0]
     filename, style = s[0], {'file':'pickle'}.get(s[1],s[1])
     try:
         log.msg("Loading %s..." % filename)
@@ -523,31 +480,30 @@ Please read the 'Using Application' HOWTO for details.
 
 
 
-def reportProfile(report_profile, name):
-    """
-    DEPRECATED since Twisted 8.0. This does nothing.
-    """
-    warnings.warn("reportProfile is deprecated and a no-op since Twisted 8.0.",
-                  category=DeprecationWarning)
+def _reactorAction():
+    return usage.CompleteList([r.shortName for r in reactors.getReactorTypes()])
 
 
-
-def _reactorZshAction():
-    return "(%s)" % " ".join([r.shortName for r in reactors.getReactorTypes()])
-
 class ReactorSelectionMixin:
     """
     Provides options for selecting a reactor to install.
+
+    If a reactor is installed, the short name which was used to locate it is
+    saved as the value for the C{"reactor"} key.
     """
-    zsh_actions = {"reactor" : _reactorZshAction}
+    compData = usage.Completions(
+        optActions={"reactor": _reactorAction})
+
     messageOutput = sys.stdout
+    _getReactorTypes = staticmethod(reactors.getReactorTypes)
 
 
     def opt_help_reactors(self):
         """
         Display a list of possibly available reactor names.
         """
-        for r in reactors.getReactorTypes():
+        rcts = sorted(self._getReactorTypes(), key=attrgetter('shortName'))
+        for r in rcts:
             self.messageOutput.write('    %-4s\t%s\n' %
                                      (r.shortName, r.description))
         raise SystemExit(0)
@@ -574,6 +530,8 @@ class ReactorSelectionMixin:
                    "%s.\nSee the list of available reactors with "
                    "--help-reactors" % (e,))
             raise usage.UsageError(msg)
+        else:
+            self["reactor"] = shortName
     opt_r = opt_reactor
 
 
@@ -581,18 +539,22 @@ class ReactorSelectionMixin:
 
 class ServerOptions(usage.Options, ReactorSelectionMixin):
 
+    longdesc = ("twistd reads a twisted.application.service.Application out "
+                "of a file and runs it.")
+
     optFlags = [['savestats', None,
                  "save the Stats object rather than the text output of "
                  "the profiler."],
                 ['no_save','o',   "do not save state on shutdown"],
                 ['encrypted', 'e',
-                 "The specified tap/aos/xml file is encrypted."],
-                ['nothotshot', None,
-                 "DEPRECATED. Don't use the hotshot profiler even if "
-                 "it's available."]]
+                 "The specified tap/aos file is encrypted."]]
 
     optParameters = [['logfile','l', None,
                       "log to a specified file, - for stdout"],
+                     ['logger', None, None,
+                      "A fully-qualified name to a log observer factory to use "
+                      "for the initial log observer.  Takes precedence over "
+                      "--logfile and --syslog (when available)."],
                      ['profile', 'p', None,
                       "Run in profile mode, dumping results to specified file"],
                      ['profiler', None, "hotshot",
@@ -603,37 +565,29 @@ class ServerOptions(usage.Options, ReactorSelectionMixin):
                      ['python','y', None,
                       "read an application from within a Python file "
                       "(implies -o)"],
-                     ['xml', 'x', None,
-                      "Read an application from a .tax file "
-                      "(Marmalade format)."],
                      ['source', 's', None,
                       "Read an application from a .tas file (AOT format)."],
                      ['rundir','d','.',
-                      'Change to a supplied directory before running'],
-                     ['report-profile', None, None,
-                      'E-mail address to use when reporting dynamic execution '
-                      'profiler stats.  This should not be combined with '
-                      'other profiling options.  This will only take effect '
-                      'if the application to be run has an application '
-                      'name.']]
+                      'Change to a supplied directory before running']]
 
-    #zsh_altArgDescr = {"foo":"use this description for foo instead"}
-    #zsh_multiUse = ["foo", "bar"]
-    zsh_mutuallyExclusive = [("file", "python", "xml", "source")]
-    zsh_actions = {"file":'_files -g "*.tap"',
-                   "python":'_files -g "*.(tac|py)"',
-                   "xml":'_files -g "*.tax"',
-                   "source":'_files -g "*.tas"',
-                   "rundir":"_dirs"}
-    #zsh_actionDescr = {"logfile":"log file name", "random":"random seed"}
+    compData = usage.Completions(
+        mutuallyExclusive=[("file", "python", "source")],
+        optActions={"file": usage.CompleteFiles("*.tap"),
+                    "python": usage.CompleteFiles("*.(tac|py)"),
+                    "source": usage.CompleteFiles("*.tas"),
+                    "rundir": usage.CompleteDirs()}
+        )
+
+    _getPlugins = staticmethod(plugin.getPlugins)
 
     def __init__(self, *a, **kw):
         self['debug'] = False
         usage.Options.__init__(self, *a, **kw)
 
+
     def opt_debug(self):
         """
-        run the application in the Python Debugger (implies nodaemon),
+        Run the application in the Python Debugger (implies nodaemon),
         sending SIGUSR2 will drop into debugger
         """
         defer.setDebugging(True)
@@ -643,7 +597,8 @@ class ServerOptions(usage.Options, ReactorSelectionMixin):
 
 
     def opt_spew(self):
-        """Print an insanely verbose log of everything that happens.
+        """
+        Print an insanely verbose log of everything that happens.
         Useful when debugging freezes or locks in complex code."""
         sys.settrace(util.spewer)
         try:
@@ -653,32 +608,35 @@ class ServerOptions(usage.Options, ReactorSelectionMixin):
         threading.settrace(util.spewer)
 
 
-    def opt_report_profile(self, value):
-        """
-        DEPRECATED.
-
-        Manage --report-profile option, which does nothing currently.
-        """
-        warnings.warn("--report-profile option is deprecated and a no-op "
-                      "since Twisted 8.0.", category=DeprecationWarning)
-
-
     def parseOptions(self, options=None):
         if options is None:
             options = sys.argv[1:] or ["--help"]
         usage.Options.parseOptions(self, options)
 
+
     def postOptions(self):
         if self.subCommand or self['python']:
             self['no_save'] = True
+        if self['logger'] is not None:
+            try:
+                self['logger'] = namedAny(self['logger'])
+            except Exception, e:
+                raise usage.UsageError("Logger '%s' could not be imported: %s" 
+                                       % (self['logger'], e))
+
 
     def subCommands(self):
-        from twisted import plugin
-        plugins = plugin.getPlugins(service.IServiceMaker)
+        plugins = self._getPlugins(service.IServiceMaker)
         self.loadedPlugins = {}
-        for plug in plugins:
+        for plug in sorted(plugins, key=attrgetter('tapname')):
             self.loadedPlugins[plug.tapname] = plug
-            yield (plug.tapname, None, lambda: plug.options(), plug.description)
+            yield (plug.tapname,
+                   None,
+                   # Avoid resolving the options attribute right away, in case
+                   # it's a property with a non-trivial getter (eg, one which
+                   # imports modules).
+                   lambda plug=plug: plug.options(),
+                   plug.description)
     subCommands = property(subCommands)
 
 
@@ -709,6 +667,8 @@ def convertStyle(filein, typein, passphrase, fileout, typeout, encrypt):
         fileout = None
     sob.IPersistable(application).save(filename=fileout, passphrase=passphrase)
 
+
+
 def startApplication(application, save):
     from twisted.internet import reactor
     service.IService(application).startService()
@@ -718,14 +678,3 @@ def startApplication(application, save):
     reactor.addSystemEventTrigger('before', 'shutdown',
                                   service.IService(application).stopService)
 
-def getLogFile(logfilename):
-    """
-    Build a log file from the full path.
-    """
-    warnings.warn(
-        "app.getLogFile is deprecated. Use "
-        "twisted.python.logfile.LogFile.fromFullPath instead",
-        DeprecationWarning, stacklevel=2)
-
-    return logfile.LogFile.fromFullPath(logfilename)
-
