@@ -1,23 +1,142 @@
 # -*- test-case-name: twisted.test.test_abstract -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
 """
 Support for generic select()able objects.
-
-Maintainer: Itamar Shtull-Trauring
 """
 
 from zope.interface import implements
 
 # Twisted Imports
-from twisted.python import log, reflect, failure
-from twisted.persisted import styles
+from twisted.python import reflect, failure
 from twisted.internet import interfaces, main
 
 
-class FileDescriptor(log.Logger, styles.Ephemeral, object):
+class _ConsumerMixin(object):
+    """
+    L{IConsumer} implementations can mix this in to get C{registerProducer} and
+    C{unregisterProducer} methods which take care of keeping track of a
+    producer's state.
+
+    Subclasses must provide three attributes which L{_ConsumerMixin} will read
+    but not write:
+
+      - connected: A C{bool} which is C{True} as long as the the consumer has
+        someplace to send bytes (for example, a TCP connection), and then
+        C{False} when it no longer does.
+
+      - disconnecting: A C{bool} which is C{False} until something like
+        L{ITransport.loseConnection} is called, indicating that the send buffer
+        should be flushed and the connection lost afterwards.  Afterwards,
+        C{True}.
+
+      - disconnected: A C{bool} which is C{False} until the consumer no longer
+        has a place to send bytes, then C{True}.
+
+    Subclasses must also override the C{startWriting} method.
+
+    @ivar producer: C{None} if no producer is registered, otherwise the
+        registered producer.
+
+    @ivar producerPaused: A flag indicating whether the producer is currently
+        paused.
+    @type producerPaused: C{bool} or C{int}
+
+    @ivar streamingProducer: A flag indicating whether the producer was
+        registered as a streaming (ie push) producer or not (ie a pull
+        producer).  This will determine whether the consumer may ever need to
+        pause and resume it, or if it can merely call C{resumeProducing} on it
+        when buffer space is available.
+    @ivar streamingProducer: C{bool} or C{int}
+
+    """
+    producer = None
+    producerPaused = False
+    streamingProducer = False
+
+    def startWriting(self):
+        """
+        Override in a subclass to cause the reactor to monitor this selectable
+        for write events.  This will be called once in C{unregisterProducer} if
+        C{loseConnection} has previously been called, so that the connection can
+        actually close.
+        """
+        raise NotImplementedError("%r did not implement startWriting")
+
+
+    def registerProducer(self, producer, streaming):
+        """
+        Register to receive data from a producer.
+
+        This sets this selectable to be a consumer for a producer.  When this
+        selectable runs out of data on a write() call, it will ask the producer
+        to resumeProducing(). When the FileDescriptor's internal data buffer is
+        filled, it will ask the producer to pauseProducing(). If the connection
+        is lost, FileDescriptor calls producer's stopProducing() method.
+
+        If streaming is true, the producer should provide the IPushProducer
+        interface. Otherwise, it is assumed that producer provides the
+        IPullProducer interface. In this case, the producer won't be asked to
+        pauseProducing(), but it has to be careful to write() data only when its
+        resumeProducing() method is called.
+        """
+        if self.producer is not None:
+            raise RuntimeError(
+                "Cannot register producer %s, because producer %s was never "
+                "unregistered." % (producer, self.producer))
+        if self.disconnected:
+            producer.stopProducing()
+        else:
+            self.producer = producer
+            self.streamingProducer = streaming
+            if not streaming:
+                producer.resumeProducing()
+
+
+    def unregisterProducer(self):
+        """
+        Stop consuming data from a producer, without disconnecting.
+        """
+        self.producer = None
+        if self.connected and self.disconnecting:
+            self.startWriting()
+
+
+
+class _LogOwner(object):
+    """
+    Mixin to help implement L{interfaces.ILoggingContext} for transports which
+    have a protocol, the log prefix of which should also appear in the
+    transport's log prefix.
+    """
+    implements(interfaces.ILoggingContext)
+
+    def _getLogPrefix(self, applicationObject):
+        """
+        Determine the log prefix to use for messages related to
+        C{applicationObject}, which may or may not be an
+        L{interfaces.ILoggingContext} provider.
+
+        @return: A C{str} giving the log prefix to use.
+        """
+        if interfaces.ILoggingContext.providedBy(applicationObject):
+            return applicationObject.logPrefix()
+        return applicationObject.__class__.__name__
+
+
+    def logPrefix(self):
+        """
+        Override this method to insert custom logging behavior.  Its
+        return value will be inserted in front of every line.  It may
+        be called more times than the number of output lines.
+        """
+        return "-"
+
+
+
+class FileDescriptor(_ConsumerMixin, _LogOwner):
     """An object which can be operated on by select().
 
     This is an abstract superclass of all objects which may be notified when
@@ -25,9 +144,6 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
     valid to be passed to select(2).
     """
     connected = 0
-    producerPaused = 0
-    streamingProducer = 0
-    producer = None
     disconnected = 0
     disconnecting = 0
     _writeDisconnecting = False
@@ -47,6 +163,7 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
         self._tempDataBuffer = [] # will be added to dataBuffer in doWrite
         self._tempDataLen = 0
 
+
     def connectionLost(self, reason):
         """The connection was lost.
 
@@ -57,7 +174,6 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
 
         Clean up state here, but make sure to call back up to FileDescriptor.
         """
-
         self.disconnected = 1
         self.connected = 0
         if self.producer is not None:
@@ -82,7 +198,8 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
 
 
     def doRead(self):
-        """Called when data is avaliable for reading.
+        """
+        Called when data is available for reading.
 
         Subclasses must override this method. The result will be interpreted
         in the same way as a result of doWrite().
@@ -207,6 +324,9 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
         streaming producer is registered, it will be paused until the buffered
         data is written to the underlying file descriptor.
         """
+        for i in iovec:
+            if isinstance(i, unicode): # no, really, I mean it
+                raise TypeError("Data must not be unicode")
         if not self.connected or not iovec or self._writeDisconnected:
             return
         self._tempDataBuffer.extend(iovec)
@@ -287,36 +407,6 @@ class FileDescriptor(log.Logger, styles.Ephemeral, object):
     producer = None
     bufferSize = 2**2**2**2
 
-    def registerProducer(self, producer, streaming):
-        """Register to receive data from a producer.
-
-        This sets this selectable to be a consumer for a producer.  When this
-        selectable runs out of data on a write() call, it will ask the producer
-        to resumeProducing(). When the FileDescriptor's internal data buffer is
-        filled, it will ask the producer to pauseProducing(). If the connection
-        is lost, FileDescriptor calls producer's stopProducing() method.
-
-        If streaming is true, the producer should provide the IPushProducer
-        interface. Otherwise, it is assumed that producer provides the
-        IPullProducer interface. In this case, the producer won't be asked
-        to pauseProducing(), but it has to be careful to write() data only
-        when its resumeProducing() method is called.
-        """
-        if self.producer is not None:
-            raise RuntimeError("Cannot register producer %s, because producer %s was never unregistered." % (producer, self.producer))
-        if self.disconnected:
-            producer.stopProducing()
-        else:
-            self.producer = producer
-            self.streamingProducer = streaming
-            if not streaming:
-                producer.resumeProducing()
-
-    def unregisterProducer(self):
-        """Stop consuming data from a producer, without disconnecting.
-        """
-        self.producer = None
-
     def stopConsuming(self):
         """Stop consuming data.
 
