@@ -1,14 +1,23 @@
 # -*- test-case-name: twisted.test.test_defer,twisted.test.test_defgen,twisted.internet.test.test_inlinecb -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Support for results that aren't immediately available.
 
 Maintainer: Glyph Lefkowitz
+
+@var _NO_RESULT: The result used to represent the fact that there is no
+    result. B{Never ever ever use this as an actual result for a Deferred}.  You
+    have been warned.
+
+@var _CONTINUE: A marker left in L{Deferred.callbacks} to indicate a Deferred
+    chain.  Always accompanied by a Deferred instance in the args tuple pointing
+    at the Deferred which is chained to the Deferred which has this marker.
 """
 
 import traceback
+import types
 import warnings
 from sys import exc_info
 
@@ -22,9 +31,17 @@ class AlreadyCalledError(Exception):
     pass
 
 
+class CancelledError(Exception):
+    """
+    This error is raised by default when a L{Deferred} is cancelled.
+    """
+
 
 class TimeoutError(Exception):
-    pass
+    """
+    This exception is deprecated.  It is used only by the deprecated
+    L{Deferred.setTimeout} method.
+    """
 
 
 
@@ -116,7 +133,7 @@ def maybeDeferred(f, *args, **kw):
     try:
         result = f(*args, **kw)
     except:
-        return fail(failure.Failure())
+        return fail(failure.Failure(captureVars=Deferred.debug))
 
     if isinstance(result, Deferred):
         return result
@@ -155,6 +172,11 @@ def getDebugging():
     return Deferred.debug
 
 
+# See module docstring.
+_NO_RESULT = object()
+_CONTINUE = object()
+
+
 
 class Deferred:
     """
@@ -169,14 +191,41 @@ class Deferred:
     that come from outside packages that are not under our control, we use
     threads (see for example L{twisted.enterprise.adbapi}).
 
-    For more information about Deferreds, see doc/howto/defer.html or
-    U{http://twistedmatrix.com/projects/core/documentation/howto/defer.html}
+    For more information about Deferreds, see doc/core/howto/defer.html or
+    U{http://twistedmatrix.com/documents/current/core/howto/defer.html}
+
+    When creating a Deferred, you may provide a canceller function, which
+    will be called by d.cancel() to let you do any clean-up necessary if the
+    user decides not to wait for the deferred to complete.
+
+    @ivar called: A flag which is C{False} until either C{callback} or
+        C{errback} is called and afterwards always C{True}.
+    @type called: C{bool}
+
+    @ivar paused: A counter of how many unmatched C{pause} calls have been made
+        on this instance.
+    @type paused: C{int}
+
+    @ivar _suppressAlreadyCalled: A flag used by the cancellation mechanism
+        which is C{True} if the Deferred has no canceller and has been
+        cancelled, C{False} otherwise.  If C{True}, it can be expected that
+        C{callback} or C{errback} will eventually be called and the result
+        should be silently discarded.
+    @type _suppressAlreadyCalled: C{bool}
+
+    @ivar _runningCallbacks: A flag which is C{True} while this instance is
+        executing its callback chain, used to stop recursive execution of
+        L{_runCallbacks}
+    @type _runningCallbacks: C{bool}
+
+    @ivar _chainedTo: If this Deferred is waiting for the result of another
+        Deferred, this is a reference to the other Deferred.  Otherwise, C{None}.
     """
 
-    called = 0
+    called = False
     paused = 0
-    timeoutCall = None
     _debugInfo = None
+    _suppressAlreadyCalled = False
 
     # Are we currently running a user-installed callback?  Meant to prevent
     # recursive running of callbacks when a reentrant call to add a callback is
@@ -187,9 +236,34 @@ class Deferred:
     # sets it directly.
     debug = False
 
+    _chainedTo = None
 
-    def __init__(self):
+    def __init__(self, canceller=None):
+        """
+        Initialize a L{Deferred}.
+
+        @param canceller: a callable used to stop the pending operation
+            scheduled by this L{Deferred} when L{Deferred.cancel} is
+            invoked. The canceller will be passed the deferred whose
+            cancelation is requested (i.e., self).
+
+            If a canceller is not given, or does not invoke its argument's
+            C{callback} or C{errback} method, L{Deferred.cancel} will
+            invoke L{Deferred.errback} with a L{CancelledError}.
+
+            Note that if a canceller is not given, C{callback} or
+            C{errback} may still be invoked exactly once, even though
+            defer.py will have already invoked C{errback}, as described
+            above.  This allows clients of code which returns a L{Deferred}
+            to cancel it without requiring the L{Deferred} instantiator to
+            provide any specific implementation support for cancellation.
+            New in 10.1.
+
+        @type canceller: a 1-argument callable which takes a L{Deferred}. The
+            return result is ignored.
+        """
         self.callbacks = []
+        self._canceller = canceller
         if self.debug:
             self._debugInfo = DebugInfo()
             self._debugInfo.creator = traceback.format_stack()[:-1]
@@ -262,7 +336,13 @@ class Deferred:
         chain of d1. Thus any event that fires d1 will also fire d2.
         However, the converse is B{not} true; if d2 is fired d1 will not be
         affected.
+
+        Note that unlike the case where chaining is caused by a L{Deferred}
+        being returned from a callback, it is possible to cause the call
+        stack size limit to be exceeded by chaining many L{Deferred}s
+        together with C{chainDeferred}.
         """
+        d._chainedTo = self
         return self.addCallbacks(d.callback, d.errback)
 
 
@@ -270,11 +350,13 @@ class Deferred:
         """
         Run all success callbacks that have been added to this L{Deferred}.
 
-        Each callback will have its result passed as the first
-        argument to the next; this way, the callbacks act as a
-        'processing chain'. Also, if the success-callback returns a L{Failure}
-        or raises an L{Exception}, processing will continue on the *error*-
-        callback chain.
+        Each callback will have its result passed as the first argument to
+        the next; this way, the callbacks act as a 'processing chain'.  If
+        the success-callback returns a L{Failure} or raises an L{Exception},
+        processing will continue on the *error* callback chain.  If a
+        callback (or errback) returns another L{Deferred}, this L{Deferred}
+        will be chained to it (and further callbacks will not run until that
+        L{Deferred} has a result).
         """
         assert not isinstance(result, Deferred)
         self._startRunCallbacks(result)
@@ -301,7 +383,9 @@ class Deferred:
         @raise NoCurrentExceptionError: If C{fail} is C{None} but there is
             no current exception state.
         """
-        if not isinstance(fail, failure.Failure):
+        if fail is None:
+            fail = failure.Failure(captureVars=self.debug)
+        elif not isinstance(fail, failure.Failure):
             fail = failure.Failure(fail)
 
         self._startRunCallbacks(fail)
@@ -325,13 +409,40 @@ class Deferred:
             self._runCallbacks()
 
 
-    def _continue(self, result):
-        self.result = result
-        self.unpause()
+    def cancel(self):
+        """
+        Cancel this L{Deferred}.
+
+        If the L{Deferred} has not yet had its C{errback} or C{callback} method
+        invoked, call the canceller function provided to the constructor. If
+        that function does not invoke C{callback} or C{errback}, or if no
+        canceller function was provided, errback with L{CancelledError}.
+
+        If this L{Deferred} is waiting on another L{Deferred}, forward the
+        cancellation to the other L{Deferred}.
+        """
+        if not self.called:
+            canceller = self._canceller
+            if canceller:
+                canceller(self)
+            else:
+                # Arrange to eat the callback that will eventually be fired
+                # since there was no real canceller.
+                self._suppressAlreadyCalled = True
+            if not self.called:
+                # There was no canceller, or the canceller didn't call
+                # callback or errback.
+                self.errback(failure.Failure(CancelledError()))
+        elif isinstance(self.result, Deferred):
+            # Waiting for another deferred -- cancel it instead.
+            self.result.cancel()
 
 
     def _startRunCallbacks(self, result):
         if self.called:
+            if self._suppressAlreadyCalled:
+                self._suppressAlreadyCalled = False
+                return
             if self.debug:
                 if self._debugInfo is None:
                     self._debugInfo = DebugInfo()
@@ -344,90 +455,160 @@ class Deferred:
             self._debugInfo.invoker = traceback.format_stack()[:-2]
         self.called = True
         self.result = result
-        if self.timeoutCall:
-            try:
-                self.timeoutCall.cancel()
-            except:
-                pass
-
-            del self.timeoutCall
         self._runCallbacks()
 
 
+    def _continuation(self):
+        """
+        Build a tuple of callback and errback with L{_continue} to be used by
+        L{_addContinue} and L{_removeContinue} on another Deferred.
+        """
+        return ((_CONTINUE, (self,), None),
+                (_CONTINUE, (self,), None))
+
+
     def _runCallbacks(self):
+        """
+        Run the chain of callbacks once a result is available.
+
+        This consists of a simple loop over all of the callbacks, calling each
+        with the current result and making the current result equal to the
+        return value (or raised exception) of that call.
+
+        If C{self._runningCallbacks} is true, this loop won't run at all, since
+        it is already running above us on the call stack.  If C{self.paused} is
+        true, the loop also won't run, because that's what it means to be
+        paused.
+
+        The loop will terminate before processing all of the callbacks if a
+        C{Deferred} without a result is encountered.
+
+        If a C{Deferred} I{with} a result is encountered, that result is taken
+        and the loop proceeds.
+
+        @note: The implementation is complicated slightly by the fact that
+            chaining (associating two Deferreds with each other such that one
+            will wait for the result of the other, as happens when a Deferred is
+            returned from a callback on another Deferred) is supported
+            iteratively rather than recursively, to avoid running out of stack
+            frames when processing long chains.
+        """
         if self._runningCallbacks:
             # Don't recursively run callbacks
             return
-        if not self.paused:
-            while self.callbacks:
-                item = self.callbacks.pop(0)
+
+        # Keep track of all the Deferreds encountered while propagating results
+        # up a chain.  The way a Deferred gets onto this stack is by having
+        # added its _continuation() to the callbacks list of a second Deferred
+        # and then that second Deferred being fired.  ie, if ever had _chainedTo
+        # set to something other than None, you might end up on this stack.
+        chain = [self]
+
+        while chain:
+            current = chain[-1]
+
+            if current.paused:
+                # This Deferred isn't going to produce a result at all.  All the
+                # Deferreds up the chain waiting on it will just have to...
+                # wait.
+                return
+
+            finished = True
+            current._chainedTo = None
+            while current.callbacks:
+                item = current.callbacks.pop(0)
                 callback, args, kw = item[
-                    isinstance(self.result, failure.Failure)]
+                    isinstance(current.result, failure.Failure)]
                 args = args or ()
                 kw = kw or {}
+
+                # Avoid recursion if we can.
+                if callback is _CONTINUE:
+                    # Give the waiting Deferred our current result and then
+                    # forget about that result ourselves.
+                    chainee = args[0]
+                    chainee.result = current.result
+                    current.result = None
+                    # Making sure to update _debugInfo
+                    if current._debugInfo is not None:
+                        current._debugInfo.failResult = None
+                    chainee.paused -= 1
+                    chain.append(chainee)
+                    # Delay cleaning this Deferred and popping it from the chain
+                    # until after we've dealt with chainee.
+                    finished = False
+                    break
+
                 try:
-                    self._runningCallbacks = True
+                    current._runningCallbacks = True
                     try:
-                        self.result = callback(self.result, *args, **kw)
+                        current.result = callback(current.result, *args, **kw)
                     finally:
-                        self._runningCallbacks = False
-                    if isinstance(self.result, Deferred):
-                        # note: this will cause _runCallbacks to be called
-                        # recursively if self.result already has a result.
-                        # This shouldn't cause any problems, since there is no
-                        # relevant state in this stack frame at this point.
-                        # The recursive call will continue to process
-                        # self.callbacks until it is empty, then return here,
-                        # where there is no more work to be done, so this call
-                        # will return as well.
-                        self.pause()
-                        self.result.addBoth(self._continue)
-                        break
+                        current._runningCallbacks = False
                 except:
-                    self.result = failure.Failure()
+                    # Including full frame information in the Failure is quite
+                    # expensive, so we avoid it unless self.debug is set.
+                    current.result = failure.Failure(captureVars=self.debug)
+                else:
+                    if isinstance(current.result, Deferred):
+                        # The result is another Deferred.  If it has a result,
+                        # we can take it and keep going.
+                        resultResult = getattr(current.result, 'result', _NO_RESULT)
+                        if resultResult is _NO_RESULT or isinstance(resultResult, Deferred) or current.result.paused:
+                            # Nope, it didn't.  Pause and chain.
+                            current.pause()
+                            current._chainedTo = current.result
+                            # Note: current.result has no result, so it's not
+                            # running its callbacks right now.  Therefore we can
+                            # append to the callbacks list directly instead of
+                            # using addCallbacks.
+                            current.result.callbacks.append(current._continuation())
+                            break
+                        else:
+                            # Yep, it did.  Steal it.
+                            current.result.result = None
+                            # Make sure _debugInfo's failure state is updated.
+                            if current.result._debugInfo is not None:
+                                current.result._debugInfo.failResult = None
+                            current.result = resultResult
 
-        if isinstance(self.result, failure.Failure):
-            self.result.cleanFailure()
-            if self._debugInfo is None:
-                self._debugInfo = DebugInfo()
-            self._debugInfo.failResult = self.result
-        else:
-            if self._debugInfo is not None:
-                self._debugInfo.failResult = None
+            if finished:
+                # As much of the callback chain - perhaps all of it - as can be
+                # processed right now has been.  The current Deferred is waiting on
+                # another Deferred or for more callbacks.  Before finishing with it,
+                # make sure its _debugInfo is in the proper state.
+                if isinstance(current.result, failure.Failure):
+                    # Stash the Failure in the _debugInfo for unhandled error
+                    # reporting.
+                    current.result.cleanFailure()
+                    if current._debugInfo is None:
+                        current._debugInfo = DebugInfo()
+                    current._debugInfo.failResult = current.result
+                else:
+                    # Clear out any Failure in the _debugInfo, since the result
+                    # is no longer a Failure.
+                    if current._debugInfo is not None:
+                        current._debugInfo.failResult = None
 
+                # This Deferred is done, pop it from the chain and move back up
+                # to the Deferred which supplied us with our result.
+                chain.pop()
 
-    def setTimeout(self, seconds, timeoutFunc=timeout, *args, **kw):
-        """
-        Set a timeout function to be triggered if I am not called.
-
-        @param seconds: How long to wait (from now) before firing the
-        C{timeoutFunc}.
-
-        @param timeoutFunc: will receive the L{Deferred} and *args, **kw as its
-        arguments.  The default C{timeoutFunc} will call the errback with a
-        L{TimeoutError}.
-        """
-        warnings.warn(
-            "Deferred.setTimeout is deprecated.  Look for timeout "
-            "support specific to the API you are using instead.",
-            DeprecationWarning, stacklevel=2)
-
-        if self.called:
-            return
-        assert not self.timeoutCall, "Don't call setTimeout twice on the same Deferred."
-
-        from twisted.internet import reactor
-        self.timeoutCall = reactor.callLater(
-            seconds,
-            lambda: self.called or timeoutFunc(self, *args, **kw))
-        return self.timeoutCall
 
     def __str__(self):
+        """
+        Return a string representation of this C{Deferred}.
+        """
         cname = self.__class__.__name__
-        if hasattr(self, 'result'):
-            return "<%s at %s  current result: %r>" % (cname, hex(unsignedID(self)),
-                                                       self.result)
-        return "<%s at %s>" % (cname, hex(unsignedID(self)))
+        result = getattr(self, 'result', _NO_RESULT)
+        myID = hex(unsignedID(self))
+        if self._chainedTo is not None:
+            result = ' waiting on Deferred at %s' % (hex(unsignedID(self._chainedTo)),)
+        elif result is _NO_RESULT:
+            result = ''
+        else:
+            result = ' current result: %r' % (result,)
+        return "<%s at %s%s>" % (cname, myID, result)
     __repr__ = __str__
 
 
@@ -439,7 +620,6 @@ class DebugInfo:
 
     failResult = None
 
-
     def _getDebugTracebacks(self):
         info = ''
         if hasattr(self, "creator"):
@@ -522,40 +702,62 @@ class FirstError(Exception):
 
 class DeferredList(Deferred):
     """
-    I combine a group of deferreds into one callback.
+    L{DeferredList} is a tool for collecting the results of several Deferreds.
 
-    I track a list of L{Deferred}s for their callbacks, and make a single
-    callback when they have all completed, a list of (success, result)
-    tuples, 'success' being a boolean.
+    This tracks a list of L{Deferred}s for their results, and makes a single
+    callback when they have all completed.  By default, the ultimate result is a
+    list of (success, result) tuples, 'success' being a boolean.
+    L{DeferredList} exposes the same API that L{Deferred} does, so callbacks and
+    errbacks can be added to it in the same way.
 
-    Note that you can still use a L{Deferred} after putting it in a
-    DeferredList.  For example, you can suppress 'Unhandled error in Deferred'
-    messages by adding errbacks to the Deferreds *after* putting them in the
-    DeferredList, as a DeferredList won't swallow the errors.  (Although a more
-    convenient way to do this is simply to set the consumeErrors flag)
+    L{DeferredList} is implemented by adding callbacks and errbacks to each
+    L{Deferred} in the list passed to it.  This means callbacks and errbacks
+    added to the Deferreds before they are passed to L{DeferredList} will change
+    the result that L{DeferredList} sees (i.e., L{DeferredList} is not special).
+    Callbacks and errbacks can also be added to the Deferreds after they are
+    passed to L{DeferredList} and L{DeferredList} may change the result that
+    they see.
+
+    See the documentation for the C{__init__} arguments for more information.
     """
 
-    fireOnOneCallback = 0
-    fireOnOneErrback = 0
+    fireOnOneCallback = False
+    fireOnOneErrback = False
 
-
-    def __init__(self, deferredList, fireOnOneCallback=0, fireOnOneErrback=0,
-                 consumeErrors=0):
+    def __init__(self, deferredList, fireOnOneCallback=False,
+                 fireOnOneErrback=False, consumeErrors=False):
         """
         Initialize a DeferredList.
 
-        @type deferredList:  C{list} of L{Deferred}s
         @param deferredList: The list of deferreds to track.
-        @param fireOnOneCallback: (keyword param) a flag indicating that
-                             only one callback needs to be fired for me to call
-                             my callback
-        @param fireOnOneErrback: (keyword param) a flag indicating that
-                            only one errback needs to be fired for me to call
-                            my errback
-        @param consumeErrors: (keyword param) a flag indicating that any errors
-                            raised in the original deferreds should be
-                            consumed by this DeferredList.  This is useful to
-                            prevent spurious warnings being logged.
+        @type deferredList:  C{list} of L{Deferred}s
+
+        @param fireOnOneCallback: (keyword param) a flag indicating that this
+            L{DeferredList} will fire when the first L{Deferred} in
+            C{deferredList} fires with a non-failure result without waiting for
+            any of the other Deferreds.  When this flag is set, the DeferredList
+            will fire with a two-tuple: the first element is the result of the
+            Deferred which fired; the second element is the index in
+            C{deferredList} of that Deferred.
+        @type fireOnOneCallback: C{bool}
+
+        @param fireOnOneErrback: (keyword param) a flag indicating that this
+            L{DeferredList} will fire when the first L{Deferred} in
+            C{deferredList} fires with a failure result without waiting for any
+            of the other Deferreds.  When this flag is set, if a Deferred in the
+            list errbacks, the DeferredList will errback with a L{FirstError}
+            failure wrapping the failure of that Deferred.
+        @type fireOnOneErrback: C{bool}
+
+        @param consumeErrors: (keyword param) a flag indicating that failures in
+            any of the included L{Deferreds} should not be propagated to
+            errbacks added to the individual L{Deferreds} after this
+            L{DeferredList} is constructed.  After constructing the
+            L{DeferredList}, any errors in the individual L{Deferred}s will be
+            converted to a callback result of C{None}.  This is useful to
+            prevent spurious 'Unhandled error in Deferred' messages from being
+            logged.  This does not prevent C{fireOnOneErrback} from working.
+        @type consumeErrors: C{bool}
         """
         self.resultList = [None] * len(deferredList)
         Deferred.__init__(self)
@@ -600,7 +802,7 @@ class DeferredList(Deferred):
 
 
 
-def _parseDListResult(l, fireOnOneErrback=0):
+def _parseDListResult(l, fireOnOneErrback=False):
     if __debug__:
         for success, value in l:
             assert success
@@ -608,16 +810,30 @@ def _parseDListResult(l, fireOnOneErrback=0):
 
 
 
-def gatherResults(deferredList):
+def gatherResults(deferredList, consumeErrors=False):
     """
-    Returns list with result of given L{Deferred}s.
+    Returns, via a L{Deferred}, a list with the results of the given
+    L{Deferred}s - in effect, a "join" of multiple deferred operations.
 
-    This builds on L{DeferredList} but is useful since you don't
-    need to parse the result for success/failure.
+    The returned L{Deferred} will fire when I{all} of the provided L{Deferred}s
+    have fired, or when any one of them has failed.
+
+    This differs from L{DeferredList} in that you don't need to parse
+    the result for success/failure.
 
     @type deferredList:  C{list} of L{Deferred}s
+
+    @param consumeErrors: (keyword param) a flag, defaulting to False,
+        indicating that failures in any of the given L{Deferreds} should not be
+        propagated to errbacks added to the individual L{Deferreds} after this
+        L{gatherResults} invocation.  Any such errors in the individual
+        L{Deferred}s will be converted to a callback result of C{None}.  This
+        is useful to prevent spurious 'Unhandled error in Deferred' messages
+        from being logged.  This parameter is available since 11.1.0.
+    @type consumeErrors: C{bool}
     """
-    d = DeferredList(deferredList, fireOnOneErrback=1)
+    d = DeferredList(deferredList, fireOnOneErrback=True,
+                                   consumeErrors=consumeErrors)
     d.addCallback(_parseDListResult)
     return d
 
@@ -716,12 +932,12 @@ def deferredGenerator(f):
     There are two important functions involved: L{waitForDeferred}, and
     L{deferredGenerator}.  They are used together, like this::
 
+        @deferredGenerator
         def thingummy():
             thing = waitForDeferred(makeSomeRequestResultingInDeferred())
             yield thing
             thing = thing.getResult()
             print thing #the result! hoorj!
-        thingummy = deferredGenerator(thingummy)
 
     L{waitForDeferred} returns something that you should immediately yield; when
     your generator is resumed, calling C{thing.getResult()} will either give you
@@ -744,6 +960,7 @@ def deferredGenerator(f):
     The L{Deferred} returned from your deferred generator may also errback if your
     generator raised an exception.  For example::
 
+        @deferredGenerator
         def thingummy():
             thing = waitForDeferred(makeSomeRequestResultingInDeferred())
             yield thing
@@ -755,7 +972,6 @@ def deferredGenerator(f):
             else:
                 # will trigger an errback
                 raise Exception('DESTROY ALL LIFE')
-        thingummy = deferredGenerator(thingummy)
 
     Put succinctly, these functions connect deferred-using code with this 'fake
     blocking' style in both directions: L{waitForDeferred} converts from a
@@ -910,16 +1126,20 @@ def inlineCallbacks(f):
     the L{deferredGenerator} function instead, which accomplishes the same
     thing, but with somewhat more boilerplate.  For example::
 
+        @inlineCallBacks
         def thingummy():
             thing = yield makeSomeRequestResultingInDeferred()
             print thing #the result! hoorj!
-        thingummy = inlineCallbacks(thingummy)
 
     When you call anything that results in a L{Deferred}, you can simply yield it;
     your generator will automatically be resumed when the Deferred's result is
     available. The generator will be sent the result of the L{Deferred} with the
     'send' method on generators, or if the result was a failure, 'throw'.
 
+    Things that are not L{Deferred}s may also be yielded, and your generator
+    will be resumed with the same object sent back. This means C{yield}
+    performs an operation roughly equivalent to L{maybeDeferred}.
+
     Your inlineCallbacks-enabled generator will return a L{Deferred} object, which
     will result in the return value of the generator (or will fail with a
     failure object if your generator raises an unhandled exception). Note that
@@ -927,9 +1147,17 @@ def inlineCallbacks(f):
     instead. Falling off the end of the generator, or simply using C{return}
     will cause the L{Deferred} to have a result of C{None}.
 
+    Be aware that L{returnValue} will not accept a L{Deferred} as a parameter.
+    If you believe the thing you'd like to return could be a L{Deferred}, do
+    this::
+
+        result = yield result
+        returnValue(result)
+
     The L{Deferred} returned from your deferred generator may errback if your
     generator raised an exception::
 
+        @inlineCallbacks
         def thingummy():
             thing = yield makeSomeRequestResultingInDeferred()
             if thing == 'I love Twisted':
@@ -938,10 +1166,19 @@ def inlineCallbacks(f):
             else:
                 # will trigger an errback
                 raise Exception('DESTROY ALL LIFE')
-        thingummy = inlineCallbacks(thingummy)
     """
     def unwindGenerator(*args, **kwargs):
-        return _inlineCallbacks(None, f(*args, **kwargs), Deferred())
+        try:
+            gen = f(*args, **kwargs)
+        except _DefGen_Return:
+            raise TypeError(
+                "inlineCallbacks requires %r to produce a generator; instead"
+                "caught returnValue being used in a non-generator" % (f,))
+        if not isinstance(gen, types.GeneratorType):
+            raise TypeError(
+                "inlineCallbacks requires %r to produce a generator; "
+                "instead got %r" % (f, gen))
+        return _inlineCallbacks(None, gen, Deferred())
     return mergeFunctionMetadata(f, unwindGenerator)
 
 
@@ -994,12 +1231,27 @@ class DeferredLock(_ConcurrencyPrimitive):
     """
     A lock for event driven systems.
 
-    @ivar locked: C{True} when this Lock has been acquired, false at all
-    other times.  Do not change this value, but it is useful to
-    examine for the equivalent of a "non-blocking" acquisition.
+    @ivar locked: C{True} when this Lock has been acquired, false at all other
+        times.  Do not change this value, but it is useful to examine for the
+        equivalent of a "non-blocking" acquisition.
     """
 
-    locked = 0
+    locked = False
+
+
+    def _cancelAcquire(self, d):
+        """
+        Remove a deferred d from our waiting list, as the deferred has been
+        canceled.
+
+        Note: We do not need to wrap this in a try/except to catch d not
+        being in self.waiting because this canceller will not be called if
+        d has fired. release() pops a deferred out of self.waiting and
+        calls it, so the canceller will no longer be called.
+
+        @param d: The deferred that has been canceled.
+        """
+        self.waiting.remove(d)
 
 
     def acquire(self):
@@ -1011,11 +1263,11 @@ class DeferredLock(_ConcurrencyPrimitive):
         @return: a L{Deferred} which fires on lock acquisition.
         @rtype: a L{Deferred}
         """
-        d = Deferred()
+        d = Deferred(canceller=self._cancelAcquire)
         if self.locked:
             self.waiting.append(d)
         else:
-            self.locked = 1
+            self.locked = True
             d.callback(self)
         return d
 
@@ -1029,10 +1281,10 @@ class DeferredLock(_ConcurrencyPrimitive):
         resource is free.
         """
         assert self.locked, "Tried to release an unlocked lock"
-        self.locked = 0
+        self.locked = False
         if self.waiting:
             # someone is waiting to acquire lock
-            self.locked = 1
+            self.locked = True
             d = self.waiting.pop(0)
             d.callback(self)
 
@@ -1041,15 +1293,39 @@ class DeferredLock(_ConcurrencyPrimitive):
 class DeferredSemaphore(_ConcurrencyPrimitive):
     """
     A semaphore for event driven systems.
-    """
 
+    @ivar tokens: At most this many users may acquire this semaphore at
+        once.
+    @type tokens: C{int}
+
+    @ivar limit: The difference between C{tokens} and the number of users
+        which have currently acquired this semaphore.
+    @type limit: C{int}
+    """
 
     def __init__(self, tokens):
         _ConcurrencyPrimitive.__init__(self)
+        if tokens < 1:
+            raise ValueError("DeferredSemaphore requires tokens >= 1")
         self.tokens = tokens
         self.limit = tokens
 
 
+    def _cancelAcquire(self, d):
+        """
+        Remove a deferred d from our waiting list, as the deferred has been
+        canceled.
+
+        Note: We do not need to wrap this in a try/except to catch d not
+        being in self.waiting because this canceller will not be called if
+        d has fired. release() pops a deferred out of self.waiting and
+        calls it, so the canceller will no longer be called.
+
+        @param d: The deferred that has been canceled.
+        """
+        self.waiting.remove(d)
+
+
     def acquire(self):
         """
         Attempt to acquire the token.
@@ -1057,7 +1333,7 @@ class DeferredSemaphore(_ConcurrencyPrimitive):
         @return: a L{Deferred} which fires on token acquisition.
         """
         assert self.tokens >= 0, "Internal inconsistency??  tokens should never be negative"
-        d = Deferred()
+        d = Deferred(canceller=self._cancelAcquire)
         if not self.tokens:
             self.waiting.append(d)
         else:
@@ -1118,6 +1394,21 @@ class DeferredQueue(object):
         self.backlog = backlog
 
 
+    def _cancelGet(self, d):
+        """
+        Remove a deferred d from our waiting list, as the deferred has been
+        canceled.
+
+        Note: We do not need to wrap this in a try/except to catch d not
+        being in self.waiting because this canceller will not be called if
+        d has fired. put() pops a deferred out of self.waiting and calls
+        it, so the canceller will no longer be called.
+
+        @param d: The deferred that has been canceled.
+        """
+        self.waiting.remove(d)
+
+
     def put(self, obj):
         """
         Add an object to this queue.
@@ -1145,7 +1436,7 @@ class DeferredQueue(object):
         if self.pending:
             return succeed(self.pending.pop(0))
         elif self.backlog is None or len(self.waiting) < self.backlog:
-            d = Deferred()
+            d = Deferred(canceller=self._cancelGet)
             self.waiting.append(d)
             return d
         else:
