@@ -1,5 +1,5 @@
-# -*- test-case-name: twisted.internet.test.test_gtk2reactor -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.internet.test -*-
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -21,8 +21,10 @@ integration.
 """
 
 # System Imports
-import sys
+import sys, signal
+
 from zope.interface import implements
+
 try:
     if not hasattr(sys, 'frozen'):
         # Don't want to check this for py2exe
@@ -38,10 +40,10 @@ if hasattr(gobject, "threads_init"):
     gobject.threads_init()
 
 # Twisted Imports
-from twisted.python import log, runtime, failure
+from twisted.python import log, runtime
 from twisted.python.compat import set
 from twisted.internet.interfaces import IReactorFDSet
-from twisted.internet import main, posixbase, error, selectreactor
+from twisted.internet import base, posixbase, selectreactor
 
 POLL_DISCONNECTED = gobject.IO_HUP | gobject.IO_ERR | gobject.IO_NVAL
 
@@ -65,7 +67,28 @@ def _our_mainquit():
 
 
 
-class Gtk2Reactor(posixbase.PosixReactorBase):
+class _Gtk2SignalMixin(object):
+    if runtime.platformType == 'posix':
+        def _handleSignals(self):
+            # Let the base class do its thing, but pygtk is probably
+            # going to stomp on us so go beyond that and set up some
+            # signal handling which pygtk won't mess with.  This would
+            # be better done by letting this reactor select a
+            # different implementation of installHandler for
+            # _SIGCHLDWaker to use.  Then, at least, we could fall
+            # back to our extension module.  See #4286.
+            from twisted.internet.process import reapAllProcesses as _reapAllProcesses
+            base._SignalReactorMixin._handleSignals(self)
+            signal.signal(signal.SIGCHLD, lambda *a: self.callFromThread(_reapAllProcesses))
+            if getattr(signal, "siginterrupt", None) is not None:
+                signal.siginterrupt(signal.SIGCHLD, False)
+            # Like the base, reap processes now in case a process
+            # exited before the handlers above were installed.
+            _reapAllProcesses()
+
+
+
+class Gtk2Reactor(_Gtk2SignalMixin, posixbase.PosixReactorBase, posixbase._PollLikeMixin):
     """
     GTK+-2 event loop reactor.
 
@@ -82,6 +105,10 @@ class Gtk2Reactor(posixbase.PosixReactorBase):
     """
     implements(IReactorFDSet)
 
+    _POLL_DISCONNECTED = POLL_DISCONNECTED
+    _POLL_IN = gobject.IO_IN
+    _POLL_OUT = gobject.IO_OUT
+
     def __init__(self, useGtk=True):
         self._simtag = None
         self._reads = set()
@@ -104,6 +131,7 @@ class Gtk2Reactor(posixbase.PosixReactorBase):
             self.__crash = _our_mainquit
             self.__run = gtk.main
 
+
     # The input_add function in pygtk1 checks for objects with a
     # 'fileno' method and, if present, uses the result of that method
     # as the input source. The pygtk2 input_add does not do this. The
@@ -249,37 +277,9 @@ class Gtk2Reactor(posixbase.PosixReactorBase):
             self.__run()
 
 
-    def _doReadOrWrite(self, source, condition, faildict={
-        error.ConnectionDone: failure.Failure(error.ConnectionDone()),
-        error.ConnectionLost: failure.Failure(error.ConnectionLost()),
-        }):
-        why = None
-        didRead = None
-        if condition & POLL_DISCONNECTED and \
-               not (condition & gobject.IO_IN):
-            why = main.CONNECTION_LOST
-        else:
-            try:
-                if condition & gobject.IO_IN:
-                    why = source.doRead()
-                    didRead = source.doRead
-                if not why and condition & gobject.IO_OUT:
-                    # if doRead caused connectionLost, don't call doWrite
-                    # if doRead is doWrite, don't call it again.
-                    if not source.disconnected and source.doWrite != didRead:
-                        why = source.doWrite()
-                        didRead = source.doWrite # if failed it was in write
-            except:
-                why = sys.exc_info()[1]
-                log.msg('Error In %s' % source)
-                log.deferr()
-
-        if why:
-            self._disconnectSelectable(source, why, didRead == source.doRead)
-
-
     def callback(self, source, condition):
-        log.callWithLogger(source, self._doReadOrWrite, source, condition)
+        log.callWithLogger(
+            source, self._doReadOrWrite, source, source, condition)
         self.simulate() # fire Twisted timers
         return 1 # 1=don't auto-remove the source
 
@@ -299,7 +299,7 @@ class Gtk2Reactor(posixbase.PosixReactorBase):
 
 
 
-class PortableGtkReactor(selectreactor.SelectReactor):
+class PortableGtkReactor(_Gtk2SignalMixin, selectreactor.SelectReactor):
     """
     Reactor that works on Windows.
 
@@ -323,10 +323,11 @@ class PortableGtkReactor(selectreactor.SelectReactor):
         self.startRunning(installSignalHandlers=installSignalHandlers)
         gobject.timeout_add(0, self.simulate)
         # mainloop is deprecated in newer versions
-        if hasattr(gtk, 'main'):
-            gtk.main()
-        else:
-            gtk.mainloop()
+        if self._started:
+            if hasattr(gtk, 'main'):
+                gtk.main()
+            else:
+                gtk.mainloop()
 
 
     def simulate(self):
