@@ -986,6 +986,7 @@ class ILoggingContext(Interface):
         """
 
 
+
 class IFileDescriptor(ILoggingContext):
     """
     An interface representing a UNIX-style numeric file descriptor.
@@ -993,10 +994,17 @@ class IFileDescriptor(ILoggingContext):
 
     def fileno():
         """
+        @raise: If the descriptor no longer has a valid file descriptor
+            number associated with it.
+
         @return: The platform-specified representation of a file descriptor
-                 number.
+            number.  Or C{-1} if the descriptor no longer has a valid file
+            descriptor number associated with it.  As long as the descriptor
+            is valid, calls to this method on a particular instance must
+            return the same value.
         """
 
+
     def connectionLost(reason):
         """
         Called when the connection was lost.
@@ -1016,6 +1024,7 @@ class IFileDescriptor(ILoggingContext):
         """
 
 
+
 class IReadDescriptor(IFileDescriptor):
     """
     An L{IFileDescriptor} that can read.
@@ -1026,6 +1035,10 @@ class IReadDescriptor(IFileDescriptor):
     def doRead():
         """
         Some data is available for reading on your descriptor.
+
+        @return: If an error is encountered which causes the descriptor to
+            no longer be valid, a L{Failure} should be returned.  Otherwise,
+            C{None}.
         """
 
 
@@ -1039,6 +1052,10 @@ class IWriteDescriptor(IFileDescriptor):
     def doWrite():
         """
         Some data can be written to your descriptor.
+
+        @return: If an error is encountered which causes the descriptor to
+            no longer be valid, a L{Failure} should be returned.  Otherwise,
+            C{None}.
         """
 
 
@@ -1133,16 +1150,25 @@ class IConsumer(Interface):
 
 
 
+deprecatedModuleAttribute(Version("Twisted", 11, 1, 0),
+    "Please use IConsumer (and IConsumer.unregisterProducer) instead.",
+    __name__, "IFinishableConsumer")
+
 class IFinishableConsumer(IConsumer):
     """
-    A Consumer for producers that finish.
+    A Consumer for producers that finish.  This interface offers no advantages
+    over L{IConsumer} and is deprecated.  Please use
+    L{IConsumer.unregisterProducer} instead of L{IFinishableConsumer.finish}.
     """
 
     def finish():
         """
-        The producer has finished producing.
+        The producer has finished producing.  This method is deprecated.
+        Please use L{IConsumer.unregisterProducer} instead.
         """
 
+
+
 class IProducer(Interface):
     """
     A producer produces data for a consumer.
@@ -1455,6 +1481,19 @@ class ITCPTransport(ITransport):
         producer.
         """
 
+
+    def abortConnection():
+        """
+        Close the connection abruptly.
+
+        Discards any buffered data, stops any registered producer,
+        and, if possible, notifies the other end of the unclean
+        closure.
+
+        @since: 11.1
+        """
+
+
     def getTcpNoDelay():
         """
         Return if C{TCP_NODELAY} is enabled.
@@ -1555,8 +1594,6 @@ class IProcessTransport(ITransport):
         Similar to L{ITransport.write} but also allows the file descriptor in
         the child process which will receive the bytes to be specified.
 
-        This is not available on all platforms.
-
         @type childFD: C{int}
         @param childFD: The file descriptor to which to write.
 
@@ -1564,6 +1601,10 @@ class IProcessTransport(ITransport):
         @param data: The bytes to write.
 
         @return: C{None}
+
+        @raise KeyError: If C{childFD} is not a file descriptor that was mapped
+            in the child when L{IReactorProcess.spawnProcess} was used to create
+            it.
         """
 
     def loseConnection():
@@ -1838,14 +1879,12 @@ class IStreamClientEndpointStringParser(Interface):
     )
 
 
-    def parseStreamClient(reactor, *args, **kwargs):
+    def parseStreamClient(*args, **kwargs):
         """
         This method is invoked by L{endpoints.clientFromString}, if the type of
         endpoint matches the return value from this
         L{IStreamClientEndpointStringParser}'s C{prefix} method.
 
-        @param reactor: The reactor passed to C{clientFromString}.
-
         @param args: The string arguments, minus the endpoint type, in the
             endpoint description string, parsed according to the rules
             described in L{endpoints.quoteStringArgument}.  For example, if the
