@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -7,12 +7,12 @@ TCP support for IOCP reactor
 
 import socket, operator, errno, struct
 
-from zope.interface import implements, directlyProvides
+from zope.interface import implements, classImplements
 
 from twisted.internet import interfaces, error, address, main, defer
-from twisted.internet.abstract import isIPAddress
+from twisted.internet.abstract import _LogOwner, isIPAddress
 from twisted.internet.tcp import _SocketCloser, Connector as TCPConnector
-from twisted.persisted import styles
+from twisted.internet.tcp import _AbortingMixin
 from twisted.python import log, failure, reflect, util
 
 from twisted.internet.iocpreactor import iocpsupport as _iocp, abstract
@@ -24,12 +24,9 @@ from twisted.internet.iocpreactor.const import ERROR_CONNECTION_REFUSED
 from twisted.internet.iocpreactor.const import ERROR_NETWORK_UNREACHABLE
 
 try:
-    from twisted.protocols.tls import TLSMemoryBIOFactory, TLSMemoryBIOProtocol
+    from twisted.internet._newtls import startTLS as _startTLS
 except ImportError:
-    TLSMemoryBIOProtocol = TLSMemoryBIOFactory = None
-    _extraInterfaces = ()
-else:
-    _extraInterfaces = (interfaces.ITLSTransport,)
+    _startTLS = None
 
 # ConnectEx returns these. XXX: find out what it does for timeout
 connectExErrors = {
@@ -37,55 +34,17 @@ connectExErrors = {
         ERROR_NETWORK_UNREACHABLE: errno.WSAENETUNREACH,
         }
 
-
-class _BypassTLS(object):
+class Connection(abstract.FileHandle, _SocketCloser, _AbortingMixin):
     """
-    L{_BypassTLS} is used as the transport object for the TLS protocol object
-    used to implement C{startTLS}.  Its methods skip any TLS logic which
-    C{startTLS} enables.
-
-    @ivar _connection: A L{Connection} which TLS has been started on which will
-        be proxied to by this object.  Any method which has its behavior
-        altered after C{startTLS} will be skipped in favor of the base class's
-        implementation.  This allows the TLS protocol object to have direct
-        access to the transport, necessary to actually implement TLS.
-    """
-    def __init__(self, connection):
-        self._connection = connection
-
-
-    def __getattr__(self, name):
-        return getattr(self._connection, name)
-
-
-    def write(self, data):
-        return abstract.FileHandle.write(self._connection, data)
-
-
-    def writeSequence(self, iovec):
-        return abstract.FileHandle.writeSequence(self._connection, iovec)
-
-
-    def loseConnection(self, reason=None):
-        return abstract.FileHandle.loseConnection(self._connection, reason)
-
-
-
-class Connection(abstract.FileHandle, _SocketCloser):
-    """
-    @ivar _tls: C{False} to indicate the connection is in normal TCP mode,
+    @ivar TLS: C{False} to indicate the connection is in normal TCP mode,
         C{True} to indicate that TLS has been started and that operations must
         be routed through the L{TLSMemoryBIOProtocol} instance.
-
-    @ivar _tlsClientDefault: A flag which must be set by a subclass.  If set to
-        C{True}, L{startTLS} will default to initiating SSL as a client.  If
-        set to C{False}, L{startTLS} will default to initiating SSL as a
-        server.
     """
     implements(IReadWriteHandle, interfaces.ITCPTransport,
-               interfaces.ISystemHandle, *_extraInterfaces)
+               interfaces.ISystemHandle)
+
+    TLS = False
 
-    _tls = False
 
     def __init__(self, sock, proto, reactor=None):
         abstract.FileHandle.__init__(self, reactor)
@@ -108,7 +67,12 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
 
     def writeToHandle(self, buff, evt):
-        return _iocp.send(self.getFileHandle(), buff, evt)
+        """
+        Send C{buff} to current file handle using C{_iocp.send}. The buffer
+        sent is limited to a size of C{self.SEND_LIMIT}.
+        """
+        return _iocp.send(self.getFileHandle(),
+            buffer(buff, 0, self.SEND_LIMIT), evt)
 
 
     def _closeWriteConnection(self):
@@ -139,8 +103,12 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
 
     def connectionLost(self, reason):
+        if self.disconnected:
+            return
         abstract.FileHandle.connectionLost(self, reason)
-        self._closeSocket()
+        isClean = (reason is None or
+                   not reason.check(error.ConnectionAborted))
+        self._closeSocket(isClean)
         protocol = self.protocol
         del self.protocol
         del self.socket
@@ -173,36 +141,12 @@ class Connection(abstract.FileHandle, _SocketCloser):
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, enabled)
 
 
-    if TLSMemoryBIOFactory is not None:
+    if _startTLS is not None:
         def startTLS(self, contextFactory, normal=True):
             """
             @see: L{ITLSTransport.startTLS}
             """
-            # Figure out which direction the SSL goes in.  If normal is True,
-            # we'll go in the direction indicated by the subclass.  Otherwise,
-            # we'll go the other way (client = not normal ^ _tlsClientDefault,
-            # in other words).
-            if normal:
-                client = self._tlsClientDefault
-            else:
-                client = not self._tlsClientDefault
-
-            tlsFactory = TLSMemoryBIOFactory(contextFactory, client, None)
-            tlsProtocol = TLSMemoryBIOProtocol(tlsFactory, self.protocol, False)
-            self.protocol = tlsProtocol
-
-            self.getHandle = tlsProtocol.getHandle
-            self.getPeerCertificate = tlsProtocol.getPeerCertificate
-
-            # Mark the transport as secure.
-            directlyProvides(self, interfaces.ISSLTransport)
-
-            # Remember we did this so that write and writeSequence can send the
-            # data to the right place.
-            self._tls = True
-
-            # Hook it up
-            self.protocol.makeConnection(_BypassTLS(self))
+            _startTLS(self, contextFactory, normal, abstract.FileHandle)
 
 
     def write(self, data):
@@ -213,7 +157,9 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.write}
         """
-        if self._tls:
+        if self.disconnected:
+            return
+        if self.TLS:
             self.protocol.write(data)
         else:
             abstract.FileHandle.write(self, data)
@@ -227,7 +173,9 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.writeSequence}
         """
-        if self._tls:
+        if self.disconnected:
+            return
+        if self.TLS:
             self.protocol.writeSequence(iovec)
         else:
             abstract.FileHandle.writeSequence(self, iovec)
@@ -240,15 +188,22 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.loseConnection}
         """
-        if self._tls:
+        if self.TLS:
             if self.connected and not self.disconnecting:
                 self.protocol.loseConnection()
         else:
             abstract.FileHandle.loseConnection(self, reason)
 
+if _startTLS is not None:
+    classImplements(Connection, interfaces.ITLSTransport)
 
 
 class Client(Connection):
+    """
+    @ivar _tlsClientDefault: Always C{True}, indicating that this is a client
+        connection, and by default when TLS is negotiated this class will act as
+        a TLS client.
+    """
     addressFamily = socket.AF_INET
     socketType = socket.SOCK_STREAM
 
@@ -298,7 +253,7 @@ class Client(Connection):
             return
 
         try:
-            self._closeSocket()
+            self._closeSocket(True)
         except AttributeError:
             pass
         else:
@@ -322,12 +277,13 @@ class Client(Connection):
             self.failIfNotConnected(error.getConnectError((rc,
                                     errno.errorcode.get(rc, 'Unknown error'))))
         else:
-            self.socket.setsockopt(socket.SOL_SOCKET,
-                                   SO_UPDATE_CONNECT_CONTEXT,
-                                   struct.pack('I', self.socket.fileno()))
+            self.socket.setsockopt(
+                socket.SOL_SOCKET, SO_UPDATE_CONNECT_CONTEXT,
+                struct.pack('P', self.socket.fileno()))
             self.protocol = self.connector.buildProtocol(self.getPeer())
             self.connected = True
-            self.logstr = self.protocol.__class__.__name__+",client"
+            logPrefix = self._getLogPrefix(self.protocol)
+            self.logstr = logPrefix + ",client"
             self.protocol.makeConnection(self)
             self.startReading()
 
@@ -342,10 +298,7 @@ class Client(Connection):
         evt = _iocp.Event(self.cbConnect, self)
 
         rc = _iocp.connect(self.socket.fileno(), self.realAddress, evt)
-        if rc == ERROR_IO_PENDING:
-            return
-        else:
-            evt.ignore = True
+        if rc and rc != ERROR_IO_PENDING:
             self.cbConnect(rc, 0, 0, evt)
 
 
@@ -355,8 +308,7 @@ class Client(Connection):
 
         This indicates the address from which I am connecting.
         """
-        return address.IPv4Address('TCP', *(self.socket.getsockname() +
-                                            ('INET',)))
+        return address.IPv4Address('TCP', *self.socket.getsockname())
 
 
     def getPeer(self):
@@ -365,7 +317,7 @@ class Client(Connection):
 
         This indicates the address that I am connected to.
         """
-        return address.IPv4Address('TCP', *(self.realAddress + ('INET',)))
+        return address.IPv4Address('TCP', *self.realAddress)
 
 
     def __repr__(self):
@@ -389,6 +341,10 @@ class Server(Connection):
 
     I am a serverside network connection transport; a socket which came from an
     accept() on a server.
+
+    @ivar _tlsClientDefault: Always C{False}, indicating that this is a server
+        connection, and by default when TLS is negotiated this class will act as
+        a TLS server.
     """
 
     _tlsClientDefault = False
@@ -406,8 +362,8 @@ class Server(Connection):
         self.serverAddr = serverAddr
         self.clientAddr = clientAddr
         self.sessionno = sessionno
-        self.logstr = "%s,%s,%s" % (self.protocol.__class__.__name__,
-                                    sessionno, self.clientAddr.host)
+        logPrefix = self._getLogPrefix(self.protocol)
+        self.logstr = "%s,%s,%s" % (logPrefix, sessionno, self.clientAddr.host)
         self.repstr = "<%s #%s on %s>" % (self.protocol.__class__.__name__,
                                           self.sessionno, self.serverAddr.port)
         self.connected = True
@@ -447,7 +403,7 @@ class Connector(TCPConnector):
 
 
 
-class Port(styles.Ephemeral, _SocketCloser):
+class Port(_SocketCloser, _LogOwner):
     implements(interfaces.IListeningPort)
 
     connected = False
@@ -458,12 +414,15 @@ class Port(styles.Ephemeral, _SocketCloser):
 
     sessionno = 0
 
-    maxAccepts = 100
-
     # Actual port number being listened on, only set to a non-None
     # value when we are actually listening.
     _realPortNumber = None
 
+    # A string describing the connections which will be created by this port.
+    # Normally this is C{"TCP"}, since this is a TCP port, but when the TLS
+    # implementation re-uses this class it overrides the value with C{"TLS"}.
+    # Only used for logging.
+    _type = 'TCP'
 
     def __init__(self, port, factory, backlog=50, interface='', reactor=None):
         self.port = port
@@ -498,7 +457,7 @@ class Port(styles.Ephemeral, _SocketCloser):
         # reflect what the OS actually assigned us.
         self._realPortNumber = skt.getsockname()[1]
 
-        log.msg("%s starting on %s" % (self.factory.__class__,
+        log.msg("%s starting on %s" % (self._getLogPrefix(self.factory),
                                        self._realPortNumber))
 
         self.factory.doStart()
@@ -528,11 +487,18 @@ class Port(styles.Ephemeral, _SocketCloser):
     stopListening = loseConnection
 
 
+    def _logConnectionLostMsg(self):
+        """
+        Log message for closing port
+        """
+        log.msg('(%s Port %s Closed)' % (self._type, self._realPortNumber))
+
+
     def connectionLost(self, reason):
         """
         Cleans up the socket.
         """
-        log.msg('(Port %s Closed)' % self._realPortNumber)
+        self._logConnectionLostMsg()
         self._realPortNumber = None
         d = None
         if hasattr(self, "deferred"):
@@ -542,7 +508,7 @@ class Port(styles.Ephemeral, _SocketCloser):
         self.disconnected = True
         self.reactor.removeActiveHandle(self)
         self.connected = False
-        self._closeSocket()
+        self._closeSocket(True)
         del self.socket
         del self.getFileHandle
 
@@ -573,8 +539,7 @@ class Port(styles.Ephemeral, _SocketCloser):
 
         This indicates the server's address.
         """
-        return address.IPv4Address('TCP', *(self.socket.getsockname() +
-                                            ('INET',)))
+        return address.IPv4Address('TCP', *self.socket.getsockname())
 
 
     def cbAccept(self, rc, bytes, evt):
@@ -594,8 +559,9 @@ class Port(styles.Ephemeral, _SocketCloser):
                     (errno.errorcode.get(rc, 'unknown error'), rc))
             return False
         else:
-            evt.newskt.setsockopt(socket.SOL_SOCKET, SO_UPDATE_ACCEPT_CONTEXT,
-                                  struct.pack('I', self.socket.fileno()))
+            evt.newskt.setsockopt(
+                socket.SOL_SOCKET, SO_UPDATE_ACCEPT_CONTEXT,
+                struct.pack('P', self.socket.fileno()))
             family, lAddr, rAddr = _iocp.get_accept_addrs(evt.newskt.fileno(),
                                                           evt.buff)
             assert family == self.addressFamily
@@ -608,32 +574,24 @@ class Port(styles.Ephemeral, _SocketCloser):
                 s = self.sessionno
                 self.sessionno = s+1
                 transport = Server(evt.newskt, protocol,
-                        address.IPv4Address('TCP', rAddr[0], rAddr[1], 'INET'),
-                        address.IPv4Address('TCP', lAddr[0], lAddr[1], 'INET'),
+                        address.IPv4Address('TCP', rAddr[0], rAddr[1]),
+                        address.IPv4Address('TCP', lAddr[0], lAddr[1]),
                         s, self.reactor)
                 protocol.makeConnection(transport)
             return True
 
 
     def doAccept(self):
-        numAccepts = 0
-        while 1:
-            evt = _iocp.Event(self.cbAccept, self)
+        evt = _iocp.Event(self.cbAccept, self)
 
-            # see AcceptEx documentation
-            evt.buff = buff = _iocp.AllocateReadBuffer(2 * (self.addrLen + 16))
+        # see AcceptEx documentation
+        evt.buff = buff = _iocp.AllocateReadBuffer(2 * (self.addrLen + 16))
 
-            evt.newskt = newskt = self.reactor.createSocket(self.addressFamily,
-                                                            self.socketType)
-            rc = _iocp.accept(self.socket.fileno(), newskt.fileno(), buff, evt)
+        evt.newskt = newskt = self.reactor.createSocket(self.addressFamily,
+                                                        self.socketType)
+        rc = _iocp.accept(self.socket.fileno(), newskt.fileno(), buff, evt)
 
-            if (rc == ERROR_IO_PENDING
-                or (not rc and numAccepts >= self.maxAccepts)):
-                break
-            else:
-                evt.ignore = True
-                if not self.handleAccept(rc, evt):
-                    break
-            numAccepts += 1
+        if rc and rc != ERROR_IO_PENDING:
+            self.handleAccept(rc, evt)
 
 
