@@ -1,5 +1,5 @@
-# -*- test-case-name: twisted.test.test_internet -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.test.test_internet,twisted.internet.test.test_posixbase -*-
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -10,12 +10,13 @@ import warnings
 import socket
 import errno
 import os
+import sys
 
 from zope.interface import implements, classImplements
 
 from twisted.python.compat import set
 from twisted.internet.interfaces import IReactorUNIX, IReactorUNIXDatagram
-from twisted.internet.interfaces import IReactorTCP, IReactorUDP, IReactorSSL, IReactorArbitrary
+from twisted.internet.interfaces import IReactorTCP, IReactorUDP, IReactorSSL, _IReactorArbitrary
 from twisted.internet.interfaces import IReactorProcess, IReactorMulticast
 from twisted.internet.interfaces import IHalfCloseableDescriptor
 from twisted.internet import error
@@ -26,12 +27,21 @@ from twisted.persisted import styles
 from twisted.python.runtime import platformType, platform
 
 from twisted.internet.base import ReactorBase, _SignalReactorMixin
+from twisted.internet.main import CONNECTION_DONE, CONNECTION_LOST
+
+# Exceptions that doSelect might return frequently
+_NO_FILENO = error.ConnectionFdescWentAway('Handler has no fileno method')
+_NO_FILEDESC = error.ConnectionFdescWentAway('File descriptor lost')
+
 
 try:
-    from twisted.internet import ssl
-    sslEnabled = True
+    from twisted.protocols import tls
 except ImportError:
-    sslEnabled = False
+    tls = None
+    try:
+        from twisted.internet import ssl
+    except ImportError:
+        ssl = None
 
 try:
     from twisted.internet import unix
@@ -41,8 +51,7 @@ except ImportError:
 
 processEnabled = False
 if platformType == 'posix':
-    from twisted.internet import fdesc
-    import process
+    from twisted.internet import fdesc, process, _signals
     processEnabled = True
 
 if platform.isWindows():
@@ -103,10 +112,19 @@ class _SocketWaker(log.Logger, styles.Ephemeral):
 
 
 
-class _PipeWaker(log.Logger, styles.Ephemeral):
+class _FDWaker(object, log.Logger, styles.Ephemeral):
     """
     The I{self-pipe trick<http://cr.yp.to/docs/selfpipe.html>}, used to wake
     up the main loop from another thread or a signal handler.
+
+    L{_FDWaker} is a base class for waker implementations based on
+    writing to a pipe being monitored by the reactor.
+
+    @ivar o: The file descriptor for the end of the pipe which can be
+        written to to wake up a reactor monitoring this waker.
+
+    @ivar i: The file descriptor which should be monitored in order to
+        be awoken by this waker.
     """
     disconnected = 0
 
@@ -124,22 +142,13 @@ class _PipeWaker(log.Logger, styles.Ephemeral):
         fdesc._setCloseOnExec(self.o)
         self.fileno = lambda: self.i
 
+
     def doRead(self):
-        """Read some bytes from the pipe.
+        """
+        Read some bytes from the pipe and discard them.
         """
         fdesc.readFromFD(self.fileno(), lambda data: None)
 
-    def wakeUp(self):
-        """Write one byte to the pipe, and flush it.
-        """
-        # We don't use fdesc.writeToFD since we need to distinguish
-        # between EINTR (try again) and EAGAIN (do nothing).
-        if self.o is not None:
-            try:
-                util.untilConcludes(os.write, self.o, 'x')
-            except OSError, e:
-                if e.errno != errno.EAGAIN:
-                    raise
 
     def connectionLost(self, reason):
         """Close both ends of my pipe.
@@ -154,24 +163,84 @@ class _PipeWaker(log.Logger, styles.Ephemeral):
         del self.i, self.o
 
 
+
+class _UnixWaker(_FDWaker):
+    """
+    This class provides a simple interface to wake up the event loop.
+
+    This is used by threads or signals to wake up the event loop.
+    """
+
+    def wakeUp(self):
+        """Write one byte to the pipe, and flush it.
+        """
+        # We don't use fdesc.writeToFD since we need to distinguish
+        # between EINTR (try again) and EAGAIN (do nothing).
+        if self.o is not None:
+            try:
+                util.untilConcludes(os.write, self.o, 'x')
+            except OSError, e:
+                # XXX There is no unit test for raising the exception
+                # for other errnos. See #4285.
+                if e.errno != errno.EAGAIN:
+                    raise
+
+
+
 if platformType == 'posix':
-    _Waker = _PipeWaker
+    _Waker = _UnixWaker
 else:
     # Primarily Windows and Jython.
     _Waker = _SocketWaker
 
 
+class _SIGCHLDWaker(_FDWaker):
+    """
+    L{_SIGCHLDWaker} can wake up a reactor whenever C{SIGCHLD} is
+    received.
+
+    @see: L{twisted.internet._signals}
+    """
+    def __init__(self, reactor):
+        _FDWaker.__init__(self, reactor)
+
+
+    def install(self):
+        """
+        Install the handler necessary to make this waker active.
+        """
+        _signals.installHandler(self.o)
+
+
+    def uninstall(self):
+        """
+        Remove the handler which makes this waker active.
+        """
+        _signals.installHandler(-1)
+
+
+    def doRead(self):
+        """
+        Having woken up the reactor in response to receipt of
+        C{SIGCHLD}, reap the process which exited.
+
+        This is called whenever the reactor notices the waker pipe is
+        writeable, which happens soon after any call to the C{wakeUp}
+        method.
+        """
+        _FDWaker.doRead(self)
+        process.reapAllProcesses()
+
+
+
 class PosixReactorBase(_SignalReactorMixin, ReactorBase):
     """
     A basis for reactors that use file descriptors.
+
+    @ivar _childWaker: C{None} or a reference to the L{_SIGCHLDWaker}
+        which is used to properly notice child process termination.
     """
-    implements(IReactorArbitrary, IReactorTCP, IReactorUDP, IReactorMulticast)
-
-    def __init__(self):
-        ReactorBase.__init__(self)
-        if self.usingThreads or platformType == "posix":
-            self.installWaker()
-
+    implements(_IReactorArbitrary, IReactorTCP, IReactorUDP, IReactorMulticast)
 
     def _disconnectSelectable(self, selectable, why, isRead, faildict={
         error.ConnectionDone: failure.Failure(error.ConnectionDone()),
@@ -209,6 +278,43 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
             self.addReader(self.waker)
 
 
+    _childWaker = None
+    def _handleSignals(self):
+        """
+        Extend the basic signal handling logic to also support
+        handling SIGCHLD to know when to try to reap child processes.
+        """
+        _SignalReactorMixin._handleSignals(self)
+        if platformType == 'posix':
+            if not self._childWaker:
+                self._childWaker = _SIGCHLDWaker(self)
+                self._internalReaders.add(self._childWaker)
+                self.addReader(self._childWaker)
+            self._childWaker.install()
+            # Also reap all processes right now, in case we missed any
+            # signals before we installed the SIGCHLD waker/handler.
+            # This should only happen if someone used spawnProcess
+            # before calling reactor.run (and the process also exited
+            # already).
+            process.reapAllProcesses()
+
+    def _uninstallHandler(self):
+        """
+        If a child waker was created and installed, uninstall it now.
+
+        Since this disables reactor functionality and is only called
+        when the reactor is stopping, it doesn't provide any directly
+        useful functionality, but the cleanup of reactor-related
+        process-global state that it does helps in unit tests
+        involving multiple reactors and is generally just a nice
+        thing.
+        """
+        # XXX This would probably be an alright place to put all of
+        # the cleanup code for all internal readers (here and in the
+        # base class, anyway).  See #3063 for that cleanup task.
+        if self._childWaker:
+            self._childWaker.uninstall()
+
     # IReactorProcess
 
     def spawnProcess(self, processProtocol, executable, args=(),
@@ -225,8 +331,10 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
                 return process.Process(self, executable, args, env, path,
                                        processProtocol, uid, gid, childFDs)
         elif platformType == "win32":
-            if uid is not None or gid is not None:
-                raise ValueError("The uid and gid parameters are not supported on Windows.")
+            if uid is not None:
+                raise ValueError("Setting UID is unsupported on this platform.")
+            if gid is not None:
+                raise ValueError("Setting GID is unsupported on this platform.")
             if usePTY:
                 raise ValueError("The usePTY parameter is not supported on Windows.")
             if childFDs:
@@ -275,32 +383,11 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         c.connect()
         return c
 
-    _unspecified = object()
-    def _checkMode(self, name, mode):
-        """
-        Check C{mode} to see if a value was specified for it and emit a
-        deprecation warning if so.  Return the default value if none was
-        specified, otherwise return C{mode}.
-        """
-        if mode is not self._unspecified:
-            warnings.warn(
-                'The mode parameter of %(name)s will be removed.  Do not pass '
-                'a value for it.  Set permissions on the containing directory '
-                'before calling %(name)s, instead.' % dict(name=name),
-                category=DeprecationWarning,
-                stacklevel=3)
-        else:
-            mode = 0666
-        return mode
-
-
-    def listenUNIX(self, address, factory, backlog=50, mode=_unspecified,
-                   wantPID=0):
+    def listenUNIX(self, address, factory, backlog=50, mode=0666, wantPID=0):
         """
         @see: twisted.internet.interfaces.IReactorUNIX.listenUNIX
         """
         assert unixEnabled, "UNIX support is not present"
-        mode = self._checkMode('IReactorUNIX.listenUNIX', mode)
         p = unix.Port(address, factory, backlog, mode, self, wantPID)
         p.startListening()
         return p
@@ -309,7 +396,7 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
     # IReactorUNIXDatagram
 
     def listenUNIXDatagram(self, address, protocol, maxPacketSize=8192,
-                           mode=_unspecified):
+                           mode=0666):
         """
         Connects a given L{DatagramProtocol} to the given path.
 
@@ -318,20 +405,18 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         @returns: object conforming to L{IListeningPort}.
         """
         assert unixEnabled, "UNIX support is not present"
-        mode = self._checkMode('IReactorUNIXDatagram.listenUNIXDatagram', mode)
         p = unix.DatagramPort(address, protocol, maxPacketSize, mode, self)
         p.startListening()
         return p
 
     def connectUNIXDatagram(self, address, protocol, maxPacketSize=8192,
-                            mode=_unspecified, bindAddress=None):
+                            mode=0666, bindAddress=None):
         """
         Connects a L{ConnectedDatagramProtocol} instance to a path.
 
         EXPERIMENTAL.
         """
         assert unixEnabled, "UNIX support is not present"
-        mopde = self._checkMode('IReactorUNIXDatagram.connectUNIXDatagram', mode)
         p = unix.ConnectedDatagramPort(address, protocol, maxPacketSize, mode, bindAddress, self)
         p.startListening()
         return p
@@ -358,32 +443,61 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
     def connectSSL(self, host, port, factory, contextFactory, timeout=30, bindAddress=None):
         """@see: twisted.internet.interfaces.IReactorSSL.connectSSL
         """
-        assert sslEnabled, "SSL support is not present"
-        c = ssl.Connector(host, port, factory, contextFactory, timeout, bindAddress, self)
-        c.connect()
-        return c
+        if tls is not None:
+            tlsFactory = tls.TLSMemoryBIOFactory(contextFactory, True, factory)
+            return self.connectTCP(host, port, tlsFactory, timeout, bindAddress)
+        elif ssl is not None:
+            c = ssl.Connector(
+                host, port, factory, contextFactory, timeout, bindAddress, self)
+            c.connect()
+            return c
+        else:
+            assert False, "SSL support is not present"
+
+
 
     def listenSSL(self, port, factory, contextFactory, backlog=50, interface=''):
         """@see: twisted.internet.interfaces.IReactorSSL.listenSSL
         """
-        assert sslEnabled, "SSL support is not present"
-        p = ssl.Port(port, factory, contextFactory, backlog, interface, self)
-        p.startListening()
-        return p
+        if tls is not None:
+            tlsFactory = tls.TLSMemoryBIOFactory(contextFactory, False, factory)
+            port = self.listenTCP(port, tlsFactory, backlog, interface)
+            port._type = 'TLS'
+            return port
+        elif ssl is not None:
+            p = ssl.Port(
+                port, factory, contextFactory, backlog, interface, self)
+            p.startListening()
+            return p
+        else:
+            assert False, "SSL support is not present"
+
 
     # IReactorArbitrary
     def listenWith(self, portType, *args, **kw):
+        warnings.warn(
+            "listenWith is deprecated since Twisted 10.1.  "
+            "See IReactorFDSet.",
+            category=DeprecationWarning,
+            stacklevel=2)
         kw['reactor'] = self
         p = portType(*args, **kw)
         p.startListening()
         return p
 
+
     def connectWith(self, connectorType, *args, **kw):
+        warnings.warn(
+            "connectWith is deprecated since Twisted 10.1.  "
+            "See IReactorFDSet.",
+            category=DeprecationWarning,
+            stacklevel=2)
         kw['reactor'] = self
         c = connectorType(*args, **kw)
         c.connect()
         return c
 
+
     def _removeAll(self, readers, writers):
         """
         Remove all readers and writers, and list of removed L{IReadDescriptor}s
@@ -407,7 +521,81 @@ class PosixReactorBase(_SignalReactorMixin, ReactorBase):
         return list(removedReaders | removedWriters)
 
 
-if sslEnabled:
+class _PollLikeMixin(object):
+    """
+    Mixin for poll-like reactors.
+
+    Subclasses must define the following attributes::
+
+      - _POLL_DISCONNECTED - Bitmask for events indicating a connection was
+        lost.
+      - _POLL_IN - Bitmask for events indicating there is input to read.
+      - _POLL_OUT - Bitmask for events indicating output can be written.
+
+    Must be mixed in to a subclass of PosixReactorBase (for
+    _disconnectSelectable).
+    """
+
+    def _doReadOrWrite(self, selectable, fd, event):
+        """
+        fd is available for read or write, do the work and raise errors if
+        necessary.
+        """
+        why = None
+        inRead = False
+        if event & self._POLL_DISCONNECTED and not (event & self._POLL_IN):
+            # Handle disconnection.  But only if we finished processing all
+            # the pending input.
+            if fd in self._reads:
+                # If we were reading from the descriptor then this is a
+                # clean shutdown.  We know there are no read events pending
+                # because we just checked above.  It also might be a
+                # half-close (which is why we have to keep track of inRead).
+                inRead = True
+                why = CONNECTION_DONE
+            else:
+                # If we weren't reading, this is an error shutdown of some
+                # sort.
+                why = CONNECTION_LOST
+        else:
+            # Any non-disconnect event turns into a doRead or a doWrite.
+            try:
+                # First check to see if the descriptor is still valid.  This
+                # gives fileno() a chance to raise an exception, too. 
+                # Ideally, disconnection would always be indicated by the
+                # return value of doRead or doWrite (or an exception from
+                # one of those methods), but calling fileno here helps make
+                # buggy applications more transparent.
+                if selectable.fileno() == -1:
+                    # -1 is sort of a historical Python artifact.  Python
+                    # files and sockets used to change their file descriptor
+                    # to -1 when they closed.  For the time being, we'll
+                    # continue to support this anyway in case applications
+                    # replicated it, plus abstract.FileDescriptor.fileno
+                    # returns -1.  Eventually it'd be good to deprecate this
+                    # case.
+                    why = _NO_FILEDESC
+                else:
+                    if event & self._POLL_IN:
+                        # Handle a read event.
+                        why = selectable.doRead()
+                        inRead = True
+                    if not why and event & self._POLL_OUT:
+                        # Handle a write event, as long as doRead didn't
+                        # disconnect us.
+                        why = selectable.doWrite()
+                        inRead = False
+            except:
+                # Any exception from application code gets logged and will
+                # cause us to disconnect the selectable.
+                why = sys.exc_info()[1]
+                log.err()
+        if why:
+            self._disconnectSelectable(selectable, why, inRead)
+
+
+
+if tls is not None or ssl is not None:
     classImplements(PosixReactorBase, IReactorSSL)
 if unixEnabled:
     classImplements(PosixReactorBase, IReactorUNIX, IReactorUNIXDatagram)
