@@ -1,9 +1,7 @@
-# -*- test-case-name: twisted.test.test_factories -*-
-#
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.test.test_factories,twisted.internet.test.test_protocol -*-
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Standard implementations of Twisted protocol-related interfaces.
 
@@ -22,13 +20,14 @@ from twisted.internet import interfaces, error, defer
 
 
 class Factory:
-    """This is a factory which produces protocols.
+    """
+    This is a factory which produces protocols.
 
     By default, buildProtocol will create a protocol of the class given in
     self.protocol.
     """
 
-    implements(interfaces.IProtocolFactory)
+    implements(interfaces.IProtocolFactory, interfaces.ILoggingContext)
 
     # put a subclass of Protocol here:
     protocol = None
@@ -36,6 +35,13 @@ class Factory:
     numPorts = 0
     noisy = True
 
+    def logPrefix(self):
+        """
+        Describe this factory for log messages.
+        """
+        return self.__class__.__name__
+
+
     def doStart(self):
         """Make sure startFactory is called.
 
@@ -133,30 +139,63 @@ class ClientFactory(Factory):
 
 
 class _InstanceFactory(ClientFactory):
-    """Factory used by ClientCreator."""
+    """
+    Factory used by ClientCreator.
+
+    @ivar deferred: The L{Deferred} which represents this connection attempt and
+        which will be fired when it succeeds or fails.
+
+    @ivar pending: After a connection attempt succeeds or fails, a delayed call
+        which will fire the L{Deferred} representing this connection attempt.
+    """
 
     noisy = False
+    pending = None
 
     def __init__(self, reactor, instance, deferred):
         self.reactor = reactor
         self.instance = instance
         self.deferred = deferred
 
+
     def __repr__(self):
         return "<ClientCreator factory: %r>" % (self.instance, )
 
+
     def buildProtocol(self, addr):
-        self.reactor.callLater(0, self.deferred.callback, self.instance)
-        del self.deferred
+        """
+        Return the pre-constructed protocol instance and arrange to fire the
+        waiting L{Deferred} to indicate success establishing the connection.
+        """
+        self.pending = self.reactor.callLater(
+            0, self.fire, self.deferred.callback, self.instance)
+        self.deferred = None
         return self.instance
 
+
     def clientConnectionFailed(self, connector, reason):
-        self.reactor.callLater(0, self.deferred.errback, reason)
-        del self.deferred
+        """
+        Arrange to fire the waiting L{Deferred} with the given failure to
+        indicate the connection could not be established.
+        """
+        self.pending = self.reactor.callLater(
+            0, self.fire, self.deferred.errback, reason)
+        self.deferred = None
+
+
+    def fire(self, func, value):
+        """
+        Clear C{self.pending} to avoid a reference cycle and then invoke func
+        with the value.
+        """
+        self.pending = None
+        func(value)
+
 
 
 class ClientCreator:
-    """Client connections that do not require a factory.
+    """
+    Client connections that do not require a factory.
 
     The various connect* methods create a protocol instance using the given
     protocol class and arguments, and connect it, returning a Deferred of the
@@ -165,6 +204,14 @@ class ClientCreator:
     Useful for cases when we don't really need a factory.  Mainly this
     is when there is no shared state between protocol instances, and no need
     to reconnect.
+
+    The C{connectTCP}, C{connectUNIX}, and C{connectSSL} methods each return a
+    L{Deferred} which will fire with an instance of the protocol class passed to
+    L{ClientCreator.__init__}.  These Deferred can be cancelled to abort the
+    connection attempt (in a very unlikely case, cancelling the Deferred may not
+    prevent the protocol from being instantiated and connected to a transport;
+    if this happens, it will be disconnected immediately afterwards and the
+    Deferred will still errback with L{CancelledError}).
     """
 
     def __init__(self, reactor, protocolClass, *args, **kwargs):
@@ -173,26 +220,82 @@ class ClientCreator:
         self.args = args
         self.kwargs = kwargs
 
-    def connectTCP(self, host, port, timeout=30, bindAddress=None):
-        """Connect to remote host, return Deferred of resulting protocol instance."""
-        d = defer.Deferred()
-        f = _InstanceFactory(self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
-        self.reactor.connectTCP(host, port, f, timeout=timeout, bindAddress=bindAddress)
+
+    def _connect(self, method, *args, **kwargs):
+        """
+        Initiate a connection attempt.
+
+        @param method: A callable which will actually start the connection
+            attempt.  For example, C{reactor.connectTCP}.
+
+        @param *args: Positional arguments to pass to C{method}, excluding the
+            factory.
+
+        @param **kwargs: Keyword arguments to pass to C{method}.
+
+        @return: A L{Deferred} which fires with an instance of the protocol
+            class passed to this L{ClientCreator}'s initializer or fails if the
+            connection cannot be set up for some reason.
+        """
+        def cancelConnect(deferred):
+            connector.disconnect()
+            if f.pending is not None:
+                f.pending.cancel()
+        d = defer.Deferred(cancelConnect)
+        f = _InstanceFactory(
+            self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
+        connector = method(factory=f, *args, **kwargs)
         return d
 
-    def connectUNIX(self, address, timeout = 30, checkPID=0):
-        """Connect to Unix socket, return Deferred of resulting protocol instance."""
-        d = defer.Deferred()
-        f = _InstanceFactory(self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
-        self.reactor.connectUNIX(address, f, timeout = timeout, checkPID=checkPID)
-        return d
+
+    def connectTCP(self, host, port, timeout=30, bindAddress=None):
+        """
+        Connect to a TCP server.
+
+        The parameters are all the same as to L{IReactorTCP.connectTCP} except
+        that the factory parameter is omitted.
+
+        @return: A L{Deferred} which fires with an instance of the protocol
+            class passed to this L{ClientCreator}'s initializer or fails if the
+            connection cannot be set up for some reason.
+        """
+        return self._connect(
+            self.reactor.connectTCP, host, port, timeout=timeout,
+            bindAddress=bindAddress)
+
+
+    def connectUNIX(self, address, timeout=30, checkPID=False):
+        """
+        Connect to a Unix socket.
+
+        The parameters are all the same as to L{IReactorUNIX.connectUNIX} except
+        that the factory parameter is omitted.
+
+        @return: A L{Deferred} which fires with an instance of the protocol
+            class passed to this L{ClientCreator}'s initializer or fails if the
+            connection cannot be set up for some reason.
+        """
+        return self._connect(
+            self.reactor.connectUNIX, address, timeout=timeout,
+            checkPID=checkPID)
+
 
     def connectSSL(self, host, port, contextFactory, timeout=30, bindAddress=None):
-        """Connect to SSL server, return Deferred of resulting protocol instance."""
-        d = defer.Deferred()
-        f = _InstanceFactory(self.reactor, self.protocolClass(*self.args, **self.kwargs), d)
-        self.reactor.connectSSL(host, port, f, contextFactory, timeout=timeout, bindAddress=bindAddress)
-        return d
+        """
+        Connect to an SSL server.
+
+        The parameters are all the same as to L{IReactorSSL.connectSSL} except
+        that the factory parameter is omitted.
+
+        @return: A L{Deferred} which fires with an instance of the protocol
+            class passed to this L{ClientCreator}'s initializer or fails if the
+            connection cannot be set up for some reason.
+        """
+        return self._connect(
+            self.reactor.connectSSL, host, port,
+            contextFactory=contextFactory, timeout=timeout,
+            bindAddress=bindAddress)
+
 
 
 class ReconnectingClientFactory(ClientFactory):
@@ -204,13 +307,17 @@ class ReconnectingClientFactory(ClientFactory):
 
     @ivar maxDelay: Maximum number of seconds between connection attempts.
     @ivar initialDelay: Delay for the first reconnection attempt.
-    @ivar factor: a multiplicitive factor by which the delay grows
-    @ivar jitter: percentage of randomness to introduce into the delay length
+    @ivar factor: A multiplicitive factor by which the delay grows
+    @ivar jitter: Percentage of randomness to introduce into the delay length
         to prevent stampeding.
-    @ivar clock: the clock used to schedule reconnection. It's mainly useful to
+    @ivar clock: The clock used to schedule reconnection. It's mainly useful to
         be parametrized in tests. If the factory is serialized, this attribute
         will not be serialized, and the default value (the reactor) will be
         restored when deserialized.
+    @type clock: L{IReactorTime}
+    @ivar maxRetries: Maximum number of consecutive unsuccessful connection
+        attempts, after which no further connection attempts will be made. If
+        this is not explicitly set, no maximum is applied.
     """
     maxDelay = 3600
     initialDelay = 1.0
@@ -292,14 +399,12 @@ class ReconnectingClientFactory(ClientFactory):
         if self._callID:
             self._callID.cancel()
             self._callID = None
+        self.continueTrying = 0
         if self.connector:
-            # Hopefully this doesn't just make clientConnectionFailed
-            # retry again.
             try:
                 self.connector.stopConnecting()
             except error.NotConnectingError:
                 pass
-        self.continueTrying = 0
 
 
     def resetDelay(self):
@@ -334,22 +439,15 @@ class ServerFactory(Factory):
     """
 
 
+
 class BaseProtocol:
-    """This is the abstract superclass of all protocols.
-
-    If you are going to write a new protocol for Twisted, start here.  The
-    docstrings of this class explain how you can get started.  Any protocol
-    implementation, either client or server, should be a subclass of me.
-
-    My API is quite simple.  Implement dataReceived(data) to handle both
-    event-based and synchronous input; output can be sent through the
-    'transport' attribute, which is to be an instance that implements
-    L{twisted.internet.interfaces.ITransport}.
-
-    Some subclasses exist already to help you write common types of protocols:
-    see the L{twisted.protocols.basic} module for a few of them.
     """
+    This is the abstract superclass of all protocols.
 
+    Some methods have helpful default implementations here so that they can
+    easily be shared, but otherwise the direct subclasses of this class are more
+    interesting, L{Protocol} and L{ProcessProtocol}.
+    """
     connected = 0
     transport = None
 
@@ -379,8 +477,31 @@ connectionDone.cleanFailure()
 
 
 class Protocol(BaseProtocol):
+    """
+    This is the base class for streaming connection-oriented protocols.
+
+    If you are going to write a new connection-oriented protocol for Twisted,
+    start here.  Any protocol implementation, either client or server, should
+    be a subclass of this class.
+
+    The API is quite simple.  Implement L{dataReceived} to handle both
+    event-based and synchronous input; output can be sent through the
+    'transport' attribute, which is to be an instance that implements
+    L{twisted.internet.interfaces.ITransport}.  Override C{connectionLost} to be
+    notified when the connection ends.
+
+    Some subclasses exist already to help you write common types of protocols:
+    see the L{twisted.protocols.basic} module for a few of them.
+    """
+    implements(interfaces.IProtocol, interfaces.ILoggingContext)
+
+    def logPrefix(self):
+        """
+        Return a prefix matching the class name, to identify log messages
+        related to this protocol instance.
+        """
+        return self.__class__.__name__
 
-    implements(interfaces.IProtocol)
 
     def dataReceived(self, data):
         """Called whenever data is received.
@@ -586,6 +707,15 @@ class DatagramProtocol(AbstractDatagramProtocol):
     @ivar transport: The transport with which this protocol is associated,
         if it is associated with one.
     """
+    implements(interfaces.ILoggingContext)
+
+    def logPrefix(self):
+        """
+        Return a prefix matching the class name, to identify log messages
+        related to this protocol instance.
+        """
+        return self.__class__.__name__
+
 
     def connectionRefused(self):
         """Called due to error from write in connected mode.
