@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,13 +9,51 @@ __metaclass__ = type
 
 import signal
 import time
+import inspect
 
 from twisted.internet.abstract import FileDescriptor
-from twisted.internet.error import ReactorAlreadyRunning
+from twisted.internet.error import ReactorAlreadyRunning, ReactorNotRestartable
 from twisted.internet.defer import Deferred
 from twisted.internet.test.reactormixins import ReactorBuilder
 
 
+class ObjectModelIntegrationMixin(object):
+    """
+    Helpers for tests about the object model of reactor-related objects.
+    """
+    def assertFullyNewStyle(self, instance):
+        """
+        Assert that the given object is an instance of a new-style class and
+        that there are no classic classes in the inheritance hierarchy of
+        that class.
+
+        This is a beneficial condition because PyPy is better able to
+        optimize attribute lookup on such classes.
+        """
+        self.assertIsInstance(instance, object)
+        mro = inspect.getmro(type(instance))
+        for subclass in mro:
+            self.assertTrue(
+                issubclass(subclass, object),
+                "%r is not new-style" % (subclass,))
+
+
+
+class ObjectModelIntegrationTest(ReactorBuilder, ObjectModelIntegrationMixin):
+    """
+    Test details of object model integration against all reactors.
+    """
+
+    def test_newstyleReactor(self):
+        """
+        Checks that all reactors on a platform have method resolution order
+        containing only new style classes.
+        """
+        reactor = self.buildReactor()
+        self.assertFullyNewStyle(reactor)
+
+
+
 class SystemEventTestsBuilder(ReactorBuilder):
     """
     Builder defining tests relating to L{IReactorCore.addSystemEventTrigger}
@@ -37,7 +75,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
         """
         reactor = self.buildReactor()
         reactor.callWhenRunning(reactor.stop)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertRaises(RuntimeError, reactor.stop)
 
 
@@ -51,7 +89,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
         reactor.callWhenRunning(events.append, "first")
         reactor.callWhenRunning(events.append, "second")
         reactor.callWhenRunning(reactor.stop)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(events, ["first", "second"])
 
 
@@ -74,7 +112,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
         reactor.addSystemEventTrigger("after", "startup", afterStartup)
         reactor.callWhenRunning(reactor.stop)
         self.assertEqual(state, {})
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(
             state,
             {"before": False,
@@ -103,7 +141,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
         reactor.callWhenRunning(reactor.stop)
         self.assertEqual(phase[0], None)
         self.assertEqual(sawPhase[0], None)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(sawPhase[0], "before")
         self.assertEqual(phase[0], "after")
 
@@ -125,8 +163,8 @@ class SystemEventTestsBuilder(ReactorBuilder):
             "after", "shutdown",
             lambda: events.append(("after", "shutdown")))
         reactor.callWhenRunning(reactor.stop)
-        reactor.run()
-        self.assertEquals(events, [("before", "shutdown"),
+        self.runReactor(reactor)
+        self.assertEqual(events, [("before", "shutdown"),
                                    ("during", "shutdown"),
                                    ("after", "shutdown")])
 
@@ -145,7 +183,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
             events.append("stopped")
         reactor.callWhenRunning(stopIt)
         self.assertEqual(events, [])
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(events, ["stopped", "before shutdown"])
 
 
@@ -193,7 +231,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
         reactor = self.buildReactor()
         reactor.callWhenRunning(reentrantRun)
         reactor.callWhenRunning(reactor.stop)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(events, ["tested"])
 
 
@@ -215,7 +253,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
             reactor.stop()
         reactor = self.buildReactor()
         reactor.addSystemEventTrigger('before', 'startup', trigger)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(events, ['trigger', 'callback'])
 
 
@@ -245,7 +283,7 @@ class SystemEventTestsBuilder(ReactorBuilder):
             "before", "shutdown",
             lambda: events.append(("before", "shutdown")))
         reactor.callWhenRunning(reactor.callLater, 0, reactor.crash)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertFalse(reactor.running)
         self.assertFalse(
             events,
@@ -263,13 +301,31 @@ class SystemEventTestsBuilder(ReactorBuilder):
             reactor.crash()
         reactor = self.buildReactor()
         reactor.callWhenRunning(crash)
-        reactor.run()
+        self.runReactor(reactor)
         def stop():
             events.append(('stop', reactor.running))
             reactor.stop()
         reactor.callWhenRunning(stop)
-        reactor.run()
+        self.runReactor(reactor)
         self.assertEqual(events, ['crash', ('stop', True)])
 
 
+    def test_runAfterStop(self):
+        """
+        C{reactor.run()} raises L{ReactorNotRestartable} when called when
+        the reactor is being run after getting stopped priorly.
+        """
+        events = []
+        def restart():
+            self.assertRaises(ReactorNotRestartable, reactor.run)
+            events.append('tested')
+        reactor = self.buildReactor()
+        reactor.callWhenRunning(reactor.stop)
+        reactor.addSystemEventTrigger('after', 'shutdown', restart)
+        self.runReactor(reactor)
+        self.assertEqual(events, ['tested'])
+
+
+
 globals().update(SystemEventTestsBuilder.makeTestCaseClasses())
+globals().update(ObjectModelIntegrationTest.makeTestCaseClasses())
