@@ -1,4 +1,4 @@
-# Copyright (c) 2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 import sys
@@ -32,4 +32,4 @@ class QtreactorTestCase(unittest.TestCase):
         try:
             import twisted.internet.qtreactor
         except ImportError, e:
-            self.assertEquals(str(e), errorMessage)
+            self.assertEqual(str(e), errorMessage)
