@@ -16,14 +16,30 @@ from zope.interface import directlyProvides, providedBy
 # twisted imports
 from twisted.internet.protocol import ServerFactory, Protocol, ClientFactory
 from twisted.internet import error
+from twisted.internet.interfaces import ILoggingContext
 from twisted.python import log
 
 
+def _wrappedLogPrefix(wrapper, wrapped):
+    """
+    Compute a log prefix for a wrapper and the object it wraps.
+
+    @rtype: C{str}
+    """
+    if ILoggingContext.providedBy(wrapped):
+        logPrefix = wrapped.logPrefix()
+    else:
+        logPrefix = wrapped.__class__.__name__
+    return "%s (%s)" % (logPrefix, wrapper.__class__.__name__)
+
+
+
 class ProtocolWrapper(Protocol):
     """
     Wraps protocol instances and acts as their transport as well.
 
-    @ivar wrappedProtocol: An L{IProtocol} provider to which L{IProtocol}
+    @ivar wrappedProtocol: An L{IProtocol<twisted.internet.interfaces.IProtocol>}
+        provider to which L{IProtocol<twisted.internet.interfaces.IProtocol>}
         method calls onto this L{ProtocolWrapper} will be proxied.
 
     @ivar factory: The L{WrappingFactory} which created this
@@ -36,6 +52,15 @@ class ProtocolWrapper(Protocol):
         self.wrappedProtocol = wrappedProtocol
         self.factory = factory
 
+
+    def logPrefix(self):
+        """
+        Use a customized log prefix mentioning both the wrapped protocol and
+        the current one.
+        """
+        return _wrappedLogPrefix(self, self.wrappedProtocol)
+
+
     def makeConnection(self, transport):
         """
         When a connection is made, register this wrapper with its factory,
@@ -47,48 +72,62 @@ class ProtocolWrapper(Protocol):
         self.factory.registerProtocol(self)
         self.wrappedProtocol.makeConnection(self)
 
+
     # Transport relaying
 
     def write(self, data):
         self.transport.write(data)
 
+
     def writeSequence(self, data):
         self.transport.writeSequence(data)
 
+
     def loseConnection(self):
         self.disconnecting = 1
         self.transport.loseConnection()
 
+
     def getPeer(self):
         return self.transport.getPeer()
 
+
     def getHost(self):
         return self.transport.getHost()
 
+
     def registerProducer(self, producer, streaming):
         self.transport.registerProducer(producer, streaming)
 
+
     def unregisterProducer(self):
         self.transport.unregisterProducer()
 
+
     def stopConsuming(self):
         self.transport.stopConsuming()
 
+
     def __getattr__(self, name):
         return getattr(self.transport, name)
 
+
     # Protocol relaying
 
     def dataReceived(self, data):
         self.wrappedProtocol.dataReceived(data)
 
+
     def connectionLost(self, reason):
         self.factory.unregisterProtocol(self)
         self.wrappedProtocol.connectionLost(reason)
 
 
+
 class WrappingFactory(ClientFactory):
-    """Wraps a factory and its protocols, and keeps track of them."""
+    """
+    Wraps a factory and its protocols, and keeps track of them.
+    """
 
     protocol = ProtocolWrapper
 
@@ -96,35 +135,55 @@ class WrappingFactory(ClientFactory):
         self.wrappedFactory = wrappedFactory
         self.protocols = {}
 
+
+    def logPrefix(self):
+        """
+        Generate a log prefix mentioning both the wrapped factory and this one.
+        """
+        return _wrappedLogPrefix(self, self.wrappedFactory)
+
+
     def doStart(self):
         self.wrappedFactory.doStart()
         ClientFactory.doStart(self)
 
+
     def doStop(self):
         self.wrappedFactory.doStop()
         ClientFactory.doStop(self)
 
+
     def startedConnecting(self, connector):
         self.wrappedFactory.startedConnecting(connector)
 
+
     def clientConnectionFailed(self, connector, reason):
         self.wrappedFactory.clientConnectionFailed(connector, reason)
 
+
     def clientConnectionLost(self, connector, reason):
         self.wrappedFactory.clientConnectionLost(connector, reason)
 
+
     def buildProtocol(self, addr):
         return self.protocol(self, self.wrappedFactory.buildProtocol(addr))
 
+
     def registerProtocol(self, p):
-        """Called by protocol to register itself."""
+        """
+        Called by protocol to register itself.
+        """
         self.protocols[p] = 1
 
+
     def unregisterProtocol(self, p):
-        """Called by protocols when they go away."""
+        """
+        Called by protocols when they go away.
+        """
         del self.protocols[p]
 
 
+
 class ThrottlingProtocol(ProtocolWrapper):
     """Protocol for ThrottlingFactory."""
 
