@@ -3,9 +3,10 @@
 # See LICENSE for details.
 
 """
-Implementation of a TLS transport (L{ISSLTransport}) as an L{IProtocol}
-layered on top of any L{ITransport} implementation, based on OpenSSL's
-memory BIO features.
+Implementation of a TLS transport (L{ISSLTransport}) as an
+L{IProtocol<twisted.internet.interfaces.IProtocol>} layered on top of any
+L{ITransport<twisted.internet.interfaces.ITransport>} implementation, based on
+U{OpenSSL<http://www.openssl.org>}'s memory BIO features.
 
 L{TLSMemoryBIOFactory} is a L{WrappingFactory} which wraps protocols created by
 the factory it wraps with L{TLSMemoryBIOProtocol}.  L{TLSMemoryBIOProtocol}
@@ -27,13 +28,11 @@ implement SSL and TLS.  Typical usage of this module looks like this::
     reactor.listenTCP(12345, tlsFactory)
     reactor.run()
 
-Because the reactor's SSL and TLS APIs are likely implemented in a more
-efficient way, it is more common to use them (see L{IReactorSSL} and
-L{ITLSTransport}).  However, this API offers somewhat more flexibility; for
-example, a L{TLSMemoryBIOProtocol} instance can use another instance of
-L{TLSMemoryBIOProtocol} as its transport, yielding TLS over TLS - useful to
-implement onion routing.  Or it can be used to run TLS over a UNIX socket,
-or over stdio to a child process.
+This API offers somewhat more flexibility than
+L{twisted.internet.interfaces.IReactorSSL}; for example, a L{TLSMemoryBIOProtocol}
+instance can use another instance of L{TLSMemoryBIOProtocol} as its transport,
+yielding TLS over TLS - useful to implement onion routing.  It can also be used
+to run TLS over unusual transports, such as UNIX sockets and stdio.
 """
 
 
@@ -50,12 +49,162 @@ except TypeError, e:
 from zope.interface import implements
 
 from twisted.python.failure import Failure
+from twisted.python import log
+from twisted.python.reflect import safe_str
 from twisted.internet.interfaces import ISystemHandle, ISSLTransport
-from twisted.internet.main import CONNECTION_DONE, CONNECTION_LOST
+from twisted.internet.interfaces import IPushProducer, ILoggingContext
+from twisted.internet.main import CONNECTION_LOST
 from twisted.internet.protocol import Protocol
+from twisted.internet.task import cooperate
 from twisted.protocols.policies import ProtocolWrapper, WrappingFactory
 
 
+class _PullToPush(object):
+    """
+    An adapter that converts a non-streaming to a streaming producer.
+
+    Because of limitations of the producer API, this adapter requires the
+    cooperation of the consumer. When the consumer's C{registerProducer} is
+    called with a non-streaming producer, it must wrap it with L{_PullToPush}
+    and then call C{startStreaming} on the resulting object. When the
+    consumer's C{unregisterProducer} is called, it must call
+    C{stopStreaming} on the L{_PullToPush} instance.
+
+    If the underlying producer throws an exception from C{resumeProducing},
+    the producer will be unregistered from the consumer.
+
+    @ivar _producer: the underling non-streaming producer.
+
+    @ivar _consumer: the consumer with which the underlying producer was
+                     registered.
+
+    @ivar _finished: C{bool} indicating whether the producer has finished.
+
+    @ivar _coopTask: the result of calling L{cooperate}, the task driving the
+                     streaming producer.
+    """
+    implements(IPushProducer)
+
+    _finished = False
+
+
+    def __init__(self, pullProducer, consumer):
+        self._producer = pullProducer
+        self._consumer = consumer
+
+
+    def _pull(self):
+        """
+        A generator that calls C{resumeProducing} on the underlying producer
+        forever.
+
+        If C{resumeProducing} throws an exception, the producer is
+        unregistered, which should result in streaming stopping.
+        """
+        while True:
+            try:
+                self._producer.resumeProducing()
+            except:
+                log.err(None, "%s failed, producing will be stopped:" %
+                        (safe_str(self._producer),))
+                try:
+                    self._consumer.unregisterProducer()
+                    # The consumer should now call stopStreaming() on us,
+                    # thus stopping the streaming.
+                except:
+                    # Since the consumer blew up, we may not have had
+                    # stopStreaming() called, so we just stop on our own:
+                    log.err(None, "%s failed to unregister producer:" %
+                            (safe_str(self._consumer),))
+                    self._finished = True
+                    return
+            yield None
+
+
+    def startStreaming(self):
+        """
+        This should be called by the consumer when the producer is registered.
+
+        Start streaming data to the consumer.
+        """
+        self._coopTask = cooperate(self._pull())
+
+
+    def stopStreaming(self):
+        """
+        This should be called by the consumer when the producer is unregistered.
+
+        Stop streaming data to the consumer.
+        """
+        if self._finished:
+            return
+        self._finished = True
+        self._coopTask.stop()
+
+
+    # IPushProducer implementation:
+    def pauseProducing(self):
+        self._coopTask.pause()
+
+
+    def resumeProducing(self):
+        self._coopTask.resume()
+
+
+    def stopProducing(self):
+        self.stopStreaming()
+        self._producer.stopProducing()
+
+
+
+class _ProducerMembrane(object):
+    """
+    Stand-in for producer registered with a L{TLSMemoryBIOProtocol} transport.
+
+    Ensures that producer pause/resume events from the undelying transport are
+    coordinated with pause/resume events from the TLS layer.
+
+    @ivar _producer: The application-layer producer.
+    """
+    implements(IPushProducer)
+
+    _producerPaused = False
+
+    def __init__(self, producer):
+        self._producer = producer
+
+
+    def pauseProducing(self):
+        """
+        C{pauseProducing} the underlying producer, if it's not paused.
+        """
+        if self._producerPaused:
+            return
+        self._producerPaused = True
+        self._producer.pauseProducing()
+
+
+    def resumeProducing(self):
+        """
+        C{resumeProducing} the underlying producer, if it's paused.
+        """
+        if not self._producerPaused:
+            return
+        self._producerPaused = False
+        self._producer.resumeProducing()
+
+
+    def stopProducing(self):
+        """
+        C{stopProducing} the underlying producer.
+
+        There is only a single source for this event, so it's simply passed
+        on.
+        """
+        self._producer.stopProducing()
+
+
+
 class TLSMemoryBIOProtocol(ProtocolWrapper):
     """
     L{TLSMemoryBIOProtocol} is a protocol wrapper which uses OpenSSL via a
@@ -63,11 +212,19 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
     underlying transport and decrypts bytes received from the underlying
     transport before delivering them to the wrapped protocol.
 
+    In addition to producer events from the underlying transport, the need to
+    wait for reads before a write can proceed means the
+    L{TLSMemoryBIOProtocol} may also want to pause a producer. Pause/resume
+    events are therefore merged using the L{_ProducerMembrane}
+    wrapper. Non-streaming (pull) producers are supported by wrapping them
+    with L{_PullToPush}.
+
     @ivar _tlsConnection: The L{OpenSSL.SSL.Connection} instance which is
         encrypted and decrypting this connection.
 
-    @ivar _lostConnection: A flag indicating whether connection loss has
-        already been dealt with (C{True}) or not (C{False}).
+    @ivar _lostTLSConnection: A flag indicating whether connection loss has
+        already been dealt with (C{True}) or not (C{False}). TLS disconnection
+        is distinct from the underlying connection being lost.
 
     @ivar _writeBlockedOnRead: A flag indicating whether further writing must
         wait for data to be received (C{True}) or not (C{False}).
@@ -79,9 +236,9 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
 
     @ivar _connectWrapped: A flag indicating whether or not to call
         C{makeConnection} on the wrapped protocol.  This is for the reactor's
-        L{ITLSTransport.startTLS} implementation, since it has a protocol which
-        it has already called C{makeConnection} on, and which has no interest
-        in a new transport.  See #3821.
+        L{twisted.internet.interfaces.ITLSTransport.startTLS} implementation,
+        since it has a protocol which it has already called C{makeConnection}
+        on, and which has no interest in a new transport.  See #3821.
 
     @ivar _handshakeDone: A flag indicating whether or not the handshake is
         known to have completed successfully (C{True}) or not (C{False}).  This
@@ -97,13 +254,18 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
         the connection to be lost, it is saved here.  If appropriate, this may
         be used as the reason passed to the application protocol's
         C{connectionLost} method.
+
+    @ivar _producer: The current producer registered via C{registerProducer},
+        or C{None} if no producer has been registered or a previous one was
+        unregistered.
     """
     implements(ISystemHandle, ISSLTransport)
 
     _reason = None
     _handshakeDone = False
-    _lostConnection = False
+    _lostTLSConnection = False
     _writeBlockedOnRead = False
+    _producer = None
 
     def __init__(self, factory, wrappedProtocol, _connectWrapped=True):
         ProtocolWrapper.__init__(self, factory, wrappedProtocol)
@@ -185,7 +347,7 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
         # close the connection.  Looping is necessary to make sure we
         # process all of the data which was put into the receive BIO, as
         # there is no guarantee that a single recv call will do it all.
-        while not self._lostConnection:
+        while not self._lostTLSConnection:
             try:
                 bytes = self._tlsConnection.recv(2 ** 15)
             except WantReadError:
@@ -195,32 +357,25 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
             except ZeroReturnError:
                 # TLS has shut down and no more TLS data will be received over
                 # this connection.
-                self._lostConnection = True
-                self.transport.loseConnection()
-                if not self._handshakeDone and self._reason is not None:
-                    failure = self._reason
-                else:
-                    failure = Failure(CONNECTION_DONE)
-                # Failure's are fat.  Drop the reference.
-                self._reason = None
-                ProtocolWrapper.connectionLost(self, failure)
+                self._shutdownTLS()
+                # Passing in None means the user protocol's connnectionLost
+                # will get called with reason from underlying transport:
+                self._tlsShutdownFinished(None)
             except Error, e:
                 # Something went pretty wrong.  For example, this might be a
                 # handshake failure (because there were no shared ciphers, because
                 # a certificate failed to verify, etc).  TLS can no longer proceed.
-                self._flushSendBIO()
-                self._lostConnection = True
 
-                # Squash EOF in violation of protocol into ConnectionLost
+                # Squash EOF in violation of protocol into ConnectionLost; we
+                # create Failure before calling _flushSendBio so that no new
+                # exception will get thrown in the interim.
                 if e.args[0] == -1 and e.args[1] == 'Unexpected EOF':
                     failure = Failure(CONNECTION_LOST)
                 else:
                     failure = Failure()
-                ProtocolWrapper.connectionLost(self, failure)
-                # This loseConnection call is basically tested by
-                # test_handshakeFailure.  At least one side will need to do it
-                # or the test never finishes.
-                self.transport.loseConnection()
+
+                self._flushSendBIO()
+                self._tlsShutdownFinished(failure)
             else:
                 # If we got application bytes, the handshake must be done by
                 # now.  Keep track of this to control error reporting later.
@@ -248,43 +403,100 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
             appSendBuffer = self._appSendBuffer
             self._appSendBuffer = []
             for bytes in appSendBuffer:
-                self.write(bytes)
-            if not self._writeBlockedOnRead and self.disconnecting:
-                self.loseConnection()
+                self._write(bytes)
+            if (not self._writeBlockedOnRead and self.disconnecting and
+                self.producer is None):
+                self._shutdownTLS()
+            if self._producer is not None:
+                self._producer.resumeProducing()
 
         self._flushReceiveBIO()
 
 
+    def _shutdownTLS(self):
+        """
+        Initiate, or reply to, the shutdown handshake of the TLS layer.
+        """
+        shutdownSuccess = self._tlsConnection.shutdown()
+        self._flushSendBIO()
+        if shutdownSuccess:
+            # Both sides have shutdown, so we can start closing lower-level
+            # transport. This will also happen if we haven't started
+            # negotiation at all yet, in which case shutdown succeeds
+            # immediately.
+            self.transport.loseConnection()
+
+
+    def _tlsShutdownFinished(self, reason):
+        """
+        Called when TLS connection has gone away; tell underlying transport to
+        disconnect.
+        """
+        self._reason = reason
+        self._lostTLSConnection = True
+        # Using loseConnection causes the application protocol's
+        # connectionLost method to be invoked non-reentrantly, which is always
+        # a nice feature. However, for error cases (reason != None) we might
+        # want to use abortConnection when it becomes available. The
+        # loseConnection call is basically tested by test_handshakeFailure.
+        # At least one side will need to do it or the test never finishes.
+        self.transport.loseConnection()
+
+
     def connectionLost(self, reason):
         """
         Handle the possible repetition of calls to this method (due to either
         the underlying transport going away or due to an error at the TLS
         layer) and make sure the base implementation only gets invoked once.
         """
-        if not self._lostConnection:
+        if not self._lostTLSConnection:
             # Tell the TLS connection that it's not going to get any more data
             # and give it a chance to finish reading.
             self._tlsConnection.bio_shutdown()
             self._flushReceiveBIO()
+            self._lostTLSConnection = True
+        reason = self._reason or reason
+        self._reason = None
+        ProtocolWrapper.connectionLost(self, reason)
 
 
     def loseConnection(self):
         """
         Send a TLS close alert and close the underlying connection.
         """
+        if self.disconnecting:
+            return
         self.disconnecting = True
-        if not self._writeBlockedOnRead:
-            self._tlsConnection.shutdown()
-            self._flushSendBIO()
-            self.transport.loseConnection()
+        if not self._writeBlockedOnRead and self._producer is None:
+            self._shutdownTLS()
 
 
     def write(self, bytes):
         """
         Process the given application bytes and send any resulting TLS traffic
         which arrives in the send BIO.
+
+        If C{loseConnection} was called, subsequent calls to C{write} will
+        drop the bytes on the floor.
         """
-        if self._lostConnection:
+        # Writes after loseConnection are not supported, unless a producer has
+        # been registered, in which case writes can happen until the producer
+        # is unregistered:
+        if self.disconnecting and self._producer is None:
+            return
+        self._write(bytes)
+
+
+    def _write(self, bytes):
+        """
+        Process the given application bytes and send any resulting TLS traffic
+        which arrives in the send BIO.
+
+        This may be called by C{dataReceived} with bytes that were buffered
+        before C{loseConnection} was called, which is why this function
+        doesn't check for disconnection but accepts the bytes regardless.
+        """
+        if self._lostTLSConnection:
             return
 
         leftToSend = bytes
@@ -294,17 +506,16 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
             except WantReadError:
                 self._writeBlockedOnRead = True
                 self._appSendBuffer.append(leftToSend)
+                if self._producer is not None:
+                    self._producer.pauseProducing()
                 break
-            except Error, e:
-                # Just drop the connection.  This has two useful consequences.
-                # First, for the application protocol's connectionLost method,
-                # it will squash any error into connection lost.  We *could*
-                # let the real exception propagate to application code, but the
-                # other SSL implementation doesn't.  Second, it causes the
-                # protocol's connectionLost method to be invoked
-                # non-reentrantly, which is always a nice feature.
-                self._reason = Failure()
-                self.transport.loseConnection()
+            except Error:
+                # Pretend TLS connection disconnected, which will trigger
+                # disconnect of underlying transport. The error will be passed
+                # to the application protocol's connectionLost method.  The
+                # other SSL implementation doesn't, but losing helpful
+                # debugging information is a bad idea.
+                self._tlsShutdownFinished(Failure())
                 break
             else:
                 # If we sent some bytes, the handshake must be done.  Keep
@@ -326,6 +537,33 @@ class TLSMemoryBIOProtocol(ProtocolWrapper):
         return self._tlsConnection.get_peer_certificate()
 
 
+    def registerProducer(self, producer, streaming):
+        # If we received a non-streaming producer, wrap it so it becomes a
+        # streaming producer:
+        if not streaming:
+            producer = streamingProducer = _PullToPush(producer, self)
+        producer = _ProducerMembrane(producer)
+        # This will raise an exception if a producer is already registered:
+        self.transport.registerProducer(producer, True)
+        self._producer = producer
+        # If we received a non-streaming producer, we need to start the
+        # streaming wrapper:
+        if not streaming:
+            streamingProducer.startStreaming()
+
+
+    def unregisterProducer(self):
+        # If we received a non-streaming producer, we need to stop the
+        # streaming wrapper:
+        if isinstance(self._producer._producer, _PullToPush):
+            self._producer._producer.stopStreaming()
+        self._producer = None
+        self._producerPaused = False
+        self.transport.unregisterProducer()
+        if self.disconnecting and not self._writeBlockedOnRead:
+            self._shutdownTLS()
+
+
 
 class TLSMemoryBIOFactory(WrappingFactory):
     """
@@ -339,7 +577,24 @@ class TLSMemoryBIOFactory(WrappingFactory):
     """
     protocol = TLSMemoryBIOProtocol
 
+    noisy = False  # disable unnecessary logging.
+    
     def __init__(self, contextFactory, isClient, wrappedFactory):
         WrappingFactory.__init__(self, wrappedFactory)
         self._contextFactory = contextFactory
         self._isClient = isClient
+
+
+    def logPrefix(self):
+        """
+        Annotate the wrapped factory's log prefix with some text indicating TLS
+        is in use.
+
+        @rtype: C{str}
+        """
+        if ILoggingContext.providedBy(self.wrappedFactory):
+            logPrefix = self.wrappedFactory.logPrefix()
+        else:
+            logPrefix = self.wrappedFactory.__class__.__name__
+        return "%s (TLS)" % (logPrefix,)
+
