@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.python.test.test_release -*-
-# Copyright (c) 2007-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -14,17 +14,13 @@ which must run on multiple platforms (eg the setup.py script).
 
 import textwrap
 from datetime import date
+import re
 import sys
 import os
 from tempfile import mkdtemp
 import tarfile
 
-# Popen4 isn't available on Windows.  BookBuilder won't work on Windows, but
-# we don't care. -exarkun
-try:
-    from popen2 import Popen4
-except ImportError:
-    Popen4 = None
+from subprocess import PIPE, STDOUT, Popen
 
 from twisted.python.versions import Version
 from twisted.python.filepath import FilePath
@@ -54,11 +50,13 @@ def runCommand(args):
 
     @raise CommandFailed: when the program exited with a non-0 exit code.
     """
-    process = Popen4(args)
-    stdout = process.fromchild.read()
+    process = Popen(args, stdout=PIPE, stderr=STDOUT)
+    stdout = process.stdout.read()
     exitCode = process.wait()
-    if os.WIFSIGNALED(exitCode) or os.WEXITSTATUS(exitCode):
-        raise CommandFailed(exitCode, stdout)
+    if exitCode < 0:
+        raise CommandFailed(None, -exitCode, stdout)
+    elif exitCode > 0:
+        raise CommandFailed(exitCode, None, stdout)
     return stdout
 
 
@@ -66,15 +64,19 @@ class CommandFailed(Exception):
     """
     Raised when a child process exits unsuccessfully.
 
-    @type exitCode: C{int}
-    @ivar exitCode: The exit code for the child process.
+    @type exitStatus: C{int}
+    @ivar exitStatus: The exit status for the child process.
+
+    @type exitSignal: C{int}
+    @ivar exitSignal: The exit signal for the child process.
 
     @type output: C{str}
     @ivar output: The bytes read from stdout and stderr of the child process.
     """
-    def __init__(self, exitCode, output):
-        Exception.__init__(self, exitCode, output)
-        self.exitCode = exitCode
+    def __init__(self, exitStatus, exitSignal, output):
+        Exception.__init__(self, exitStatus, exitSignal, output)
+        self.exitStatus = exitStatus
+        self.exitSignal = exitSignal
         self.output = output
 
 
@@ -108,33 +110,52 @@ def getNextVersion(version, now=None):
     return Version(version.package, major, minor, 0)
 
 
-def changeAllProjectVersions(root, versionTemplate):
+def changeAllProjectVersions(root, versionTemplate, today=None):
     """
     Change the version of all projects (including core and all subprojects).
 
+    If the current version of a project is pre-release, then also change the
+    versions in the current NEWS entries for that project.
+
     @type root: L{FilePath}
     @param root: The root of the Twisted source tree.
     @type versionTemplate: L{Version}
     @param versionTemplate: The version of all projects.  The name will be
         replaced for each respective project.
+    @type today: C{str}
+    @param today: A YYYY-MM-DD formatted string. If not provided, defaults to
+        the current day, according to the system clock.
     """
+    if not today:
+        today = date.today().strftime('%Y-%m-%d')
     for project in findTwistedProjects(root):
         if project.directory.basename() == "twisted":
             packageName = "twisted"
         else:
             packageName = "twisted." + project.directory.basename()
-        version = Version(packageName, versionTemplate.major,
-                          versionTemplate.minor, versionTemplate.micro,
-                          prerelease=versionTemplate.prerelease)
+        oldVersion = project.getVersion()
+        newVersion = Version(packageName, versionTemplate.major,
+                             versionTemplate.minor, versionTemplate.micro,
+                             prerelease=versionTemplate.prerelease)
+
+        if oldVersion.prerelease:
+            builder = NewsBuilder()
+            builder._changeNewsVersion(
+                root.child("NEWS"), builder._getNewsName(project),
+                oldVersion, newVersion, today)
+            builder._changeNewsVersion(
+                project.directory.child("topfiles").child("NEWS"),
+                builder._getNewsName(project), oldVersion, newVersion,
+                today)
 
         # The placement of the top-level README with respect to other files (eg
         # _version.py) is sufficiently different from the others that we just
         # have to handle it specially.
         if packageName == "twisted":
             _changeVersionInFile(
-                project.getVersion(), version, root.child('README').path)
+                oldVersion, newVersion, root.child('README').path)
 
-        project.updateVersion(version)
+        project.updateVersion(newVersion)
 
 
 
@@ -523,10 +544,10 @@ class BookBuilder(LoreBuilderMixin):
             try:
                 os.chdir(inputDirectory.path)
 
-                texToDVI = (
-                    "latex -interaction=nonstopmode "
-                    "-output-directory=%s %s") % (
-                    workPath.path, bookPath.path)
+                texToDVI = [
+                    "latex", "-interaction=nonstopmode",
+                    "-output-directory=" + workPath.path,
+                    bookPath.path]
 
                 # What I tell you three times is true!
                 # The first two invocations of latex on the book file allows it
@@ -539,13 +560,10 @@ class BookBuilder(LoreBuilderMixin):
                 dviPath = workPath.child(bookBaseWithoutExtension + ".dvi")
                 psPath = workPath.child(bookBaseWithoutExtension + ".ps")
                 pdfPath = workPath.child(bookBaseWithoutExtension + ".pdf")
-                self.run(
-                    "dvips -o %(postscript)s -t letter -Ppdf %(dvi)s" % {
-                        'postscript': psPath.path,
-                        'dvi': dviPath.path})
-                self.run("ps2pdf13 %(postscript)s %(pdf)s" % {
-                        'postscript': psPath.path,
-                        'pdf': pdfPath.path})
+                self.run([
+                    "dvips", "-o", psPath.path, "-t", "letter", "-Ppdf",
+                    dviPath.path])
+                self.run(["ps2pdf13", psPath.path, pdfPath.path])
                 pdfPath.moveTo(outputPath)
                 workPath.remove()
             finally:
@@ -588,31 +606,39 @@ class NewsBuilder(object):
     """
     Generate the new section of a NEWS file.
 
-    The C{_FEATURE}, C{_BUGFIX}, C{_REMOVAL}, and C{_MISC} attributes of
-    this class are symbolic names for the news entry types which are
-    supported.  Conveniently, they each also take on the value of the file
-    name extension which indicates a news entry of that type.
+    The C{_FEATURE}, C{_BUGFIX}, C{_DOC}, C{_REMOVAL}, and C{_MISC}
+    attributes of this class are symbolic names for the news entry types
+    which are supported.  Conveniently, they each also take on the value of
+    the file name extension which indicates a news entry of that type.
 
     @cvar _headings: A C{dict} mapping one of the news entry types to the
         heading to write out for that type of news entry.
 
+    @cvar _NO_CHANGES: A C{str} giving the text which appears when there are
+        no significant changes in a release.
+
     @cvar _TICKET_HINT: A C{str} giving the text which appears at the top of
         each news file and which should be kept at the top, not shifted down
         with all the other content.  Put another way, this is the text after
         which the new news text is inserted.
     """
+
     _FEATURE = ".feature"
     _BUGFIX = ".bugfix"
+    _DOC = ".doc"
     _REMOVAL = ".removal"
     _MISC = ".misc"
 
     _headings = {
         _FEATURE: "Features",
         _BUGFIX: "Bugfixes",
+        _DOC: "Improved Documentation",
         _REMOVAL: "Deprecations and Removals",
         _MISC: "Other",
         }
 
+    _NO_CHANGES = "No significant changes have been made for this release.\n"
+
     _TICKET_HINT = (
         'Ticket numbers in this file can be looked up by visiting\n'
         'http://twistedmatrix.com/trac/ticket/<number>\n'
@@ -651,6 +677,19 @@ class NewsBuilder(object):
         return results
 
 
+    def _formatHeader(self, header):
+        """
+        Format a header for a NEWS file.
+
+        A header is a title with '=' signs underlining it.
+
+        @param header: The header string to format.
+        @type header: C{str}
+        @return: A C{str} containing C{header}.
+        """
+        return header + '\n' + '=' * len(header) + '\n\n'
+
+
     def _writeHeader(self, fileObj, header):
         """
         Write a version header to the given file.
@@ -659,7 +698,7 @@ class NewsBuilder(object):
         @param header: The header to write to the file.
         @type header: C{str}
         """
-        fileObj.write(header + '\n' + '=' * len(header) + '\n\n')
+        fileObj.write(self._formatHeader(header))
 
 
     def _writeSection(self, fileObj, header, tickets):
@@ -734,6 +773,13 @@ class NewsBuilder(object):
         @param header: The top-level header to use when writing the news.
         @type header: L{str}
         """
+        changes = []
+        for part in (self._FEATURE, self._BUGFIX, self._DOC, self._REMOVAL):
+            tickets = self._findChanges(path, part)
+            if tickets:
+                changes.append((part, tickets))
+        misc = self._findChanges(path, self._MISC)
+
         oldNews = output.getContent()
         newNews = output.sibling('NEWS.new').open('w')
         if oldNews.startswith(self._TICKET_HINT):
@@ -741,31 +787,48 @@ class NewsBuilder(object):
             oldNews = oldNews[len(self._TICKET_HINT):]
 
         self._writeHeader(newNews, header)
-        for part in (self._FEATURE, self._BUGFIX, self._REMOVAL):
-            tickets = self._findChanges(path, part)
-            self._writeSection(newNews, self._headings.get(part), tickets)
-        self._writeMisc(
-            newNews,
-            self._headings.get(self._MISC),
-            self._findChanges(path, self._MISC))
+        if changes:
+            for (part, tickets) in changes:
+                self._writeSection(newNews, self._headings.get(part), tickets)
+        else:
+            newNews.write(self._NO_CHANGES)
+            newNews.write('\n')
+        self._writeMisc(newNews, self._headings.get(self._MISC), misc)
+        newNews.write('\n')
         newNews.write(oldNews)
         newNews.close()
         output.sibling('NEWS.new').moveTo(output)
 
 
-    def buildAll(self, baseDirectory):
+    def _getNewsName(self, project):
         """
-        Find all of the Twisted subprojects beneath C{baseDirectory} and update
-        their news files from the ticket change description files in their
-        I{topfiles} directories and update the news file in C{baseDirectory}
-        with all of the news.
+        Return the name of C{project} that should appear in NEWS.
+
+        @param project: A L{Project}
+        @return: The name of C{project}.
+        """
+        name = project.directory.basename().title()
+        if name == 'Twisted':
+            name = 'Core'
+        return name
+
+
+    def _iterProjects(self, baseDirectory):
+        """
+        Iterate through the Twisted projects in C{baseDirectory}, yielding
+        everything we need to know to build news for them.
+
+        Yields C{topfiles}, C{news}, C{name}, C{version} for each sub-project
+        in reverse-alphabetical order. C{topfile} is the L{FilePath} for the
+        topfiles directory, C{news} is the L{FilePath} for the NEWS file,
+        C{name} is the nice name of the project (as should appear in the NEWS
+        file), C{version} is the current version string for that project.
 
         @param baseDirectory: A L{FilePath} representing the root directory
             beneath which to find Twisted projects for which to generate
             news (see L{findTwistedProjects}).
+        @type baseDirectory: L{FilePath}
         """
-        today = self._today()
-
         # Get all the subprojects to generate news for
         projects = findTwistedProjects(baseDirectory)
         # And order them alphabetically for ease of reading
@@ -781,12 +844,55 @@ class NewsBuilder(object):
                     news = baseDirectory.child("NEWS")
                 else:
                     news = topfiles.child("NEWS")
-                name = project.directory.basename().title()
-                if name == 'Twisted':
-                    name = 'Core'
-                version = project.getVersion().base()
-                self.build(
-                    topfiles, news, "Twisted %s %s (%s)" % (name, version, today))
+                name = self._getNewsName(project)
+                version = project.getVersion()
+                yield topfiles, news, name, version
+
+
+    def buildAll(self, baseDirectory):
+        """
+        Find all of the Twisted subprojects beneath C{baseDirectory} and update
+        their news files from the ticket change description files in their
+        I{topfiles} directories and update the news file in C{baseDirectory}
+        with all of the news.
+
+        @param baseDirectory: A L{FilePath} representing the root directory
+            beneath which to find Twisted projects for which to generate
+            news (see L{findTwistedProjects}).
+        """
+        today = self._today()
+        for topfiles, news, name, version in self._iterProjects(baseDirectory):
+            self.build(
+                topfiles, news,
+                "Twisted %s %s (%s)" % (name, version.base(), today))
+
+
+    def _changeNewsVersion(self, news, name, oldVersion, newVersion, today):
+        """
+        Change all references to the current version number in a NEWS file to
+        refer to C{newVersion} instead.
+
+        @param news: The NEWS file to change.
+        @type news: L{FilePath}
+        @param name: The name of the project to change.
+        @type name: C{str}
+        @param oldVersion: The old version of the project.
+        @type oldVersion: L{Version}
+        @param newVersion: The new version of the project.
+        @type newVersion: L{Version}
+        @param today: A YYYY-MM-DD string representing today's date.
+        @type today: C{str}
+        """
+        newHeader = self._formatHeader(
+            "Twisted %s %s (%s)" % (name, newVersion.base(), today))
+        expectedHeaderRegex = re.compile(
+            r"Twisted %s %s \(\d{4}-\d\d-\d\d\)\n=+\n\n" % (
+                re.escape(name), re.escape(oldVersion.base())))
+        oldNews = news.getContent()
+        match = expectedHeaderRegex.search(oldNews)
+        if match:
+            oldHeader = match.group()
+            replaceInFile(news.path, {oldHeader: newHeader})
 
 
     def main(self, args):
@@ -838,14 +944,8 @@ class DistributionBuilder(object):
     A builder of Twisted distributions.
 
     This knows how to build tarballs for Twisted and all of its subprojects.
-
-    @type blacklist: C{list} of C{str}
-    @cvar blacklist: The list subproject names to exclude from the main Twisted
-        tarball and for which no individual project tarballs will be built.
     """
-
     from twisted.python.dist import twisted_subprojects as subprojects
-    blacklist = ["vfs", "web2"]
 
     def __init__(self, rootDirectory, outputDirectory, apiBaseURL=None):
         """
@@ -901,10 +1001,7 @@ class DistributionBuilder(object):
         """
         Build the main Twisted distribution in C{Twisted-<version>.tar.bz2}.
 
-        Projects listed in in L{blacklist} will not have their plugins, code,
-        documentation, or bin directories included.
-
-        bin/admin is also excluded.
+        bin/admin is excluded.
 
         @type version: C{str}
         @param version: The version of Twisted to build.
@@ -923,42 +1020,28 @@ class DistributionBuilder(object):
         # Generate docs!
         if docPath.isdir():
             for subProjectDir in docPath.children():
-                if (subProjectDir.isdir()
-                    and subProjectDir.basename() not in self.blacklist):
+                if subProjectDir.isdir():
                     for child in subProjectDir.walk():
                         self._buildDocInDir(child, version,
                             subProjectDir.child("howto"))
 
-        # Now, this part is nasty.  We need to exclude blacklisted subprojects
-        # from the main Twisted distribution. This means we need to exclude
-        # their bin directories, their documentation directories, their
-        # plugins, and their python packages. Given that there's no "add all
-        # but exclude these particular paths" functionality in tarfile, we have
-        # to walk through all these directories and add things that *aren't*
-        # part of the blacklisted projects.
-
         for binthing in self.rootDirectory.child("bin").children():
-            # bin/admin should also not be included.
-            if binthing.basename() not in self.blacklist + ["admin"]:
+            # bin/admin should not be included.
+            if binthing.basename() != "admin":
                 tarball.add(binthing.path,
                             buildPath("bin", binthing.basename()))
 
-        bad_plugins = ["twisted_%s.py" % (blacklisted,)
-                       for blacklisted in self.blacklist]
-
         for submodule in self.rootDirectory.child("twisted").children():
             if submodule.basename() == "plugins":
                 for plugin in submodule.children():
-                    if plugin.basename() not in bad_plugins:
-                        tarball.add(plugin.path, buildPath("twisted", "plugins",
-                                                           plugin.basename()))
-            elif submodule.basename() not in self.blacklist:
+                    tarball.add(plugin.path, buildPath("twisted", "plugins",
+                                                       plugin.basename()))
+            else:
                 tarball.add(submodule.path, buildPath("twisted",
                                                       submodule.basename()))
 
         for docDir in self.rootDirectory.child("doc").children():
-            if docDir.basename() not in self.blacklist:
-                tarball.add(docDir.path, buildPath("doc", docDir.basename()))
+            tarball.add(docDir.path, buildPath("doc", docDir.basename()))
 
         for toplevel in self.rootDirectory.children():
             if not toplevel.isdir():
@@ -1152,8 +1235,7 @@ def buildAllTarballs(checkout, destination):
 
     db.buildCore(versionString)
     for subproject in twisted_subprojects:
-        if (subproject not in db.blacklist
-            and twistedPath.child(subproject).exists()):
+        if twistedPath.child(subproject).exists():
             db.buildSubProject(subproject, versionString)
 
     db.buildTwisted(versionString)
@@ -1261,5 +1343,5 @@ class BuildAPIDocsScript(object):
         """
         if len(args) != 2:
             sys.exit("Must specify two arguments: "
-                     "Twisted checkout and distination path")
+                     "Twisted checkout and destination path")
         self.buildAPIDocs(FilePath(args[0]), FilePath(args[1]))
