@@ -1,7 +1,7 @@
 # -*- test-case-name: twisted.test.test_failure -*-
 # See also test suite twisted.test.test_pbfailure
 
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -17,6 +17,7 @@ import linecache
 import inspect
 import opcode
 from cStringIO import StringIO
+from inspect import getmro
 
 from twisted.python import reflect
 
@@ -35,12 +36,18 @@ def format_frames(frames, write, detail="default"):
     @type frames: list
     @param write: this will be called with formatted strings.
     @type write: callable
-    @param detail: Three detail levels are available:
-        default, brief, and verbose.
+    @param detail: Four detail levels are available:
+        default, brief, verbose, and verbose-vars-not-captured.
+        C{Failure.printDetailedTraceback} uses the latter when the caller asks
+        for verbose, but no vars were captured, so that an explicit warning
+        about the missing data is shown.
     @type detail: string
     """
-    if detail not in ('default', 'brief', 'verbose'):
-        raise ValueError, "Detail must be default, brief, or verbose. (not %r)" % (detail,)
+    if detail not in ('default', 'brief', 'verbose',
+                      'verbose-vars-not-captured'):
+        raise ValueError(
+            "Detail must be default, brief, verbose, or "
+            "verbose-vars-not-captured. (not %r)" % (detail,))
     w = write
     if detail == "brief":
         for method, filename, lineno, localVars, globalVars in frames:
@@ -49,6 +56,10 @@ def format_frames(frames, write, detail="default"):
         for method, filename, lineno, localVars, globalVars in frames:
             w( '  File "%s", line %s, in %s\n' % (filename, lineno, method))
             w( '    %s\n' % linecache.getline(filename, lineno).strip())
+    elif detail == "verbose-vars-not-captured":
+        for method, filename, lineno, localVars, globalVars in frames:
+            w("%s:%d: %s(...)\n" % (filename, lineno, method))
+        w(' [Capture of Locals and Globals disabled (use captureVars=True)]\n')
     elif detail == "verbose":
         for method, filename, lineno, localVars, globalVars in frames:
             w("%s:%d: %s(...)\n" % (filename, lineno, method))
@@ -102,11 +113,22 @@ class _Traceback(object):
 class _Frame(object):
     """
     A fake frame object, used by L{_Traceback}.
+
+    @ivar f_code: fake L{code<types.CodeType>} object
+    @ivar f_globals: fake f_globals dictionary (usually empty)
+    @ivar f_locals: fake f_locals dictionary (usually empty)
     """
 
     def __init__(self, name, filename):
+        """
+        @param name: method/function name for this frame.
+        @type name: C{str}
+        @param filename: filename for this frame.
+        @type name: C{str}
+        """
         self.f_code = _Code(name, filename)
         self.f_globals = {}
+        self.f_locals = {}
 
 
 class _Code(object):
@@ -125,8 +147,16 @@ class Failure:
     This is necessary because Python's built-in error mechanisms are
     inconvenient for asynchronous communication.
 
+    The C{stack} and C{frame} attributes contain frames.  Each frame is a tuple
+    of (funcName, fileName, lineNumber, localsItems, globalsItems), where
+    localsItems and globalsItems are the contents of
+    C{locals().items()}/C{globals().items()} for that frame, or an empty tuple
+    if those details were not captured.
+
     @ivar value: The exception instance responsible for this failure.
     @ivar type: The exception's class.
+    @ivar stack: list of frames, innermost last, excluding C{Failure.__init__}.
+    @ivar frames: list of frames, innermost first.
     """
 
     pickled = 0
@@ -137,7 +167,8 @@ class Failure:
     # throwExceptionIntoGenerator.
     _yieldOpcode = chr(opcode.opmap["YIELD_VALUE"])
 
-    def __init__(self, exc_value=None, exc_type=None, exc_tb=None):
+    def __init__(self, exc_value=None, exc_type=None, exc_tb=None,
+                 captureVars=False):
         """
         Initialize me with an explanation of the error.
 
@@ -156,11 +187,16 @@ class Failure:
         If C{None} is supplied for C{exc_value}, the value of C{exc_tb} is
         ignored, otherwise if C{exc_tb} is C{None}, it will be found from
         execution context (ie, L{sys.exc_info}).
+
+        @param captureVars: if set, capture locals and globals of stack
+            frames.  This is pretty slow, and makes no difference unless you
+            are going to use L{printDetailedTraceback}.
         """
         global count
         count = count + 1
         self.count = count
         self.type = self.value = tb = None
+        self.captureVars = captureVars
 
         #strings Exceptions/Failures are bad, mmkay?
         if isinstance(exc_value, (str, unicode)) and exc_type is None:
@@ -232,46 +268,54 @@ class Failure:
         #   what called upon the PB object.
 
         while f:
-            localz = f.f_locals.copy()
-            if f.f_locals is f.f_globals:
-                globalz = {}
+            if captureVars:
+                localz = f.f_locals.copy()
+                if f.f_locals is f.f_globals:
+                    globalz = {}
+                else:
+                    globalz = f.f_globals.copy()
+                for d in globalz, localz:
+                    if "__builtins__" in d:
+                        del d["__builtins__"]
+                localz = localz.items()
+                globalz = globalz.items()
             else:
-                globalz = f.f_globals.copy()
-            for d in globalz, localz:
-                if d.has_key("__builtins__"):
-                    del d["__builtins__"]
-            stack.insert(0, [
+                localz = globalz = ()
+            stack.insert(0, (
                 f.f_code.co_name,
                 f.f_code.co_filename,
                 f.f_lineno,
-                localz.items(),
-                globalz.items(),
-                ])
+                localz,
+                globalz,
+                ))
             f = f.f_back
 
         while tb is not None:
             f = tb.tb_frame
-            localz = f.f_locals.copy()
-            if f.f_locals is f.f_globals:
-                globalz = {}
+            if captureVars:
+                localz = f.f_locals.copy()
+                if f.f_locals is f.f_globals:
+                    globalz = {}
+                else:
+                    globalz = f.f_globals.copy()
+                for d in globalz, localz:
+                    if "__builtins__" in d:
+                        del d["__builtins__"]
+                localz = localz.items()
+                globalz = globalz.items()
             else:
-                globalz = f.f_globals.copy()
-            for d in globalz, localz:
-                if d.has_key("__builtins__"):
-                    del d["__builtins__"]
-
-            frames.append([
+                localz = globalz = ()
+            frames.append((
                 f.f_code.co_name,
                 f.f_code.co_filename,
                 tb.tb_lineno,
-                localz.items(),
-                globalz.items(),
-                ])
+                localz,
+                globalz,
+                ))
             tb = tb.tb_next
         if inspect.isclass(self.type) and issubclass(self.type, Exception):
-            parentCs = reflect.allYourBase(self.type)
+            parentCs = getmro(self.type)
             self.parents = map(reflect.qual, parentCs)
-            self.parents.append(reflect.qual(self.type))
         else:
             self.parents = [self.type]
 
@@ -282,15 +326,15 @@ class Failure:
         automatically re-raised if it is not a type that you expect.
 
         The reason for having this particular API is because it's very useful
-        in Deferred errback chains:
+        in Deferred errback chains::
 
-        | def _ebFoo(self, failure):
-        |     r = failure.trap(Spam, Eggs)
-        |     print 'The Failure is due to either Spam or Eggs!'
-        |     if r == Spam:
-        |         print 'Spam did it!'
-        |     elif r == Eggs:
-        |         print 'Eggs did it!'
+            def _ebFoo(self, failure):
+                r = failure.trap(Spam, Eggs)
+                print 'The Failure is due to either Spam or Eggs!'
+                if r == Spam:
+                    print 'Spam did it!'
+                elif r == Eggs:
+                    print 'Eggs did it!'
 
         If the failure is not a Spam or an Eggs, then the Failure
         will be 'passed on' to the next errback.
@@ -412,8 +456,8 @@ class Failure:
         c['frames'] = [
             [
                 v[0], v[1], v[2],
-                [(j[0], reflect.safe_repr(j[1])) for j in v[3]],
-                [(j[0], reflect.safe_repr(j[1])) for j in v[4]]
+                _safeReprVars(v[3]),
+                _safeReprVars(v[4]),
             ] for v in self.frames
         ]
 
@@ -426,8 +470,8 @@ class Failure:
             c['stack'] = [
                 [
                     v[0], v[1], v[2],
-                    [(j[0], reflect.safe_repr(j[1])) for j in v[3]],
-                    [(j[0], reflect.safe_repr(j[1])) for j in v[4]]
+                    _safeReprVars(v[3]),
+                    _safeReprVars(v[4]),
                 ] for v in self.stack
             ]
 
@@ -472,13 +516,34 @@ class Failure:
         self.printTraceback(file=io, elideFrameworkCode=elideFrameworkCode, detail=detail)
         return io.getvalue()
 
-    def printTraceback(self, file=None, elideFrameworkCode=0, detail='default'):
-        """Emulate Python's standard error reporting mechanism.
+
+    def printTraceback(self, file=None, elideFrameworkCode=False, detail='default'):
+        """
+        Emulate Python's standard error reporting mechanism.
+
+        @param file: If specified, a file-like object to which to write the
+            traceback.
+
+        @param elideFrameworkCode: A flag indicating whether to attempt to
+            remove uninteresting frames from within Twisted itself from the
+            output.
+
+        @param detail: A string indicating how much information to include
+            in the traceback.  Must be one of C{'brief'}, C{'default'}, or
+            C{'verbose'}.
         """
         if file is None:
             file = log.logerr
         w = file.write
 
+        if detail == 'verbose' and not self.captureVars:
+            # We don't have any locals or globals, so rather than show them as
+            # empty make the output explicitly say that we don't have them at
+            # all.
+            formatDetail = 'verbose-vars-not-captured'
+        else:
+            formatDetail = detail
+
         # Preamble
         if detail == 'verbose':
             w( '*--- Failure #%d%s---\n' %
@@ -489,16 +554,19 @@ class Failure:
                 hasFrames = 'Traceback'
             else:
                 hasFrames = 'Traceback (failure with no frames)'
-            w("%s: %s: %s\n" % (hasFrames, self.type, self.value))
+            w("%s: %s: %s\n" % (
+                    hasFrames,
+                    reflect.safe_str(self.type),
+                    reflect.safe_str(self.value)))
         else:
             w( 'Traceback (most recent call last):\n')
 
         # Frames, formatted in appropriate style
         if self.frames:
             if not elideFrameworkCode:
-                format_frames(self.stack[-traceupLength:], w, detail)
+                format_frames(self.stack[-traceupLength:], w, formatDetail)
                 w("%s\n" % (EXCEPTION_CAUGHT_HERE,))
-            format_frames(self.frames, w, detail)
+            format_frames(self.frames, w, formatDetail)
         elif not detail == 'brief':
             # Yeah, it's not really a traceback, despite looking like one...
             w("Failure: ")
@@ -523,6 +591,7 @@ class Failure:
         if detail == 'verbose':
             w('*--- End of Failure #%d ---\n' % self.count)
 
+
     def printBriefTraceback(self, file=None, elideFrameworkCode=0):
         """Print a traceback as densely as possible.
         """
@@ -533,19 +602,43 @@ class Failure:
         """
         self.printTraceback(file, elideFrameworkCode, detail='verbose')
 
+
+def _safeReprVars(varsDictItems):
+    """
+    Convert a list of (name, object) pairs into (name, repr) pairs.
+
+    L{twisted.python.reflect.safe_repr} is used to generate the repr, so no
+    exceptions will be raised by faulty C{__repr__} methods.
+
+    @param varsDictItems: a sequence of (name, value) pairs as returned by e.g.
+        C{locals().items()}.
+    @returns: a sequence of (name, repr) pairs.
+    """
+    return [(name, reflect.safe_repr(obj)) for (name, obj) in varsDictItems]
+
+
 # slyphon: make post-morteming exceptions tweakable
 
 DO_POST_MORTEM = True
 
 def _debuginit(self, exc_value=None, exc_type=None, exc_tb=None,
-             Failure__init__=Failure.__init__.im_func):
+               captureVars=False,
+               Failure__init__=Failure.__init__.im_func):
+    """
+    Initialize failure object, possibly spawning pdb.
+    """
     if (exc_value, exc_type, exc_tb) == (None, None, None):
         exc = sys.exc_info()
         if not exc[0] == self.__class__ and DO_POST_MORTEM:
-            print "Jumping into debugger for post-mortem of exception '%s':" % exc[1]
+            try:
+                strrepr = str(exc[1])
+            except:
+                strrepr = "broken str"
+            print "Jumping into debugger for post-mortem of exception '%s':" % (strrepr,)
             import pdb
             pdb.post_mortem(exc[2])
-    Failure__init__(self, exc_value, exc_type, exc_tb)
+    Failure__init__(self, exc_value, exc_type, exc_tb, captureVars)
+
 
 def startDebugMode():
     """Enable debug hooks for Failures."""
