@@ -1,8 +1,9 @@
 # -*- test-case-name: twisted.python.test.test_util -*-
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-import os, sys, hmac, errno, new, inspect, warnings
+import os, sys, hmac, errno, inspect, warnings
+import types
 try:
     import pwd, grp
 except ImportError:
@@ -565,10 +566,18 @@ class FancyEqMixin:
 
 
 def dsu(list, key):
+    """
+    decorate-sort-undecorate (aka "Schwartzian transform")
+
+    DEPRECATED. Use the built-in C{sorted()} instead.
+    """
+    warnings.warn(("dsu is deprecated since Twisted 10.1. "
+        "Use the built-in sorted() instead."), DeprecationWarning, stacklevel=2)
     L2 = [(key(e), i, e) for (i, e) in zip(range(len(list)), list)]
     L2.sort()
     return [e[2] for e in L2]
 
+
 try:
     from twisted.python._initgroups import initgroups as _c_initgroups
 except ImportError:
@@ -616,7 +625,7 @@ else:
 
         If the C extension is present, we're calling it, which in turn calls
         initgroups(3).
-        
+
         If not, this is done by reading the group database /etc/group and using
         all groups of which C{uid} is a member.  The additional group
         C{primaryGid} is also added to the list.
@@ -665,17 +674,47 @@ else:
 
 
 def switchUID(uid, gid, euid=False):
+    """
+    Attempts to switch the uid/euid and gid/egid for the current process.
+
+    If C{uid} is the same value as L{os.getuid} (or L{os.geteuid}),
+    this function will issue a L{UserWarning} and not raise an exception.
+
+    @type uid: C{int} or C{NoneType}
+    @param uid: the UID (or EUID) to switch the current process to. This
+                parameter will be ignored if the value is C{None}.
+
+    @type gid: C{int} or C{NoneType}
+    @param gid: the GID (or EGID) to switch the current process to. This
+                parameter will be ignored if the value is C{None}.
+
+    @type euid: C{bool}
+    @param euid: if True, set only effective user-id rather than real user-id.
+                 (This option has no effect unless the process is running
+                 as root, in which case it means not to shed all
+                 privileges, retaining the option to regain privileges
+                 in cases such as spawning processes. Use with caution.)
+    """
     if euid:
         setuid = os.seteuid
         setgid = os.setegid
+        getuid = os.geteuid
     else:
         setuid = os.setuid
         setgid = os.setgid
+        getuid = os.getuid
     if gid is not None:
         setgid(gid)
     if uid is not None:
-        initgroups(uid, gid)
-        setuid(uid)
+        if uid == getuid():
+            uidText = (euid and "euid" or "uid")
+            actionText = "tried to drop privileges and set%s %s" % (uidText, uid)
+            problemText = "%s is already %s" % (uidText, getuid())
+            warnings.warn("%s but %s; should we be root? Continuing."
+                          % (actionText, problemText))
+        else:
+            initgroups(uid, gid)
+            setuid(uid)
 
 
 class SubclassableCStringIO(object):
@@ -802,7 +841,7 @@ def mergeFunctionMetadata(f, g):
         g.__name__ = f.__name__
     except TypeError:
         try:
-            merged = new.function(
+            merged = types.FunctionType(
                 g.func_code, g.func_globals,
                 f.__name__, inspect.getargspec(g)[-1],
                 g.func_closure)
