@@ -1,11 +1,14 @@
 # -*- test-case-name: twisted.python.test.test_win32 -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Win32 utilities.
 
 See also twisted.python.shortcut.
+
+@var O_BINARY: the 'binary' mode flag on Windows, or 0 on other platforms, so it
+    may safely be OR'ed into a mask for os.open.
 """
 
 import re
@@ -26,6 +29,8 @@ ERROR_PATH_NOT_FOUND = 3
 ERROR_INVALID_NAME = 123
 ERROR_DIRECTORY = 267
 
+O_BINARY = getattr(os, "O_BINARY", 0)
+
 def _determineWindowsError():
     """
     Determine which WindowsError name to export.
