@@ -1,16 +1,22 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
-"""Tests for twisted.enterprise.adbapi."""
+"""
+Tests for twisted.enterprise.adbapi.
+"""
 
 from twisted.trial import unittest
 
-import os, stat, tempfile
+import os, stat
+import types
 
-from twisted.enterprise.adbapi import ConnectionPool, ConnectionLost
+from twisted.enterprise.adbapi import ConnectionPool, ConnectionLost, safe
+from twisted.enterprise.adbapi import Connection, Transaction
+from twisted.enterprise.adbapi import _unreleasedVersion
 from twisted.internet import reactor, defer, interfaces
-from twisted.python import log
+from twisted.python.failure import Failure
+
 
 simple_table_schema = """
 CREATE TABLE simple (
@@ -18,6 +24,7 @@ CREATE TABLE simple (
 )
 """
 
+
 class ADBAPITestBase:
     """Test the asynchronous DB-API code."""
 
@@ -26,11 +33,15 @@ class ADBAPITestBase:
     if interfaces.IReactorThreads(reactor, None) is None:
         skip = "ADB-API requires threads, no way to test without them"
 
-    def setUp(self):
+    def extraSetUp(self):
+        """
+        Set up the database and create a connection pool pointing at it.
+        """
         self.startDB()
         self.dbpool = self.makePool(cp_openfun=self.openfun)
         self.dbpool.start()
 
+
     def tearDown(self):
         d =  self.dbpool.runOperation('DROP TABLE simple')
         d.addCallback(lambda res: self.dbpool.close())
@@ -53,7 +64,7 @@ class ADBAPITestBase:
             d.addCallback(self._testPool_1_2)
             d.addCallback(self._testPool_1_3)
             d.addCallback(self._testPool_1_4)
-            d.addCallback(lambda res: log.flushErrors())
+            d.addCallback(lambda res: self.flushLoggedErrors())
         d.addCallback(self._testPool_2)
         d.addCallback(self._testPool_3)
         d.addCallback(self._testPool_4)
@@ -130,13 +141,13 @@ class ADBAPITestBase:
     def _testPool_4(self, res):
         # runInteraction
         d = self.dbpool.runInteraction(self.interaction)
-        d.addCallback(lambda res: self.assertEquals(res, "done"))
+        d.addCallback(lambda res: self.assertEqual(res, "done"))
         return d
 
     def _testPool_5(self, res):
         # withConnection
         d = self.dbpool.runWithConnection(self.withConnection)
-        d.addCallback(lambda res: self.assertEquals(res, "done"))
+        d.addCallback(lambda res: self.assertEqual(res, "done"))
         return d
 
     def _testPool_6(self, res):
@@ -185,12 +196,12 @@ class ADBAPITestBase:
         curs.execute("insert into simple(x) values(1)")
         curs.execute("select x from simple")
         res = curs.fetchall()
-        self.failUnlessEqual(len(res), 1)
-        self.failUnlessEqual(len(res[0]), 1)
-        self.failUnlessEqual(res[0][0], 1)
+        self.assertEqual(len(res), 1)
+        self.assertEqual(len(res[0]), 1)
+        self.assertEqual(res[0][0], 1)
         curs.execute("delete from simple")
         curs.execute("select x from simple")
-        self.failUnlessEqual(len(curs.fetchall()), 0)
+        self.assertEqual(len(curs.fetchall()), 0)
         curs.close()
         self.dbpool.disconnect(conn)
 
@@ -223,17 +234,17 @@ class ADBAPITestBase:
         finally:
             curs.close()
         return "done"
-        
+
     def close_withConnection(self, conn):
         conn.close()
-        
+
     def bad_withConnection(self, conn):
         curs = conn.cursor()
         try:
             curs.execute("select * from NOTABLE")
         finally:
             curs.close()
-        
+
 
 class ReconnectTestBase:
     """Test the asynchronous DB-API code with reconnect."""
@@ -241,7 +252,12 @@ class ReconnectTestBase:
     if interfaces.IReactorThreads(reactor, None) is None:
         skip = "ADB-API requires threads, no way to test without them"
 
-    def setUp(self):
+    def extraSetUp(self):
+        """
+        Skip the test if C{good_sql} is unavailable.  Otherwise, set up the
+        database, create a connection pool pointed at it, and set up a simple
+        schema in it.
+        """
         if self.good_sql is None:
             raise unittest.SkipTest('no good sql for reconnect test')
         self.startDB()
@@ -250,6 +266,7 @@ class ReconnectTestBase:
         self.dbpool.start()
         return self.dbpool.runOperation(simple_table_schema)
 
+
     def tearDown(self):
         d = self.dbpool.runOperation('DROP TABLE simple')
         d.addCallback(lambda res: self.dbpool.close())
@@ -282,7 +299,7 @@ class ReconnectTestBase:
         sql = "select count(1) from simple"
         d = defer.maybeDeferred(self.dbpool.runQuery, sql)
         d.addCallbacks(lambda res: self.fail('no exception'),
-                       lambda f: f.trap(ConnectionLost))
+                       lambda f: None)
         return d
 
     def _testPool_4(self, res):
@@ -294,6 +311,7 @@ class ReconnectTestBase:
         return d
 
     def _testPool_5(self, res):
+        self.flushLoggedErrors()
         sql = "select * from NOTABLE" # bad sql
         d = defer.maybeDeferred(self.dbpool.runQuery, sql)
         d.addCallbacks(lambda res: self.fail('no exception'),
@@ -326,18 +344,16 @@ class DBTestConnector:
     good_sql = ConnectionPool.good_sql
     early_reconnect = True # cursor() will fail on closed connection
     can_clear = True # can try to clear out tables when starting
-    needs_dbdir = False # if a temporary directory is needed for the db
 
     num_iterations = 50 # number of iterations for test loops
                         # (lower this for slow db's)
 
-    def setUpClass(self):
-        if self.needs_dbdir:
-            self.DB_DIR = tempfile.mktemp()
-            os.mkdir(self.DB_DIR)
-
+    def setUp(self):
+        self.DB_DIR = self.mktemp()
+        os.mkdir(self.DB_DIR)
         if not self.can_connect():
-            raise unittest.SkipTest, '%s: Cannot access db' % self.TEST_PREFIX
+            raise unittest.SkipTest('%s: Cannot access db' % self.TEST_PREFIX)
+        return self.extraSetUp()
 
     def can_connect(self):
         """Return true if this database is present on the system
@@ -372,7 +388,6 @@ class GadflyConnector(DBTestConnector):
     can_rollback = False
     escape_slashes = False
     good_sql = 'select * from simple where 1=0'
-    needs_dbdir = True
 
     num_iterations = 1 # slow
 
@@ -403,7 +418,6 @@ class SQLiteConnector(DBTestConnector):
     TEST_PREFIX = 'SQLite'
 
     escape_slashes = False
-    needs_dbdir = True
 
     num_iterations = 1 # slow
 
@@ -492,7 +506,6 @@ class FirebirdConnector(DBTestConnector):
     escape_slashes = False
     good_sql = None # firebird doesn't handle failed sql well
     can_clear = False # firebird is not so good
-    needs_dbdir = True
 
     num_iterations = 5 # slow
 
@@ -506,6 +519,7 @@ class FirebirdConnector(DBTestConnector):
         except:
             return False
 
+
     def startDB(self):
         import kinterbasdb
         self.DB_NAME = os.path.join(self.DB_DIR, DBTestConnector.DB_NAME)
@@ -515,6 +529,7 @@ class FirebirdConnector(DBTestConnector):
         conn = kinterbasdb.create_database(sql)
         conn.close()
 
+
     def getPoolArgs(self):
         args = ('kinterbasdb',)
         kw = {'database': self.DB_NAME, 'host': '127.0.0.1',
@@ -529,7 +544,8 @@ class FirebirdConnector(DBTestConnector):
         conn.drop_database()
 
 def makeSQLTests(base, suffix, globals):
-    """Make a test case for every db connector which can connect.
+    """
+    Make a test case for every db connector which can connect.
 
     @param base: Base class for test case. Additional base classes
                  will be a DBConnector subclass and unittest.TestCase
@@ -540,8 +556,8 @@ def makeSQLTests(base, suffix, globals):
                   PsycopgConnector, MySQLConnector, FirebirdConnector]
     for connclass in connectors:
         name = connclass.TEST_PREFIX + suffix
-        import new
-        klass = new.classobj(name, (connclass, base, unittest.TestCase), {})
+        klass = types.ClassType(name, (connclass, base, unittest.TestCase),
+                                base.__dict__)
         globals[name] = klass
 
 # GadflyADBAPITestCase SQLiteADBAPITestCase PyPgSQLADBAPITestCase
@@ -551,3 +567,271 @@ makeSQLTests(ADBAPITestBase, 'ADBAPITestCase', globals())
 # GadflyReconnectTestCase SQLiteReconnectTestCase PyPgSQLReconnectTestCase
 # PsycopgReconnectTestCase MySQLReconnectTestCase FirebirdReconnectTestCase
 makeSQLTests(ReconnectTestBase, 'ReconnectTestCase', globals())
+
+
+
+class DeprecationTestCase(unittest.TestCase):
+    """
+    Test deprecations in twisted.enterprise.adbapi
+    """
+
+    def test_safe(self):
+        """
+        Test deprecation of twisted.enterprise.adbapi.safe()
+        """
+        result = self.callDeprecated(_unreleasedVersion,
+                                     safe, "test'")
+
+        # make sure safe still behaves like the original
+        self.assertEqual(result, "test''")
+
+
+
+class FakePool(object):
+    """
+    A fake L{ConnectionPool} for tests.
+
+    @ivar connectionFactory: factory for making connections returned by the
+        C{connect} method.
+    @type connectionFactory: any callable
+    """
+    reconnect = True
+    noisy = True
+
+    def __init__(self, connectionFactory):
+        self.connectionFactory = connectionFactory
+
+
+    def connect(self):
+        """
+        Return an instance of C{self.connectionFactory}.
+        """
+        return self.connectionFactory()
+
+
+    def disconnect(self, connection):
+        """
+        Do nothing.
+        """
+
+
+
+class ConnectionTestCase(unittest.TestCase):
+    """
+    Tests for the L{Connection} class.
+    """
+
+    def test_rollbackErrorLogged(self):
+        """
+        If an error happens during rollback, L{ConnectionLost} is raised but
+        the original error is logged.
+        """
+        class ConnectionRollbackRaise(object):
+            def rollback(self):
+                raise RuntimeError("problem!")
+
+        pool = FakePool(ConnectionRollbackRaise)
+        connection = Connection(pool)
+        self.assertRaises(ConnectionLost, connection.rollback)
+        errors = self.flushLoggedErrors(RuntimeError)
+        self.assertEqual(len(errors), 1)
+        self.assertEqual(errors[0].value.args[0], "problem!")
+
+
+
+class TransactionTestCase(unittest.TestCase):
+    """
+    Tests for the L{Transaction} class.
+    """
+
+    def test_reopenLogErrorIfReconnect(self):
+        """
+        If the cursor creation raises an error in L{Transaction.reopen}, it
+        reconnects but log the error occurred.
+        """
+        class ConnectionCursorRaise(object):
+            count = 0
+
+            def reconnect(self):
+                pass
+
+            def cursor(self):
+                if self.count == 0:
+                    self.count += 1
+                    raise RuntimeError("problem!")
+
+        pool = FakePool(None)
+        transaction = Transaction(pool, ConnectionCursorRaise())
+        transaction.reopen()
+        errors = self.flushLoggedErrors(RuntimeError)
+        self.assertEqual(len(errors), 1)
+        self.assertEqual(errors[0].value.args[0], "problem!")
+
+
+
+class NonThreadPool(object):
+    def callInThreadWithCallback(self, onResult, f, *a, **kw):
+        success = True
+        try:
+            result = f(*a, **kw)
+        except Exception, e:
+            success = False
+            result = Failure()
+        onResult(success, result)
+
+
+
+class DummyConnectionPool(ConnectionPool):
+    """
+    A testable L{ConnectionPool};
+    """
+    threadpool = NonThreadPool()
+
+    def __init__(self):
+        """
+        Don't forward init call.
+        """
+        self.reactor = reactor
+
+
+
+class EventReactor(object):
+    """
+    Partial L{IReactorCore} implementation with simple event-related
+    methods.
+
+    @ivar _running: A C{bool} indicating whether the reactor is pretending
+        to have been started already or not.
+
+    @ivar triggers: A C{list} of pending system event triggers.
+    """
+    def __init__(self, running):
+        self._running = running
+        self.triggers = []
+
+
+    def callWhenRunning(self, function):
+        if self._running:
+            function()
+        else:
+            return self.addSystemEventTrigger('after', 'startup', function)
+
+
+    def addSystemEventTrigger(self, phase, event, trigger):
+        handle = (phase, event, trigger)
+        self.triggers.append(handle)
+        return handle
+
+
+    def removeSystemEventTrigger(self, handle):
+        self.triggers.remove(handle)
+
+
+
+class ConnectionPoolTestCase(unittest.TestCase):
+    """
+    Unit tests for L{ConnectionPool}.
+    """
+
+    def test_runWithConnectionRaiseOriginalError(self):
+        """
+        If rollback fails, L{ConnectionPool.runWithConnection} raises the
+        original exception and log the error of the rollback.
+        """
+        class ConnectionRollbackRaise(object):
+            def __init__(self, pool):
+                pass
+
+            def rollback(self):
+                raise RuntimeError("problem!")
+
+        def raisingFunction(connection):
+            raise ValueError("foo")
+
+        pool = DummyConnectionPool()
+        pool.connectionFactory = ConnectionRollbackRaise
+        d = pool.runWithConnection(raisingFunction)
+        d = self.assertFailure(d, ValueError)
+        def cbFailed(ignored):
+            errors = self.flushLoggedErrors(RuntimeError)
+            self.assertEqual(len(errors), 1)
+            self.assertEqual(errors[0].value.args[0], "problem!")
+        d.addCallback(cbFailed)
+        return d
+
+
+    def test_closeLogError(self):
+        """
+        L{ConnectionPool._close} logs exceptions.
+        """
+        class ConnectionCloseRaise(object):
+            def close(self):
+                raise RuntimeError("problem!")
+
+        pool = DummyConnectionPool()
+        pool._close(ConnectionCloseRaise())
+
+        errors = self.flushLoggedErrors(RuntimeError)
+        self.assertEqual(len(errors), 1)
+        self.assertEqual(errors[0].value.args[0], "problem!")
+
+
+    def test_runWithInteractionRaiseOriginalError(self):
+        """
+        If rollback fails, L{ConnectionPool.runInteraction} raises the
+        original exception and log the error of the rollback.
+        """
+        class ConnectionRollbackRaise(object):
+            def __init__(self, pool):
+                pass
+
+            def rollback(self):
+                raise RuntimeError("problem!")
+
+        class DummyTransaction(object):
+            def __init__(self, pool, connection):
+                pass
+
+        def raisingFunction(transaction):
+            raise ValueError("foo")
+
+        pool = DummyConnectionPool()
+        pool.connectionFactory = ConnectionRollbackRaise
+        pool.transactionFactory = DummyTransaction
+
+        d = pool.runInteraction(raisingFunction)
+        d = self.assertFailure(d, ValueError)
+        def cbFailed(ignored):
+            errors = self.flushLoggedErrors(RuntimeError)
+            self.assertEqual(len(errors), 1)
+            self.assertEqual(errors[0].value.args[0], "problem!")
+        d.addCallback(cbFailed)
+        return d
+
+
+    def test_unstartedClose(self):
+        """
+        If L{ConnectionPool.close} is called without L{ConnectionPool.start}
+        having been called, the pool's startup event is cancelled.
+        """
+        reactor = EventReactor(False)
+        pool = ConnectionPool('twisted.test.test_adbapi', cp_reactor=reactor)
+        # There should be a startup trigger waiting.
+        self.assertEqual(reactor.triggers, [('after', 'startup', pool._start)])
+        pool.close()
+        # But not anymore.
+        self.assertFalse(reactor.triggers)
+
+
+    def test_startedClose(self):
+        """
+        If L{ConnectionPool.close} is called after it has been started, but
+        not by its shutdown trigger, the shutdown trigger is cancelled.
+        """
+        reactor = EventReactor(True)
+        pool = ConnectionPool('twisted.test.test_adbapi', cp_reactor=reactor)
+        # There should be a shutdown trigger waiting.
+        self.assertEqual(reactor.triggers, [('during', 'shutdown', pool.finalClose)])
+        pool.close()
+        # But not anymore.
+        self.assertFalse(reactor.triggers)
