@@ -1,32 +1,24 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-import sys, copy, os, pickle, warnings
-from StringIO import StringIO
+"""
+Tests for L{twisted.application} and its interaction with
+L{twisted.persisted.sob}.
+"""
 
+import copy, os, pickle
+from StringIO import StringIO
 
 from twisted.trial import unittest, util
 from twisted.application import service, internet, app
 from twisted.persisted import sob
 from twisted.python import usage
-from twisted.python.util import sibpath
 from twisted.internet import interfaces, defer
 from twisted.protocols import wire, basic
 from twisted.internet import protocol, reactor
-from twisted.internet.utils import getProcessOutputAndValue
 from twisted.application import reactors
+from twisted.test.proto_helpers import MemoryReactor
 
-try:
-    from twisted.web import microdom
-    gotMicrodom = True
-except ImportError:
-    warnings.warn("Not testing xml persistence as twisted.web.microdom "
-                  "not available")
-    gotMicrodom = False
-
-
-oldAppSuppressions = [util.suppress(message='twisted.internet.app is deprecated',
-                                    category=DeprecationWarning)]
 
 class Dummy:
     processName=None
@@ -36,30 +28,30 @@ class TestService(unittest.TestCase):
     def testName(self):
         s = service.Service()
         s.setName("hello")
-        self.failUnlessEqual(s.name, "hello")
+        self.assertEqual(s.name, "hello")
 
     def testParent(self):
         s = service.Service()
         p = service.MultiService()
         s.setServiceParent(p)
-        self.failUnlessEqual(list(p), [s])
-        self.failUnlessEqual(s.parent, p)
+        self.assertEqual(list(p), [s])
+        self.assertEqual(s.parent, p)
 
     def testApplicationAsParent(self):
         s = service.Service()
         p = service.Application("")
         s.setServiceParent(p)
-        self.failUnlessEqual(list(service.IServiceCollection(p)), [s])
-        self.failUnlessEqual(s.parent, service.IServiceCollection(p))
+        self.assertEqual(list(service.IServiceCollection(p)), [s])
+        self.assertEqual(s.parent, service.IServiceCollection(p))
 
     def testNamedChild(self):
         s = service.Service()
         p = service.MultiService()
         s.setName("hello")
         s.setServiceParent(p)
-        self.failUnlessEqual(list(p), [s])
-        self.failUnlessEqual(s.parent, p)
-        self.failUnlessEqual(p.getServiceNamed("hello"), s)
+        self.assertEqual(list(p), [s])
+        self.assertEqual(s.parent, p)
+        self.assertEqual(p.getServiceNamed("hello"), s)
 
     def testDoublyNamedChild(self):
         s = service.Service()
@@ -81,11 +73,11 @@ class TestService(unittest.TestCase):
         s = service.Service()
         p = service.MultiService()
         s.setServiceParent(p)
-        self.failUnlessEqual(list(p), [s])
-        self.failUnlessEqual(s.parent, p)
+        self.assertEqual(list(p), [s])
+        self.assertEqual(s.parent, p)
         s.disownServiceParent()
-        self.failUnlessEqual(list(p), [])
-        self.failUnlessEqual(s.parent, None)
+        self.assertEqual(list(p), [])
+        self.assertEqual(s.parent, None)
 
     def testRunning(self):
         s = service.Service()
@@ -232,9 +224,7 @@ class TestLoading(unittest.TestCase):
     def test_simpleStoreAndLoad(self):
         a = service.Application("hello")
         p = sob.IPersistable(a)
-        for style in 'xml source pickle'.split():
-            if style == 'xml' and not gotMicrodom:
-                continue
+        for style in 'source pickle'.split():
             p.setStyle(style)
             p.save()
             a1 = service.loadApplication("hello.ta"+style[0], style)
@@ -260,10 +250,8 @@ class TestAppSupport(unittest.TestCase):
         Test loading an application file in different dump format.
         """
         a = service.Application("hello")
-        baseconfig = {'file': None, 'xml': None, 'source': None, 'python':None}
-        for style in 'source xml pickle'.split():
-            if style == 'xml' and not gotMicrodom:
-                continue
+        baseconfig = {'file': None, 'source': None, 'python':None}
+        for style in 'source pickle'.split():
             config = baseconfig.copy()
             config[{'pickle': 'file'}.get(style, style)] = 'helloapplication'
             sob.IPersistable(a).setStyle(style)
@@ -281,18 +269,10 @@ class TestAppSupport(unittest.TestCase):
         a1 = app.getApplication(config, None)
         self.assertEqual(service.IService(a1).name, "hello")
 
-    testLoadApplication.suppress = [
-        util.suppress(message="twisted.persisted.marmalade is deprecated",
-                      category=DeprecationWarning)]
-
     def test_convertStyle(self):
         appl = service.Application("lala")
-        for instyle in 'xml source pickle'.split():
-            if instyle == 'xml' and not gotMicrodom:
-                continue
-            for outstyle in 'xml source pickle'.split():
-                if outstyle == 'xml' and not gotMicrodom:
-                    continue
+        for instyle in 'source pickle'.split():
+            for outstyle in 'source pickle'.split():
                 sob.IPersistable(appl).setStyle(instyle)
                 sob.IPersistable(appl).save(filename="converttest")
                 app.convertStyle("converttest", instyle, None,
@@ -300,21 +280,6 @@ class TestAppSupport(unittest.TestCase):
                 appl2 = service.loadApplication("converttest.out", outstyle)
                 self.assertEqual(service.IService(appl2).name, "lala")
 
-    def test_getLogFile(self):
-        """
-        Test L{app.getLogFile}, veryfying the LogFile instance it returns.
-        """
-        os.mkdir("logfiledir")
-        l = app.getLogFile(os.path.join("logfiledir", "lala"))
-        self.assertEqual(l.path,
-                         os.path.abspath(os.path.join("logfiledir", "lala")))
-        self.assertEqual(l.name, "lala")
-        self.assertEqual(l.directory, os.path.abspath("logfiledir"))
-
-    test_getLogFile.suppress = [
-        util.suppress(message="app.getLogFile is deprecated. Use "
-                      "twisted.python.logfile.LogFile.fromFullPath instead",
-                      category=DeprecationWarning)]
 
     def test_startApplication(self):
         appl = service.Application("lala")
@@ -524,10 +489,44 @@ class TestInternet2(unittest.TestCase):
             [o.value.__class__ for o in self.flushLoggedErrors(ZeroDivisionError)]))
         return d
 
-    def testEverythingThere(self):
+
+    def test_genericServerDeprecated(self):
+        """
+        Instantiating L{GenericServer} emits a deprecation warning.
+        """
+        internet.GenericServer()
+        warnings = self.flushWarnings(
+            offendingFunctions=[self.test_genericServerDeprecated])
+        self.assertEqual(
+            warnings[0]['message'],
+            'GenericServer was deprecated in Twisted 10.1.')
+        self.assertEqual(warnings[0]['category'], DeprecationWarning)
+        self.assertEqual(len(warnings), 1)
+
+
+    def test_genericClientDeprecated(self):
+        """
+        Instantiating L{GenericClient} emits a deprecation warning.
+        """
+        internet.GenericClient()
+        warnings = self.flushWarnings(
+            offendingFunctions=[self.test_genericClientDeprecated])
+        self.assertEqual(
+            warnings[0]['message'],
+            'GenericClient was deprecated in Twisted 10.1.')
+        self.assertEqual(warnings[0]['category'], DeprecationWarning)
+        self.assertEqual(len(warnings), 1)
+
+
+    def test_everythingThere(self):
+        """
+        L{twisted.application.internet} dynamically defines a set of
+        L{service.Service} subclasses that in general have corresponding
+        reactor.listenXXX or reactor.connectXXX calls.
+        """
         trans = 'TCP UNIX SSL UDP UNIXDatagram Multicast'.split()
         for tran in trans[:]:
-            if not getattr(interfaces, "IReactor"+tran)(reactor, None):
+            if not getattr(interfaces, "IReactor" + tran)(reactor, None):
                 trans.remove(tran)
         if interfaces.IReactorArbitrary(reactor, None) is not None:
             trans.insert(0, "Generic")
@@ -535,13 +534,32 @@ class TestInternet2(unittest.TestCase):
             for side in 'Server Client'.split():
                 if tran == "Multicast" and side == "Client":
                     continue
-                self.assert_(hasattr(internet, tran+side))
-                method = getattr(internet, tran+side).method
+                self.assertTrue(hasattr(internet, tran + side))
+                method = getattr(internet, tran + side).method
                 prefix = {'Server': 'listen', 'Client': 'connect'}[side]
-                self.assert_(hasattr(reactor, prefix+method) or
+                self.assertTrue(hasattr(reactor, prefix + method) or
                         (prefix == "connect" and method == "UDP"))
-                o = getattr(internet, tran+side)()
+                o = getattr(internet, tran + side)()
                 self.assertEqual(service.IService(o), o)
+    test_everythingThere.suppress = [
+        util.suppress(message='GenericServer was deprecated in Twisted 10.1.',
+                      category=DeprecationWarning),
+        util.suppress(message='GenericClient was deprecated in Twisted 10.1.',
+                      category=DeprecationWarning),
+        util.suppress(message='twisted.internet.interfaces.IReactorArbitrary was '
+                      'deprecated in Twisted 10.1.0: See IReactorFDSet.')]
+
+
+    def test_importAll(self):
+        """
+        L{twisted.application.internet} dynamically defines L{service.Service}
+        subclasses. This test ensures that the subclasses exposed by C{__all__}
+        are valid attributes of the module.
+        """
+        for cls in internet.__all__:
+            self.assertTrue(
+                hasattr(internet, cls),
+                '%s not importable from twisted.application.internet' % (cls,))
 
 
     def test_reactorParametrizationInServer(self):
@@ -550,16 +568,12 @@ class TestInternet2(unittest.TestCase):
         that can be used to parametrize the reactor used to listen for
         connections.
         """
-        listen = []
-        class FakeReactor(object):
-            def listenTCP(self, portNumber, factory):
-                listen.append((portNumber, factory))
-        reactor = FakeReactor()
+        reactor = MemoryReactor()
 
         factory = object()
         t = internet.TCPServer(1234, factory, reactor=reactor)
         t.startService()
-        self.assertEquals(listen, [(1234, factory)])
+        self.assertEqual(reactor.tcpServers.pop()[:2], (1234, factory))
 
 
     def test_reactorParametrizationInClient(self):
@@ -568,16 +582,13 @@ class TestInternet2(unittest.TestCase):
         that can be used to parametrize the reactor used to create new client
         connections.
         """
-        connect = []
-        class FakeReactor(object):
-            def connectTCP(self, ip, portNumber, factory):
-                connect.append((ip, portNumber, factory))
-        reactor = FakeReactor()
+        reactor = MemoryReactor()
 
         factory = object()
         t = internet.TCPClient('127.0.0.1', 1234, factory, reactor=reactor)
         t.startService()
-        self.assertEquals(connect, [('127.0.0.1', 1234, factory)])
+        self.assertEqual(
+            reactor.tcpClients.pop()[:3], ('127.0.0.1', 1234, factory))
 
 
     def test_reactorParametrizationInServerMultipleStart(self):
@@ -585,19 +596,15 @@ class TestInternet2(unittest.TestCase):
         Like L{test_reactorParametrizationInServer}, but stop and restart the
         service and check that the given reactor is still used.
         """
-        listen = []
-        class FakeReactor(object):
-            def listenTCP(self, portNumber, factory):
-                listen.append((portNumber, factory))
-        reactor = FakeReactor()
+        reactor = MemoryReactor()
 
         factory = object()
         t = internet.TCPServer(1234, factory, reactor=reactor)
         t.startService()
-        self.assertEquals(listen, [(1234, factory)])
+        self.assertEqual(reactor.tcpServers.pop()[:2], (1234, factory))
         t.stopService()
         t.startService()
-        self.assertEquals(listen, [(1234, factory), (1234, factory)])
+        self.assertEqual(reactor.tcpServers.pop()[:2], (1234, factory))
 
 
     def test_reactorParametrizationInClientMultipleStart(self):
@@ -605,20 +612,17 @@ class TestInternet2(unittest.TestCase):
         Like L{test_reactorParametrizationInClient}, but stop and restart the
         service and check that the given reactor is still used.
         """
-        connect = []
-        class FakeReactor(object):
-            def connectTCP(self, ip, portNumber, factory):
-                connect.append((ip, portNumber, factory))
-        reactor = FakeReactor()
+        reactor = MemoryReactor()
 
         factory = object()
         t = internet.TCPClient('127.0.0.1', 1234, factory, reactor=reactor)
         t.startService()
-        self.assertEquals(connect, [('127.0.0.1', 1234, factory)])
+        self.assertEqual(
+            reactor.tcpClients.pop()[:3], ('127.0.0.1', 1234, factory))
         t.stopService()
         t.startService()
-        self.assertEquals(connect, [('127.0.0.1', 1234, factory),
-                                    ('127.0.0.1', 1234, factory)])
+        self.assertEqual(
+            reactor.tcpClients.pop()[:3], ('127.0.0.1', 1234, factory))
 
 
 
@@ -828,6 +832,7 @@ class PluggableReactorTestCase(unittest.TestCase):
         options.parseOptions(['--reactor', 'fakereactortest', 'subcommand'])
         self.assertEqual(executed[0], INSTALL_EVENT)
         self.assertEqual(executed.count(INSTALL_EVENT), 1)
+        self.assertEqual(options["reactor"], "fakereactortest")
 
 
     def test_reactorSelectionMixinNonExistent(self):
@@ -871,33 +876,3 @@ class PluggableReactorTestCase(unittest.TestCase):
                                ['--reactor', 'fakereactortest', 'subcommand'])
         self.assertIn(message, e.args[0])
         self.assertIn("help-reactors", e.args[0])
-
-
-    def test_qtStub(self):
-        """
-        Test that installing qtreactor when it's absent fails properly.
-        """
-        scriptPath = sibpath(__file__, "app_qtstub.py")
-        def _checkOutput((output, err, code)):
-            self.failIf(output, output)
-        result = getProcessOutputAndValue(
-            sys.executable,
-            args=(sys.executable, scriptPath),
-            env=None)
-        result.addCallback(_checkOutput)
-        return result
-
-
-
-class ReportProfileTestCase(unittest.TestCase):
-    """
-    Tests for L{app.reportProfile}.
-    """
-
-    def test_deprecation(self):
-        """
-        Check that L{app.reportProfile} prints a warning and does nothing else.
-        """
-        self.assertWarns(DeprecationWarning,
-            "reportProfile is deprecated and a no-op since Twisted 8.0.",
-            app.__file__, app.reportProfile, None, None)
