@@ -163,6 +163,59 @@ class WrapperTestCase(unittest.TestCase):
         self.assertTrue(IStubTransport.providedBy(proto.transport))
 
 
+    def test_factoryLogPrefix(self):
+        """
+        L{WrappingFactory.logPrefix} is customized to mention both the original
+        factory and the wrapping factory.
+        """
+        server = Server()
+        factory = policies.WrappingFactory(server)
+        self.assertEqual("Server (WrappingFactory)", factory.logPrefix())
+
+
+    def test_factoryLogPrefixFallback(self):
+        """
+        If the wrapped factory doesn't have a L{logPrefix} method,
+        L{WrappingFactory.logPrefix} falls back to the factory class name.
+        """
+        class NoFactory(object):
+            pass
+
+        server = NoFactory()
+        factory = policies.WrappingFactory(server)
+        self.assertEqual("NoFactory (WrappingFactory)", factory.logPrefix())
+
+
+    def test_protocolLogPrefix(self):
+        """
+        L{ProtocolWrapper.logPrefix} is customized to mention both the original
+        protocol and the wrapper.
+        """
+        server = Server()
+        factory = policies.WrappingFactory(server)
+        protocol = factory.buildProtocol(
+            address.IPv4Address('TCP', '127.0.0.1', 35))
+        self.assertEqual("EchoProtocol (ProtocolWrapper)",
+                         protocol.logPrefix())
+
+
+    def test_protocolLogPrefixFallback(self):
+        """
+        If the wrapped protocol doesn't have a L{logPrefix} method,
+        L{ProtocolWrapper.logPrefix} falls back to the protocol class name.
+        """
+        class NoProtocol(object):
+            pass
+
+        server = Server()
+        server.protocol = NoProtocol
+        factory = policies.WrappingFactory(server)
+        protocol = factory.buildProtocol(
+            address.IPv4Address('TCP', '127.0.0.1', 35))
+        self.assertEqual("NoProtocol (ProtocolWrapper)",
+                         protocol.logPrefix())
+
+
 
 class WrappingFactory(policies.WrappingFactory):
     protocol = lambda s, f, p: p
@@ -201,9 +254,9 @@ class ThrottlingTestCase(unittest.TestCase):
             return c3.dDisconnected
 
         def _check123(results):
-            self.assertEquals([c.connected for c in c1, c2, c3], [1, 1, 1])
-            self.assertEquals([c.disconnected for c in c1, c2, c3], [0, 0, 1])
-            self.assertEquals(len(tServer.protocols.keys()), 2)
+            self.assertEqual([c.connected for c in c1, c2, c3], [1, 1, 1])
+            self.assertEqual([c.disconnected for c in c1, c2, c3], [0, 0, 1])
+            self.assertEqual(len(tServer.protocols.keys()), 2)
             return results
 
         def _lose1(results):
@@ -216,8 +269,8 @@ class ThrottlingTestCase(unittest.TestCase):
             return c4.dConnected
 
         def _check4(results):
-            self.assertEquals(c4.connected, 1)
-            self.assertEquals(c4.disconnected, 0)
+            self.assertEqual(c4.connected, 1)
+            self.assertEqual(c4.disconnected, 0)
             return results
 
         def _cleanup(results):
@@ -252,19 +305,19 @@ class ThrottlingTestCase(unittest.TestCase):
 
         port.dataReceived("0123456789")
         port.dataReceived("abcdefghij")
-        self.assertEquals(tr.value(), "0123456789abcdefghij")
-        self.assertEquals(tServer.writtenThisSecond, 20)
+        self.assertEqual(tr.value(), "0123456789abcdefghij")
+        self.assertEqual(tServer.writtenThisSecond, 20)
         self.assertFalse(port.wrappedProtocol.paused)
 
         # at this point server should've written 20 bytes, 10 bytes
         # above the limit so writing should be paused around 1 second
         # from 'now', and resumed a second after that
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.writtenThisSecond, 0)
+        self.assertEqual(tServer.writtenThisSecond, 0)
         self.assertTrue(port.wrappedProtocol.paused)
 
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.writtenThisSecond, 0)
+        self.assertEqual(tServer.writtenThisSecond, 0)
         self.assertFalse(port.wrappedProtocol.paused)
 
 
@@ -282,30 +335,30 @@ class ThrottlingTestCase(unittest.TestCase):
 
         port.dataReceived("0123456789")
         port.dataReceived("abcdefghij")
-        self.assertEquals(tr.value(), "0123456789abcdefghij")
-        self.assertEquals(tServer.readThisSecond, 20)
+        self.assertEqual(tr.value(), "0123456789abcdefghij")
+        self.assertEqual(tServer.readThisSecond, 20)
 
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.readThisSecond, 0)
-        self.assertEquals(tr.producerState, 'paused')
+        self.assertEqual(tServer.readThisSecond, 0)
+        self.assertEqual(tr.producerState, 'paused')
 
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.readThisSecond, 0)
-        self.assertEquals(tr.producerState, 'producing')
+        self.assertEqual(tServer.readThisSecond, 0)
+        self.assertEqual(tr.producerState, 'producing')
 
         tr.clear()
         port.dataReceived("0123456789")
         port.dataReceived("abcdefghij")
-        self.assertEquals(tr.value(), "0123456789abcdefghij")
-        self.assertEquals(tServer.readThisSecond, 20)
+        self.assertEqual(tr.value(), "0123456789abcdefghij")
+        self.assertEqual(tServer.readThisSecond, 20)
 
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.readThisSecond, 0)
-        self.assertEquals(tr.producerState, 'paused')
+        self.assertEqual(tServer.readThisSecond, 0)
+        self.assertEqual(tr.producerState, 'paused')
 
         tServer.clock.advance(1.05)
-        self.assertEquals(tServer.readThisSecond, 0)
-        self.assertEquals(tr.producerState, 'producing')
+        self.assertEqual(tServer.readThisSecond, 0)
+        self.assertEqual(tr.producerState, 'producing')
 
 
 
@@ -469,7 +522,7 @@ class TestTimeout(unittest.TestCase):
         C{reactor.callLater}.
         """
         self.proto.setTimeout(10)
-        self.assertEquals(len(self.clock.calls), 1)
+        self.assertEqual(len(self.clock.calls), 1)
 
 
     def test_timeout(self):
@@ -510,7 +563,7 @@ class TestTimeout(unittest.TestCase):
         self.proto.makeConnection(StringTransport())
 
         self.proto.setTimeout(1)
-        self.assertEquals(self.proto.timeOut, 1)
+        self.assertEqual(self.proto.timeOut, 1)
 
         self.clock.pump([0, 0.9])
         self.failIf(self.proto.timedOut)
@@ -526,7 +579,7 @@ class TestTimeout(unittest.TestCase):
         self.proto.makeConnection(StringTransport())
 
         self.proto.setTimeout(None)
-        self.assertEquals(self.proto.timeOut, None)
+        self.assertEqual(self.proto.timeOut, None)
 
         self.clock.pump([0, 5, 5, 5])
         self.failIf(self.proto.timedOut)
@@ -538,10 +591,10 @@ class TestTimeout(unittest.TestCase):
         """
         self.proto.timeOut = 5
 
-        self.assertEquals(self.proto.setTimeout(10), 5)
-        self.assertEquals(self.proto.setTimeout(None), 10)
-        self.assertEquals(self.proto.setTimeout(1), None)
-        self.assertEquals(self.proto.timeOut, 1)
+        self.assertEqual(self.proto.setTimeout(10), 5)
+        self.assertEqual(self.proto.setTimeout(None), 10)
+        self.assertEqual(self.proto.setTimeout(1), None)
+        self.assertEqual(self.proto.timeOut, 1)
 
         # Clean up the DelayedCall
         self.proto.setTimeout(None)
@@ -650,13 +703,13 @@ class LoggingFactoryTestCase(unittest.TestCase):
         v = f.openFile.getvalue()
         self.assertIn("C 1: 'here are some bytes'", v)
         self.assertIn("S 1: 'here are some bytes'", v)
-        self.assertEquals(t.value(), 'here are some bytes')
+        self.assertEqual(t.value(), 'here are some bytes')
 
         t.clear()
         p.dataReceived('prepare for vector! to the extreme')
         v = f.openFile.getvalue()
         self.assertIn("SV 1: ['prepare for vector! to the extreme']", v)
-        self.assertEquals(t.value(), 'prepare for vector! to the extreme')
+        self.assertEqual(t.value(), 'prepare for vector! to the extreme')
 
         p.loseConnection()
 
