@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -7,13 +7,13 @@ Test running processes.
 
 import gzip
 import os
-import popen2
 import sys
 import signal
 import StringIO
 import errno
 import gc
 import stat
+import operator
 try:
     import fcntl
 except ImportError:
@@ -494,7 +494,7 @@ class ProcessTestCase(unittest.TestCase):
         p.transport.closeStdin()
 
         def processEnded(ign):
-            self.assertEquals(p.outF.getvalue(), "hello, worldabc123",
+            self.assertEqual(p.outF.getvalue(), "hello, worldabc123",
                               "Output follows:\n"
                               "%s\n"
                               "Error message from process_twisted follows:\n"
@@ -534,12 +534,12 @@ class ProcessTestCase(unittest.TestCase):
         p.deferred = d
         reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
         def check(ignored):
-            self.assertEquals(p.stages, [1, 2, 3, 4, 5])
+            self.assertEqual(p.stages, [1, 2, 3, 4, 5])
             f = p.reason
             f.trap(error.ProcessTerminated)
-            self.assertEquals(f.value.exitCode, 23)
+            self.assertEqual(f.value.exitCode, 23)
             # would .signal be available on non-posix?
-            # self.assertEquals(f.value.signal, None)
+            # self.assertEqual(f.value.signal, None)
             self.assertRaises(
                 error.ProcessExitedAlready, p.transport.signalProcess, 'INT')
             try:
@@ -555,11 +555,11 @@ class ProcessTestCase(unittest.TestCase):
 
         def _check(results, protocols):
             for p in protocols:
-                self.assertEquals(p.stages, [1, 2, 3, 4, 5], "[%d] stages = %s" % (id(p.transport), str(p.stages)))
+                self.assertEqual(p.stages, [1, 2, 3, 4, 5], "[%d] stages = %s" % (id(p.transport), str(p.stages)))
                 # test status code
                 f = p.reason
                 f.trap(error.ProcessTerminated)
-                self.assertEquals(f.value.exitCode, 23)
+                self.assertEqual(f.value.exitCode, 23)
 
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_tester.py")
@@ -594,7 +594,7 @@ class ProcessTestCase(unittest.TestCase):
         def asserts(ignored):
             self.failIf(p.failure, p.failure)
             self.failUnless(hasattr(p, 'buffer'))
-            self.assertEquals(len(''.join(p.buffer)), len(p.s * p.n))
+            self.assertEqual(len(''.join(p.buffer)), len(p.s * p.n))
 
         def takedownProcess(err):
             p.transport.closeStdin()
@@ -613,9 +613,9 @@ class ProcessTestCase(unittest.TestCase):
                              path=None)
 
         def processEnded(ign):
-            self.assertEquals(p.errF.getvalue(), "")
+            self.assertEqual(p.errF.getvalue(), "")
             recvdArgs = p.outF.getvalue().splitlines()
-            self.assertEquals(recvdArgs, args)
+            self.assertEqual(recvdArgs, args)
         return d.addCallback(processEnded)
 
 
@@ -953,7 +953,7 @@ class FDTest(unittest.TestCase):
                              childFDs={1:"r", 2:2},
                              )
         def processEnded(ign):
-            self.failUnlessEqual(p.outF.getvalue(),
+            self.assertEqual(p.outF.getvalue(),
                                  "here is some text\ngoodbye\n")
         return d.addCallback(processEnded)
 
@@ -1017,8 +1017,8 @@ class PosixProcessBase:
                              usePTY=self.usePTY)
         def check(ignored):
             p.reason.trap(error.ProcessDone)
-            self.assertEquals(p.reason.value.exitCode, 0)
-            self.assertEquals(p.reason.value.signal, None)
+            self.assertEqual(p.reason.value.exitCode, 0)
+            self.assertEqual(p.reason.value.signal, None)
         d.addCallback(check)
         return d
 
@@ -1038,8 +1038,8 @@ class PosixProcessBase:
 
         def check(ignored):
             p.reason.trap(error.ProcessTerminated)
-            self.assertEquals(p.reason.value.exitCode, 1)
-            self.assertEquals(p.reason.value.signal, None)
+            self.assertEqual(p.reason.value.exitCode, 1)
+            self.assertEqual(p.reason.value.signal, None)
         d.addCallback(check)
         return d
 
@@ -1311,6 +1311,13 @@ class MockOS(object):
         return -3
 
 
+    def fstat(self, fd):
+        """
+        Fake C{os.fstat}.  Return a C{os.stat_result} filled with garbage.
+        """
+        return os.stat_result((0,) * 10)
+
+
     def fdopen(self, fd, flag):
         """
         Fake C{os.fdopen}. Return a StringIO object whose content can be tested
@@ -1527,6 +1534,12 @@ class MockOS(object):
         """
         return 0, 0, 1, 2
 
+    def listdir(self, path):
+        """
+        Override C{os.listdir}, returning fake contents of '/dev/fd'
+        """
+        return "-1", "-2"
+
 
 
 if process is not None:
@@ -1621,7 +1634,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=False)
         except SystemError:
             self.assert_(self.mockos.exited)
-            self.assertEquals(
+            self.assertEqual(
                 self.mockos.actions, [("fork", False), "exec", "exit"])
         else:
             self.fail("Should not be here")
@@ -1645,7 +1658,7 @@ class MockProcessTestCase(unittest.TestCase):
                              usePTY=False)
         # It should close the first read pipe, and the 2 last write pipes
         self.assertEqual(set(self.mockos.closed), set([-1, -4, -6]))
-        self.assertEquals(self.mockos.actions, [("fork", False), "waitpid"])
+        self.assertEqual(self.mockos.actions, [("fork", False), "waitpid"])
 
 
     def test_mockForkInParentGarbageCollectorEnabled(self):
@@ -1686,7 +1699,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=True)
         except SystemError:
             self.assert_(self.mockos.exited)
-            self.assertEquals(
+            self.assertEqual(
                 self.mockos.actions, [("fork", False), "exec", "exit"])
         else:
             self.fail("Should not be here")
@@ -1796,7 +1809,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=False)
         except SystemError:
             self.assert_(self.mockos.exited)
-            self.assertEquals(
+            self.assertEqual(
                 self.mockos.actions, [("fork", False), "exec", "exit"])
             # Check that fd have been closed
             self.assertIn(0, self.mockos.closed)
@@ -1822,7 +1835,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=False, uid=8080)
         except SystemError:
             self.assert_(self.mockos.exited)
-            self.assertEquals(self.mockos.actions,
+            self.assertEqual(self.mockos.actions,
                 [('setuid', 0), ('setgid', 0), ('fork', False),
                   ('switchuid', 8080, 1234), 'exec', 'exit'])
         else:
@@ -1841,7 +1854,7 @@ class MockProcessTestCase(unittest.TestCase):
         p = TrivialProcessProtocol(d)
         reactor.spawnProcess(p, cmd, ['ouch'], env=None,
                              usePTY=False, uid=8080)
-        self.assertEquals(self.mockos.actions,
+        self.assertEqual(self.mockos.actions,
             [('setuid', 0), ('setgid', 0), ('fork', False),
              ('setregid', 1235, 1234), ('setreuid', 1237, 1236), 'waitpid'])
 
@@ -1861,7 +1874,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=True, uid=8081)
         except SystemError:
             self.assert_(self.mockos.exited)
-            self.assertEquals(self.mockos.actions,
+            self.assertEqual(self.mockos.actions,
                 [('setuid', 0), ('setgid', 0), ('fork', False),
                   ('switchuid', 8081, 1234), 'exec', 'exit'])
         else:
@@ -1885,7 +1898,7 @@ class MockProcessTestCase(unittest.TestCase):
                                  usePTY=True, uid=8080)
         finally:
             process.PTYProcess = oldPTYProcess
-        self.assertEquals(self.mockos.actions,
+        self.assertEqual(self.mockos.actions,
             [('setuid', 0), ('setgid', 0), ('fork', False),
              ('setregid', 1235, 1234), ('setreuid', 1237, 1236), 'waitpid'])
 
@@ -1902,12 +1915,12 @@ class MockProcessTestCase(unittest.TestCase):
         p = TrivialProcessProtocol(d)
         proc = reactor.spawnProcess(p, cmd, ['ouch'], env=None,
                              usePTY=False)
-        self.assertEquals(self.mockos.actions, [("fork", False), "waitpid"])
+        self.assertEqual(self.mockos.actions, [("fork", False), "waitpid"])
 
         self.mockos.raiseWaitPid = OSError()
         proc.reapProcess()
         errors = self.flushLoggedErrors()
-        self.assertEquals(len(errors), 1)
+        self.assertEqual(len(errors), 1)
         errors[0].trap(OSError)
 
 
@@ -1924,7 +1937,7 @@ class MockProcessTestCase(unittest.TestCase):
         p = TrivialProcessProtocol(d)
         proc = reactor.spawnProcess(p, cmd, ['ouch'], env=None,
                                     usePTY=False)
-        self.assertEquals(self.mockos.actions, [("fork", False), "waitpid"])
+        self.assertEqual(self.mockos.actions, [("fork", False), "waitpid"])
 
         self.mockos.raiseWaitPid = OSError()
         self.mockos.raiseWaitPid.errno = errno.ECHILD
@@ -1968,22 +1981,29 @@ class MockProcessTestCase(unittest.TestCase):
 class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
     # add two non-pty test cases
 
-    def testStderr(self):
-        # we assume there is no file named ZZXXX..., both in . and in /tmp
-        cmd = self.getCommand('ls')
+    def test_stderr(self):
+        """
+        Bytes written to stderr by the spawned process are passed to the
+        C{errReceived} callback on the C{ProcessProtocol} passed to
+        C{spawnProcess}.
+        """
+        cmd = sys.executable
+
+        value = "42"
 
         p = Accumulator()
         d = p.endedDeferred = defer.Deferred()
         reactor.spawnProcess(p, cmd,
-                             [cmd,
-                              "ZZXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"],
+                             [cmd, "-c", 
+                              "import sys; sys.stderr.write('%s')" % (value,)],
                              env=None, path="/tmp",
                              usePTY=self.usePTY)
 
         def processEnded(ign):
-            self.assertEquals(lsOut, p.errF.getvalue())
+            self.assertEqual(value, p.errF.getvalue())
         return d.addCallback(processEnded)
 
+
     def testProcess(self):
         cmd = self.getCommand('gzip')
         s = "there's no place like home!\n" * 3
@@ -1998,7 +2018,7 @@ class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
             f = p.outF
             f.seek(0, 0)
             gf = gzip.GzipFile(fileobj=f)
-            self.assertEquals(gf.read(), s)
+            self.assertEqual(gf.read(), s)
         return d.addCallback(processEnded)
 
 
@@ -2027,7 +2047,7 @@ class PosixProcessTestCasePTY(unittest.TestCase, PosixProcessBase):
         def processEnded(ign):
             self.assertRaises(
                 error.ProcessExitedAlready, p.transport.signalProcess, 'HUP')
-            self.assertEquals(
+            self.assertEqual(
                 p.outF.getvalue(),
                 "hello world!\r\nhello world!\r\n",
                 "Error message from process_tty follows:\n\n%s\n\n" % p.outF.getvalue())
@@ -2082,8 +2102,8 @@ class Win32ProcessTestCase(unittest.TestCase):
         p.transport.closeStdin()
 
         def processEnded(ign):
-            self.assertEquals(p.errF.getvalue(), "err\nerr\n")
-            self.assertEquals(p.outF.getvalue(), "out\nhello, world\nout\n")
+            self.assertEqual(p.errF.getvalue(), "err\nerr\n")
+            self.assertEqual(p.outF.getvalue(), "out\nhello, world\nout\n")
         return d.addCallback(processEnded)
 
 
@@ -2188,6 +2208,32 @@ class Win32ProcessTestCase(unittest.TestCase):
 
 
 
+class Win32UnicodeEnvironmentTest(unittest.TestCase):
+    """
+    Tests for Unicode environment on Windows
+    """
+    goodKey = u'UNICODE'
+    goodValue = u'UNICODE'
+
+    def test_encodableUnicodeEnvironment(self):
+        """
+        Test C{os.environ} (inherited by every subprocess on Windows) that
+        contains an ascii-encodable Unicode string. This is different from
+        passing Unicode environment explicitly to spawnProcess (which is not
+        supported).
+        """
+        os.environ[self.goodKey] = self.goodValue
+        self.addCleanup(operator.delitem, os.environ, self.goodKey)
+
+        p = GetEnvironmentDictionary.run(reactor, [], {})
+        def gotEnvironment(environ):
+            self.assertEqual(
+                environ[self.goodKey.encode('ascii')],
+                self.goodValue.encode('ascii'))
+        return p.getResult().addCallback(gotEnvironment)
+
+
+
 class Dumbwin32procPidTest(unittest.TestCase):
     """
     Simple test for the pid attribute of Process on win32.
@@ -2215,11 +2261,11 @@ class Dumbwin32procPidTest(unittest.TestCase):
                                   cmd,
                                   {},
                                   None)
-        self.assertEquals(42, p.pid)
-        self.assertEquals("<Process pid=42>", repr(p))
+        self.assertEqual(42, p.pid)
+        self.assertEqual("<Process pid=42>", repr(p))
 
         def pidCompleteCb(result):
-            self.assertEquals(None, p.pid)
+            self.assertEqual(None, p.pid)
         return d.addCallback(pidCompleteCb)
 
 
@@ -2293,7 +2339,7 @@ class UtilTestCase(unittest.TestCase):
                          j(self.bazfoo, "executable")]
         if runtime.platform.isWindows():
             expectedPaths.append(j(self.bazbar, "executable"))
-        self.assertEquals(paths, expectedPaths)
+        self.assertEqual(paths, expectedPaths)
 
 
     def testWhichPathExt(self):
@@ -2312,7 +2358,7 @@ class UtilTestCase(unittest.TestCase):
                          j(self.bazfoo, "executable.bin")]
         if runtime.platform.isWindows():
             expectedPaths.append(j(self.bazbar, "executable"))
-        self.assertEquals(paths, expectedPaths)
+        self.assertEqual(paths, expectedPaths)
 
 
 
@@ -2334,18 +2380,34 @@ class ClosingPipesProcessProtocol(protocol.ProcessProtocol):
         self.errput += data
 
 
+
 class ClosingPipes(unittest.TestCase):
 
     def doit(self, fd):
+        """
+        Create a child process and close one of its output descriptors using
+        L{IProcessTransport.closeStdout} or L{IProcessTransport.closeStderr}.
+        Return a L{Deferred} which fires after verifying that the descriptor was
+        really closed.
+        """
         p = ClosingPipesProcessProtocol(True)
-        p.deferred.addCallbacks(
-            callback=lambda _: self.fail("I wanted an errback."),
-            errback=self._endProcess, errbackArgs=(p,))
-        reactor.spawnProcess(p, sys.executable,
-                             [sys.executable, '-u', '-c',
-                              r'raw_input(); import sys, os; os.write(%d, "foo\n"); sys.exit(42)' % fd],
-                             env=None)
-        p.transport.write('go\n')
+        self.assertFailure(p.deferred, error.ProcessTerminated)
+        p.deferred.addCallback(self._endProcess, p)
+        reactor.spawnProcess(
+            p, sys.executable, [
+                sys.executable, '-u', '-c',
+                'raw_input()\n'
+                'import sys, os, time\n'
+                # Give the system a bit of time to notice the closed
+                # descriptor.  Another option would be to poll() for HUP
+                # instead of relying on an os.write to fail with SIGPIPE. 
+                # However, that wouldn't work on OS X (or Windows?).
+                'for i in range(1000):\n'
+                '    os.write(%d, "foo\\n")\n'
+                '    time.sleep(0.01)\n'
+                'sys.exit(42)\n' % (fd,)
+                ],
+            env=None)
 
         if fd == 1:
             p.transport.closeStdout()
@@ -2354,37 +2416,48 @@ class ClosingPipes(unittest.TestCase):
         else:
             raise RuntimeError
 
+        # Give the close time to propagate
+        p.transport.write('go\n')
+
         # make the buggy case not hang
         p.transport.closeStdin()
         return p.deferred
 
+
     def _endProcess(self, reason, p):
-        self.failIf(reason.check(error.ProcessDone),
-                    'Child should fail due to EPIPE.')
-        reason.trap(error.ProcessTerminated)
+        """
+        Check that a failed write prevented the process from getting to its
+        custom exit code.
+        """
         # child must not get past that write without raising
-        self.failIfEqual(reason.value.exitCode, 42,
-                         'process reason was %r' % reason)
-        self.failUnlessEqual(p.output, '')
+        self.assertNotEquals(
+            reason.exitCode, 42, 'process reason was %r' % reason)
+        self.assertEqual(p.output, '')
         return p.errput
 
+
     def test_stdout(self):
-        """ProcessProtocol.transport.closeStdout actually closes the pipe."""
+        """
+        ProcessProtocol.transport.closeStdout actually closes the pipe.
+        """
         d = self.doit(1)
         def _check(errput):
-            self.failIfEqual(errput.find('OSError'), -1)
+            self.assertIn('OSError', errput)
             if runtime.platform.getType() != 'win32':
-                self.failIfEqual(errput.find('Broken pipe'), -1)
+                self.assertIn('Broken pipe', errput)
         d.addCallback(_check)
         return d
 
+
     def test_stderr(self):
-        """ProcessProtocol.transport.closeStderr actually closes the pipe."""
+        """
+        ProcessProtocol.transport.closeStderr actually closes the pipe.
+        """
         d = self.doit(2)
         def _check(errput):
             # there should be no stderr open, so nothing for it to
             # write the error to.
-            self.failUnlessEqual(errput, '')
+            self.assertEqual(errput, '')
         d.addCallback(_check)
         return d
 
@@ -2395,14 +2468,12 @@ if (runtime.platform.getType() != 'posix') or (not interfaces.IReactorProcess(re
     PosixProcessTestCasePTY.skip = skipMessage
     TestTwoProcessesPosix.skip = skipMessage
     FDTest.skip = skipMessage
-else:
-    # do this before running the tests: it uses SIGCHLD and stuff internally
-    lsOut = popen2.popen3("/bin/ls ZZXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")[2].read()
 
 if (runtime.platform.getType() != 'win32') or (not interfaces.IReactorProcess(reactor, None)):
     Win32ProcessTestCase.skip = skipMessage
     TestTwoProcessesNonPosix.skip = skipMessage
     Dumbwin32procPidTest.skip = skipMessage
+    Win32UnicodeEnvironmentTest.skip = skipMessage
 
 if not interfaces.IReactorProcess(reactor, None):
     ProcessTestCase.skip = skipMessage
