@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.trial.test.test_runner -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -8,15 +8,23 @@ A miscellany of code used to run Trial tests.
 Maintainer: Jonathan Lange
 """
 
+__all__ = [
+    'suiteVisit', 'TestSuite',
+
+    'DestructiveTestSuite', 'DocTestCase', 'DryRunVisitor',
+    'ErrorHolder', 'LoggedSuite', 'PyUnitTestCase',
+    'TestHolder', 'TestLoader', 'TrialRunner', 'TrialSuite',
+
+    'filenameToModule', 'isPackage', 'isPackageDirectory', 'isTestCase',
+    'name', 'samefile', 'NOT_IN_TEST',
+    ]
 
 import pdb
 import os, types, warnings, sys, inspect, imp
-import random, doctest, time
+import doctest, time
 
 from twisted.python import reflect, log, failure, modules, filepath
-from twisted.python.util import dsu
 from twisted.python.compat import set
-from twisted.python.lockfile import FilesystemLock
 
 from twisted.internet import defer
 from twisted.trial import util, unittest
@@ -32,24 +40,6 @@ pyunit = __import__('unittest')
 
 
 
-class _WorkingDirectoryBusy(Exception):
-    """
-    A working directory was specified to the runner, but another test run is
-    currently using that directory.
-    """
-
-
-
-class _NoTrialMarker(Exception):
-    """
-    No trial marker file could be found.
-
-    Raised when trial attempts to remove a trial temporary working directory
-    that does not contain a marker file.
-    """
-
-
-
 def isPackage(module):
     """Given an object return True if the object looks like a package"""
     if not isinstance(module, types.ModuleType):
@@ -130,6 +120,20 @@ def _resolveDirectory(fn):
     return fn
 
 
+def _getMethodNameInClass(method):
+    """
+    Find the attribute name on the method's class which refers to the method.
+
+    For some methods, notably decorators which have not had __name__ set correctly:
+
+    getattr(method.im_class, method.__name__) != method
+    """
+    if getattr(method.im_class, method.__name__, object()) != method:
+        for alias in dir(method.im_class):
+            if getattr(method.im_class, alias, object()) == method:
+                return alias
+    return method.__name__
+
 
 class DestructiveTestSuite(TestSuite):
     """
@@ -180,25 +184,6 @@ class LoggedSuite(TestSuite):
 
 
 
-class DocTestSuite(TestSuite):
-    """
-    DEPRECATED in Twisted 8.0.
-
-    Behaves like doctest.DocTestSuite, but decorates individual TestCases so
-    they support visit and so that id() behaviour is meaningful and consistent
-    between Python versions.
-    """
-
-    def __init__(self, testModule):
-        warnings.warn("DocTestSuite is deprecated in Twisted 8.0.",
-                      category=DeprecationWarning, stacklevel=2)
-        TestSuite.__init__(self)
-        suite = doctest.DocTestSuite(testModule)
-        for test in suite._tests: #yay encapsulation
-            self.addTest(ITestCase(test))
-
-
-
 class PyUnitTestCase(object):
     """
     DEPRECATED in Twisted 8.0.
@@ -304,7 +289,7 @@ class TrialSuite(TestSuite):
 def name(thing):
     """
     @param thing: an object from modules (instance of PythonModule,
-    PythonAttribute), a TestCase subclass, or an instance of a TestCase.
+        PythonAttribute), a TestCase subclass, or an instance of a TestCase.
     """
     if isTestCase(thing):
         # TestCase subclass
@@ -322,8 +307,8 @@ def name(thing):
 
 def isTestCase(obj):
     """
-    Returns C{True} if C{obj} is a class that contains test cases, C{False}
-    otherwise. Used to find all the tests in a module.
+    @return: C{True} if C{obj} is a class that contains test cases, C{False}
+        otherwise. Used to find all the tests in a module.
     """
     try:
         return issubclass(obj, pyunit.TestCase)
@@ -340,6 +325,8 @@ class TestHolder(object):
 
     implements(ITestCase)
 
+    failureException = None
+
     def __init__(self, description):
         """
         @param description: A string to be displayed L{TestResult}.
@@ -347,10 +334,30 @@ class TestHolder(object):
         self.description = description
 
 
+    def __call__(self, result):
+        return self.run(result)
+
+
     def id(self):
         return self.description
 
 
+    def countTestCases(self):
+        return 0
+
+
+    def run(self, result):
+        """
+        This test is just a placeholder. Run the test successfully.
+
+        @param result: The C{TestResult} to store the results in.
+        @type result: L{twisted.trial.itrial.ITestResult}.
+        """
+        result.startTest(self)
+        result.addSuccess(self)
+        result.stopTest(self)
+
+
     def shortDescription(self):
         return self.description
 
@@ -369,28 +376,30 @@ class ErrorHolder(TestHolder):
         @param description: A string used by C{TestResult}s to identify this
         error. Generally, this is the name of a module that failed to import.
 
-        @param error: The error to be added to the result. Can be an exc_info
+        @param error: The error to be added to the result. Can be an `exc_info`
         tuple or a L{twisted.python.failure.Failure}.
         """
         super(ErrorHolder, self).__init__(description)
-        self.error = error
+        self.error = util.excInfoOrFailureToExcInfo(error)
 
 
     def __repr__(self):
-        return "<ErrorHolder description=%r error=%r>" % (self.description,
-                                                          self.error)
+        return "<ErrorHolder description=%r error=%s%s>" % (
+            # Format the exception type and arguments explicitly, as exception
+            # objects do not have nice looking string formats on Python 2.4.
+            self.description, self.error[0].__name__, self.error[1].args)
 
 
     def run(self, result):
+        """
+        Run the test, reporting the error.
+
+        @param result: The C{TestResult} to store the results in.
+        @type result: L{twisted.trial.itrial.ITestResult}.
+        """
+        result.startTest(self)
         result.addError(self, self.error)
-
-
-    def __call__(self, result):
-        return self.run(result)
-
-
-    def countTestCases(self):
-        return 0
+        result.stopTest(self)
 
 
     def visit(self, visitor):
@@ -436,7 +445,7 @@ class TestLoader(object):
 
         @param xs: A list of test cases, class or modules.
         """
-        return dsu(xs, self.sorter)
+        return sorted(xs, key=self.sorter)
 
     def findTestClasses(self, module):
         """Given a module, return all Trial test classes"""
@@ -521,7 +530,7 @@ class TestLoader(object):
         """
         if not isinstance(method, types.MethodType):
             raise TypeError("%r not a method" % (method,))
-        return self._makeCase(method.im_class, method.__name__)
+        return self._makeCase(method.im_class, _getMethodNameInClass(method))
 
     def _makeCase(self, klass, methodName):
         return klass(methodName)
@@ -652,11 +661,27 @@ class TestLoader(object):
             except:
                 errors.append(ErrorHolder(name, failure.Failure()))
         suites = [self.loadAnything(thing, recurse)
-                  for thing in set(things)]
+                  for thing in self._uniqueTests(things)]
         suites.extend(errors)
         return self.suiteFactory(suites)
 
 
+    def _uniqueTests(self, things):
+        """
+        Gather unique suite objects from loaded things. This will guarantee
+        uniqueness of inherited methods on TestCases which would otherwise hash
+        to same value and collapse to one test unexpectedly if using simpler
+        means: e.g. set().
+        """
+        entries = []
+        for thing in things:
+            if isinstance(thing, types.MethodType):
+                entries.append((thing, thing.im_class))
+            else:
+                entries.append((thing,))
+        return [entry[0] for entry in set(entries)]
+
+
 
 class DryRunVisitor(object):
     """
@@ -695,76 +720,24 @@ class TrialRunner(object):
             import readline
         except ImportError:
             print "readline module not available"
-            hasattr(sys, 'exc_clear') and sys.exc_clear()
+            sys.exc_clear()
         for path in ('.pdbrc', 'pdbrc'):
             if os.path.exists(path):
                 try:
                     rcFile = file(path, 'r')
                 except IOError:
-                    hasattr(sys, 'exc_clear') and sys.exc_clear()
+                    sys.exc_clear()
                 else:
                     dbg.rcLines.extend(rcFile.readlines())
         return dbg
 
 
-    def _removeSafely(self, path):
-        """
-        Safely remove a path, recursively.
-
-        If C{path} does not contain a node named C{"_trial_marker"}, a
-        L{_NoTrialMarker} exception is raised and the path is not removed.
-
-        @type path: L{twisted.python.filepath.FilePath}
-        @param path: The absolute path to a test directory
-        """
-        if not path.child('_trial_marker').exists():
-            raise _NoTrialMarker(
-                '%r is not a trial temporary path, refusing to remove it'
-                % (path,))
-
-        try:
-            path.remove()
-        except OSError, e:
-            print ("could not remove %r, caught OSError [Errno %s]: %s"
-                   % (path, e.errno, e.strerror))
-            try:
-                newPath = filepath.FilePath('_trial_temp_old%s'
-                                            % random.randint(0, 99999999))
-                path.moveTo(newPath)
-            except OSError, e:
-                print ("could not rename path, caught OSError [Errno %s]: %s"
-                       % (e.errno, e.strerror))
-                raise
-
-
     def _setUpTestdir(self):
         self._tearDownLogFile()
         currentDir = os.getcwd()
         base = filepath.FilePath(self.workingDirectory)
-        counter = 0
-        while True:
-            if counter:
-                testdir = base.sibling('%s-%d' % (base.basename(), counter))
-            else:
-                testdir = base
-
-            self._testDirLock = FilesystemLock(testdir.path + '.lock')
-            if self._testDirLock.lock():
-                # It is not in use
-                if testdir.exists():
-                    # It exists though - delete it
-                    self._removeSafely(testdir)
-                break
-            else:
-                # It is in use
-                if self.workingDirectory == '_trial_temp':
-                    counter += 1
-                else:
-                    raise _WorkingDirectoryBusy()
-
-        testdir.makedirs()
+        testdir, self._testDirLock = util._unusedTestDirectory(base)
         os.chdir(testdir.path)
-        file('_trial_marker', 'w').close()
         return currentDir
 
 
@@ -902,4 +875,3 @@ class TrialRunner(object):
             if not result.wasSuccessful():
                 break
         return result
-
