@@ -1,17 +1,16 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2011 Twisted Matrix Laboratories.
 # See LICENSE for details
 
 """
 Tests for assertions provided by L{twisted.trial.unittest.TestCase}.
 """
 
-import warnings, StringIO
+import warnings
 from pprint import pformat
 
 from twisted.python import reflect, failure
 from twisted.python.deprecate import deprecated, getVersionString
 from twisted.python.versions import Version
-from twisted.python.util import dsu
 from twisted.internet import defer
 from twisted.trial import unittest, runner, reporter
 
@@ -33,6 +32,9 @@ class TestAssertions(unittest.TestCase):
     Tests for TestCase's assertion methods.  That is, failUnless*,
     failIf*, assert*.
 
+    Note: As of 11.2, assertEqual is preferred over the failUnlessEqual(s)
+    variants.  Tests have been modified to reflect this preference.
+
     This is pretty paranoid.  Still, a certain paranoia is healthy if you
     are testing a unit testing framework.
     """
@@ -53,12 +55,11 @@ class TestAssertions(unittest.TestCase):
 
     def test_failingException_fails(self):
         test = runner.TestLoader().loadClass(TestAssertions.FailingTest)
-        io = StringIO.StringIO()
         result = reporter.TestResult()
         test.run(result)
         self.failIf(result.wasSuccessful())
-        self.failUnlessEqual(result.errors, [])
-        self.failUnlessEqual(len(result.failures), 1)
+        self.assertEqual(result.errors, [])
+        self.assertEqual(len(result.failures), 1)
 
     def test_failIf(self):
         for notTrue in [0, 0.0, False, None, (), []]:
@@ -67,7 +68,7 @@ class TestAssertions(unittest.TestCase):
             try:
                 self.failIf(true, "failed on %r" % (true,))
             except self.failureException, e:
-                self.failUnlessEqual(str(e), "failed on %r" % (true,))
+                self.assertEqual(str(e), "failed on %r" % (true,))
             else:
                 self.fail("Call to failIf(%r) didn't fail" % (true,))
 
@@ -76,36 +77,40 @@ class TestAssertions(unittest.TestCase):
             try:
                 self.failUnless(notTrue, "failed on %r" % (notTrue,))
             except self.failureException, e:
-                self.failUnlessEqual(str(e), "failed on %r" % (notTrue,))
+                self.assertEqual(str(e), "failed on %r" % (notTrue,))
             else:
                 self.fail("Call to failUnless(%r) didn't fail" % (notTrue,))
         for true in [1, True, 'cat', [1,2], (3,4)]:
             self.failUnless(true, "failed on %r" % (true,))
 
+
     def _testEqualPair(self, first, second):
-        x = self.failUnlessEqual(first, second)
+        x = self.assertEqual(first, second)
         if x != first:
-            self.fail("failUnlessEqual should return first parameter")
+            self.fail("assertEqual should return first parameter")
+
 
     def _testUnequalPair(self, first, second):
         try:
-            self.failUnlessEqual(first, second)
+            self.assertEqual(first, second)
         except self.failureException, e:
             expected = 'not equal:\na = %s\nb = %s\n' % (
                 pformat(first), pformat(second))
             if str(e) != expected:
                 self.fail("Expected: %r; Got: %s" % (expected, str(e)))
         else:
-            self.fail("Call to failUnlessEqual(%r, %r) didn't fail"
+            self.fail("Call to assertEqual(%r, %r) didn't fail"
                       % (first, second))
 
-    def test_failUnlessEqual_basic(self):
+
+    def test_assertEqual_basic(self):
         self._testEqualPair('cat', 'cat')
         self._testUnequalPair('cat', 'dog')
         self._testEqualPair([1], [1])
         self._testUnequalPair([1], 'orange')
 
-    def test_failUnlessEqual_custom(self):
+
+    def test_assertEqual_custom(self):
         x = MockEquality('first')
         y = MockEquality('second')
         z = MockEquality('fecund')
@@ -114,7 +119,8 @@ class TestAssertions(unittest.TestCase):
         self._testUnequalPair(x, y)
         self._testUnequalPair(y, z)
 
-    def test_failUnlessEqualMessage(self):
+
+    def test_assertEqualMessage(self):
         """
         When a message is passed to L{assertEqual}, it is included in the
         error message.
@@ -127,7 +133,7 @@ class TestAssertions(unittest.TestCase):
             "message\nnot equal:\na = 'foo'\nb = 'bar'\n")
 
 
-    def test_failUnlessEqualNoneMessage(self):
+    def test_assertEqualNoneMessage(self):
         """
         If a message is specified as C{None}, it is not included in the error
         message of L{assertEqual}.
@@ -137,11 +143,11 @@ class TestAssertions(unittest.TestCase):
         self.assertEqual(str(exception), "not equal:\na = 'foo'\nb = 'bar'\n")
 
 
-    def test_failUnlessEqual_incomparable(self):
+    def test_assertEqual_incomparable(self):
         apple = MockEquality('apple')
         orange = ['orange']
         try:
-            self.failUnlessEqual(apple, orange)
+            self.assertEqual(apple, orange)
         except self.failureException:
             self.fail("Fail raised when ValueError ought to have been raised.")
         except ValueError:
@@ -151,6 +157,7 @@ class TestAssertions(unittest.TestCase):
             self.fail("Comparing %r and %r should have raised an exception"
                       % (apple, orange))
 
+
     def _raiseError(self, error):
         raise error
 
@@ -166,7 +173,7 @@ class TestAssertions(unittest.TestCase):
         except TypeError:
             self.fail("failUnlessRaises shouldn't re-raise unexpected "
                       "exceptions")
-        except self.failureException, e:
+        except self.failureException:
             # what we expect
             pass
         else:
@@ -176,7 +183,7 @@ class TestAssertions(unittest.TestCase):
         try:
             self.failUnlessRaises(ValueError, lambda : None)
         except self.failureException, e:
-            self.failUnlessEqual(str(e),
+            self.assertEqual(str(e),
                                  'ValueError not raised (None returned)')
         else:
             self.fail("Exception not raised. Should have failed")
@@ -190,7 +197,7 @@ class TestAssertions(unittest.TestCase):
         try:
             x = self.failUnlessRaises(self.failureException, self._raiseError,
                                       ValueError)
-        except self.failureException, e:
+        except self.failureException:
             # what we expect
             pass
         else:
@@ -199,7 +206,7 @@ class TestAssertions(unittest.TestCase):
     def test_failIfEqual_basic(self):
         x, y, z = [1], [2], [1]
         ret = self.failIfEqual(x, y)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              "failIfEqual should return first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failIfEqual, x, x)
@@ -211,7 +218,7 @@ class TestAssertions(unittest.TestCase):
         y = MockEquality('second')
         z = MockEquality('fecund')
         ret = self.failIfEqual(x, y)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              "failIfEqual should return first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failIfEqual, x, x)
@@ -221,7 +228,7 @@ class TestAssertions(unittest.TestCase):
     def test_failUnlessIdentical(self):
         x, y, z = [1], [1], [2]
         ret = self.failUnlessIdentical(x, x)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              'failUnlessIdentical should return first '
                              'parameter')
         self.failUnlessRaises(self.failureException,
@@ -233,7 +240,7 @@ class TestAssertions(unittest.TestCase):
         x, y, z = 1.0, 1.1, 1.2
         self.failUnlessApproximates(x, x, 0.2)
         ret = self.failUnlessApproximates(x, y, 0.2)
-        self.failUnlessEqual(ret, x, "failUnlessApproximates should return "
+        self.assertEqual(ret, x, "failUnlessApproximates should return "
                              "first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failUnlessApproximates, x, z, 0.1)
@@ -247,7 +254,7 @@ class TestAssertions(unittest.TestCase):
         z = 8.000002
         self.failUnlessAlmostEqual(x, x, precision)
         ret = self.failUnlessAlmostEqual(x, z, precision)
-        self.failUnlessEqual(ret, x, "failUnlessAlmostEqual should return "
+        self.assertEqual(ret, x, "failUnlessAlmostEqual should return "
                              "first parameter (%r, %r)" % (ret, x))
         self.failUnlessRaises(self.failureException,
                               self.failUnlessAlmostEqual, x, y, precision)
@@ -258,7 +265,7 @@ class TestAssertions(unittest.TestCase):
         y = 8.00001
         z = 8.000002
         ret = self.failIfAlmostEqual(x, y, precision)
-        self.failUnlessEqual(ret, x, "failIfAlmostEqual should return "
+        self.assertEqual(ret, x, "failIfAlmostEqual should return "
                              "first parameter (%r, %r)" % (ret, x))
         self.failUnlessRaises(self.failureException,
                               self.failIfAlmostEqual, x, x, precision)
@@ -271,7 +278,7 @@ class TestAssertions(unittest.TestCase):
         z = "the cat sat"
         self.failUnlessSubstring(x, x)
         ret = self.failUnlessSubstring(x, z)
-        self.failUnlessEqual(ret, x, 'should return first parameter')
+        self.assertEqual(ret, x, 'should return first parameter')
         self.failUnlessRaises(self.failureException,
                               self.failUnlessSubstring, x, y)
         self.failUnlessRaises(self.failureException,
@@ -283,7 +290,7 @@ class TestAssertions(unittest.TestCase):
         z = "the cat sat"
         self.failIfSubstring(z, x)
         ret = self.failIfSubstring(x, y)
-        self.failUnlessEqual(ret, x, 'should return first parameter')
+        self.assertEqual(ret, x, 'should return first parameter')
         self.failUnlessRaises(self.failureException,
                               self.failIfSubstring, x, x)
         self.failUnlessRaises(self.failureException,
@@ -357,7 +364,7 @@ class TestAssertions(unittest.TestCase):
             return a
         r = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 123)
-        self.assertEquals(r, 123)
+        self.assertEqual(r, 123)
 
 
     def test_assertWarnsRegistryClean(self):
@@ -370,11 +377,11 @@ class TestAssertions(unittest.TestCase):
             return a
         r1 = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 123)
-        self.assertEquals(r1, 123)
+        self.assertEqual(r1, 123)
         # The warning should be raised again
         r2 = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 321)
-        self.assertEquals(r2, 321)
+        self.assertEqual(r2, 321)
 
 
     def test_assertWarnsError(self):
@@ -463,10 +470,10 @@ class TestAssertions(unittest.TestCase):
         w = Warn()
         r = self.assertWarns(DeprecationWarning, "Bar deprecated", __file__,
             w.deprecated, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
         r = self.assertWarns(DeprecationWarning, "Bar deprecated", __file__,
             w.deprecated, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
 
 
     def test_assertWarnsOnCall(self):
@@ -480,10 +487,10 @@ class TestAssertions(unittest.TestCase):
         w = Warn()
         r = self.assertWarns(DeprecationWarning, "Egg deprecated", __file__,
             w, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
         r = self.assertWarns(DeprecationWarning, "Egg deprecated", __file__,
             w, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
 
 
     def test_assertWarnsFilter(self):
@@ -495,7 +502,7 @@ class TestAssertions(unittest.TestCase):
             return a
         r = self.assertWarns(PendingDeprecationWarning, "Woo deprecated",
             __file__, deprecated, 123)
-        self.assertEquals(r, 123)
+        self.assertEqual(r, 123)
 
 
     def test_assertWarnsMultipleWarnings(self):
@@ -523,7 +530,7 @@ class TestAssertions(unittest.TestCase):
         e = self.assertRaises(self.failureException,
                 self.assertWarns, DeprecationWarning, "Woo deprecated",
                 __file__, deprecated, 123)
-        self.assertEquals(str(e), "Can't handle different warnings")
+        self.assertEqual(str(e), "Can't handle different warnings")
 
 
     def test_assertWarnsAfterUnassertedWarning(self):
@@ -578,6 +585,19 @@ class TestAssertions(unittest.TestCase):
         a = A()
         self.assertRaises(self.failureException, self.assertIsInstance, a, (B, C))
 
+
+    def test_assertIsInstanceCustomMessage(self):
+        """
+        If L{TestCase.assertIsInstance} is passed a custom message as its 3rd
+        argument, the message is included in the failure exception raised when
+        the assertion fails.
+        """
+        exc = self.assertRaises(
+            self.failureException,
+            self.assertIsInstance, 3, str, "Silly assertion")
+        self.assertIn("Silly assertion", str(exc))
+
+
     def test_assertNotIsInstance(self):
         """
         Test a true condition of assertNotIsInstance.
@@ -605,7 +625,8 @@ class TestAssertions(unittest.TestCase):
         a = A()
         error = self.assertRaises(self.failureException,
                                   self.assertNotIsInstance, a, A)
-        self.assertEquals(str(error), "%r is an instance of %s" % (a, A))
+        self.assertEqual(str(error), "%r is an instance of %s" % (a, A))
+
 
     def test_assertNotIsInstanceErrorMultipleClasses(self):
         """
@@ -617,6 +638,17 @@ class TestAssertions(unittest.TestCase):
         self.assertRaises(self.failureException, self.assertNotIsInstance, a, (A, B))
 
 
+    def test_assertDictEqual(self):
+        """
+        L{twisted.trial.unittest.TestCase} supports the C{assertDictEqual}
+        method inherited from the standard library in Python 2.7.
+        """
+        self.assertDictEqual({'a': 1}, {'a': 1})
+    if getattr(unittest.TestCase, 'assertDictEqual', None) is None:
+        test_assertDictEqual.skip = (
+            "assertDictEqual is not available on this version of Python")
+
+
 
 class TestAssertionNames(unittest.TestCase):
     """
@@ -630,17 +662,27 @@ class TestAssertionNames(unittest.TestCase):
     def _name(self, x):
         return x.__name__
 
-    def test_failUnless_matches_assert(self):
-        asserts = self._getAsserts()
-        failUnlesses = reflect.prefixedMethods(self, 'failUnless')
-        self.failUnlessEqual(dsu(asserts, self._name),
-                             dsu(failUnlesses, self._name))
+
+    def test_failUnlessMatchesAssert(self):
+        """
+        The C{failUnless*} test methods are a subset of the C{assert*} test
+        methods.  This is intended to ensure that methods using the
+        I{failUnless} naming scheme are not added without corresponding methods
+        using the I{assert} naming scheme.  The I{assert} naming scheme is
+        preferred, and new I{assert}-prefixed methods may be added without
+        corresponding I{failUnless}-prefixed methods.
+        """
+        asserts = set(self._getAsserts())
+        failUnlesses = set(reflect.prefixedMethods(self, 'failUnless'))
+        self.assertEqual(
+            failUnlesses, asserts.intersection(failUnlesses))
+
 
     def test_failIf_matches_assertNot(self):
         asserts = reflect.prefixedMethods(unittest.TestCase, 'assertNot')
         failIfs = reflect.prefixedMethods(unittest.TestCase, 'failIf')
-        self.failUnlessEqual(dsu(asserts, self._name),
-                             dsu(failIfs, self._name))
+        self.assertEqual(sorted(asserts, key=self._name),
+                             sorted(failIfs, key=self._name))
 
     def test_equalSpelling(self):
         for name, value in vars(self).items():
@@ -649,11 +691,11 @@ class TestAssertionNames(unittest.TestCase):
             if name.endswith('Equal'):
                 self.failUnless(hasattr(self, name+'s'),
                                 "%s but no %ss" % (name, name))
-                self.failUnlessEqual(value, getattr(self, name+'s'))
+                self.assertEqual(value, getattr(self, name+'s'))
             if name.endswith('Equals'):
                 self.failUnless(hasattr(self, name[:-1]),
                                 "%s but no %s" % (name, name[:-1]))
-                self.failUnlessEqual(value, getattr(self, name[:-1]))
+                self.assertEqual(value, getattr(self, name[:-1]))
 
 
 class TestCallDeprecated(unittest.TestCase):
@@ -663,34 +705,28 @@ class TestCallDeprecated(unittest.TestCase):
 
     version = Version('Twisted', 8, 0, 0)
 
-    def oldMethod(self, x):
-        """
-        Deprecated method for testing.
-        """
-        return x
-
-
     def test_callDeprecatedSuppressesWarning(self):
         """
         callDeprecated calls a deprecated callable, suppressing the
         deprecation warning.
         """
-        self.callDeprecated(self.version, self.oldMethod, 'foo')
+        self.callDeprecated(self.version, oldMethod, 'foo')
         self.assertEqual(
             self.flushWarnings(), [], "No warnings should be shown")
 
 
     def test_callDeprecatedCallsFunction(self):
         """
-        L{callDeprecated} actually calls the callable passed to it.
+        L{callDeprecated} actually calls the callable passed to it, and
+        forwards the result.
         """
-        result = self.callDeprecated(self.version, self.oldMethod, 'foo')
+        result = self.callDeprecated(self.version, oldMethod, 'foo')
         self.assertEqual('foo', result)
 
 
     def test_failsWithoutDeprecation(self):
         """
-        callDeprecated raises a test failure if the callable is not
+        L{callDeprecated} raises a test failure if the callable is not
         deprecated.
         """
         def notDeprecated():
@@ -711,7 +747,7 @@ class TestCallDeprecated(unittest.TestCase):
         exception = self.assertRaises(
             self.failureException,
             self.callDeprecated,
-            differentVersion, self.oldMethod, 'foo')
+            differentVersion, oldMethod, 'foo')
         self.assertIn(getVersionString(self.version), str(exception))
         self.assertIn(getVersionString(differentVersion), str(exception))
 
@@ -727,7 +763,7 @@ class TestCallDeprecated(unittest.TestCase):
         differentVersion = Version('Foo', 1, 2, 3)
 
         def nestedDeprecation(*args):
-            return self.oldMethod(*args)
+            return oldMethod(*args)
         nestedDeprecation = deprecated(differentVersion)(nestedDeprecation)
 
         self.callDeprecated(differentVersion, nestedDeprecation, 24)
@@ -738,5 +774,44 @@ class TestCallDeprecated(unittest.TestCase):
         warningsShown = self.flushWarnings()
         self.assertEqual(len(warningsShown), 1)
 
-TestCallDeprecated.oldMethod = deprecated(TestCallDeprecated.version)(
-    TestCallDeprecated.oldMethod)
+
+    def test_callDeprecationWithMessage(self):
+        """
+        L{callDeprecated} can take a message argument used to check the warning
+        emitted.
+        """
+        self.callDeprecated((self.version, "newMethod"),
+                            oldMethodReplaced, 1)
+
+
+    def test_callDeprecationWithWrongMessage(self):
+        """
+        If the message passed to L{callDeprecated} doesn't match,
+        L{callDeprecated} raises a test failure.
+        """
+        exception = self.assertRaises(
+            self.failureException,
+            self.callDeprecated,
+            (self.version, "something.wrong"),
+            oldMethodReplaced, 1)
+        self.assertIn(getVersionString(self.version), str(exception))
+        self.assertIn("please use newMethod instead", str(exception))
+
+
+
+
+@deprecated(TestCallDeprecated.version)
+def oldMethod(x):
+    """
+    Deprecated method for testing.
+    """
+    return x
+
+
+@deprecated(TestCallDeprecated.version, replacement="newMethod")
+def oldMethodReplaced(x):
+    """
+    Another deprecated method, which has been deprecated in favor of the
+    mythical 'newMethod'.
+    """
+    return 2 * x
