@@ -41,23 +41,23 @@ class FinderTest(packages.PackageTest):
     def test_findPackage(self):
         sample1 = self.loader.findByName('twisted')
         import twisted as sample2
-        self.failUnlessEqual(sample1, sample2)
+        self.assertEqual(sample1, sample2)
 
     def test_findModule(self):
         sample1 = self.loader.findByName('twisted.trial.test.sample')
         import sample as sample2
-        self.failUnlessEqual(sample1, sample2)
+        self.assertEqual(sample1, sample2)
 
     def test_findFile(self):
         path = util.sibpath(__file__, 'sample.py')
         sample1 = self.loader.findByName(path)
         import sample as sample2
-        self.failUnlessEqual(sample1, sample2)
+        self.assertEqual(sample1, sample2)
 
     def test_findObject(self):
         sample1 = self.loader.findByName('twisted.trial.test.sample.FooTest')
         import sample
-        self.failUnlessEqual(sample.FooTest, sample1)
+        self.assertEqual(sample.FooTest, sample1)
 
     def test_findNonModule(self):
         self.failUnlessRaises(AttributeError,
@@ -86,7 +86,7 @@ class FileTest(packages.SysPathManglingTest):
     def test_moduleInPath(self):
         sample1 = runner.filenameToModule(util.sibpath(__file__, 'sample.py'))
         import sample as sample2
-        self.failUnlessEqual(sample2, sample1)
+        self.assertEqual(sample2, sample1)
 
 
     def test_moduleNotInPath(self):
@@ -108,7 +108,7 @@ class FileTest(packages.SysPathManglingTest):
             os.path.join(self.parent, 'goodpackage', 'test_sample.py'))
         self.mangleSysPath(self.newPath)
         from goodpackage import test_sample as sample2
-        self.failUnlessEqual(os.path.splitext(sample2.__file__)[0],
+        self.assertEqual(os.path.splitext(sample2.__file__)[0],
                              os.path.splitext(sample1.__file__)[0])
 
 
@@ -116,7 +116,7 @@ class FileTest(packages.SysPathManglingTest):
         package1 = runner.filenameToModule(os.path.join(self.parent,
                                                         'goodpackage'))
         import goodpackage
-        self.failUnlessEqual(goodpackage, package1)
+        self.assertEqual(goodpackage, package1)
 
 
     def test_packageNotInPath(self):
@@ -138,7 +138,7 @@ class FileTest(packages.SysPathManglingTest):
             os.path.join(self.parent, 'goodpackage'))
         self.mangleSysPath(self.newPath)
         import goodpackage
-        self.failUnlessEqual(os.path.splitext(goodpackage.__file__)[0],
+        self.assertEqual(os.path.splitext(goodpackage.__file__)[0],
                              os.path.splitext(package1.__file__)[0])
 
 
@@ -157,7 +157,7 @@ class FileTest(packages.SysPathManglingTest):
         fd.close()
         try:
             module = runner.filenameToModule(filename)
-            self.failUnlessEqual(filename, module.__file__)
+            self.assertEqual(filename, module.__file__)
         finally:
             os.remove(filename)
 
@@ -193,21 +193,21 @@ class LoaderTest(packages.SysPathManglingTest):
     def test_sortCases(self):
         import sample
         suite = self.loader.loadClass(sample.AlphabetTest)
-        self.failUnlessEqual(['test_a', 'test_b', 'test_c'],
+        self.assertEqual(['test_a', 'test_b', 'test_c'],
                              [test._testMethodName for test in suite._tests])
         newOrder = ['test_b', 'test_c', 'test_a']
         sortDict = dict(zip(newOrder, range(3)))
         self.loader.sorter = lambda x : sortDict.get(x.shortDescription(), -1)
         suite = self.loader.loadClass(sample.AlphabetTest)
-        self.failUnlessEqual(newOrder,
+        self.assertEqual(newOrder,
                              [test._testMethodName for test in suite._tests])
 
 
     def test_loadMethod(self):
         import sample
         suite = self.loader.loadMethod(sample.FooTest.test_foo)
-        self.failUnlessEqual(1, suite.countTestCases())
-        self.failUnlessEqual('test_foo', suite._testMethodName)
+        self.assertEqual(1, suite.countTestCases())
+        self.assertEqual('test_foo', suite._testMethodName)
 
 
     def test_loadFailingMethod(self):
@@ -216,8 +216,8 @@ class LoaderTest(packages.SysPathManglingTest):
         suite = self.loader.loadMethod(erroneous.TestRegularFail.test_fail)
         result = reporter.TestResult()
         suite.run(result)
-        self.failUnlessEqual(result.testsRun, 1)
-        self.failUnlessEqual(len(result.failures), 1)
+        self.assertEqual(result.testsRun, 1)
+        self.assertEqual(len(result.failures), 1)
 
 
     def test_loadNonMethod(self):
@@ -238,8 +238,8 @@ class LoaderTest(packages.SysPathManglingTest):
         """
         import sample
         suite = self.loader.loadMethod(sample.DecorationTest.test_badDecorator)
-        self.assertEquals(1, suite.countTestCases())
-        self.assertEquals('test_badDecorator', suite._testMethodName)
+        self.assertEqual(1, suite.countTestCases())
+        self.assertEqual('test_badDecorator', suite._testMethodName)
 
 
     def test_loadGoodDecorator(self):
@@ -250,8 +250,8 @@ class LoaderTest(packages.SysPathManglingTest):
         import sample
         suite = self.loader.loadMethod(
             sample.DecorationTest.test_goodDecorator)
-        self.assertEquals(1, suite.countTestCases())
-        self.assertEquals('test_goodDecorator', suite._testMethodName)
+        self.assertEqual(1, suite.countTestCases())
+        self.assertEqual('test_goodDecorator', suite._testMethodName)
 
 
     def test_loadRenamedDecorator(self):
@@ -263,15 +263,15 @@ class LoaderTest(packages.SysPathManglingTest):
         import sample
         suite = self.loader.loadMethod(
             sample.DecorationTest.test_renamedDecorator)
-        self.assertEquals(1, suite.countTestCases())
-        self.assertEquals('test_renamedDecorator', suite._testMethodName)
+        self.assertEqual(1, suite.countTestCases())
+        self.assertEqual('test_renamedDecorator', suite._testMethodName)
 
 
     def test_loadClass(self):
         import sample
         suite = self.loader.loadClass(sample.FooTest)
-        self.failUnlessEqual(2, suite.countTestCases())
-        self.failUnlessEqual(['test_bar', 'test_foo'],
+        self.assertEqual(2, suite.countTestCases())
+        self.assertEqual(['test_bar', 'test_foo'],
                              [test._testMethodName for test in suite._tests])
 
 
@@ -311,7 +311,7 @@ class LoaderTest(packages.SysPathManglingTest):
     def test_loadPackage(self):
         import goodpackage
         suite = self.loader.loadPackage(goodpackage)
-        self.failUnlessEqual(7, suite.countTestCases())
+        self.assertEqual(7, suite.countTestCases())
 
 
     def test_loadNonPackage(self):
@@ -334,40 +334,40 @@ class LoaderTest(packages.SysPathManglingTest):
     def test_loadPackageRecursive(self):
         import goodpackage
         suite = self.loader.loadPackage(goodpackage, recurse=True)
-        self.failUnlessEqual(14, suite.countTestCases())
+        self.assertEqual(14, suite.countTestCases())
 
 
     def test_loadAnythingOnModule(self):
         import sample
         suite = self.loader.loadAnything(sample)
-        self.failUnlessEqual(sample.__name__,
+        self.assertEqual(sample.__name__,
                              suite._tests[0]._tests[0].__class__.__module__)
 
 
     def test_loadAnythingOnClass(self):
         import sample
         suite = self.loader.loadAnything(sample.FooTest)
-        self.failUnlessEqual(2, suite.countTestCases())
+        self.assertEqual(2, suite.countTestCases())
 
 
     def test_loadAnythingOnMethod(self):
         import sample
         suite = self.loader.loadAnything(sample.FooTest.test_foo)
-        self.failUnlessEqual(1, suite.countTestCases())
+        self.assertEqual(1, suite.countTestCases())
 
 
     def test_loadAnythingOnPackage(self):
         import goodpackage
         suite = self.loader.loadAnything(goodpackage)
         self.failUnless(isinstance(suite, self.loader.suiteFactory))
-        self.failUnlessEqual(7, suite.countTestCases())
+        self.assertEqual(7, suite.countTestCases())
 
 
     def test_loadAnythingOnPackageRecursive(self):
         import goodpackage
         suite = self.loader.loadAnything(goodpackage, recurse=True)
         self.failUnless(isinstance(suite, self.loader.suiteFactory))
-        self.failUnlessEqual(14, suite.countTestCases())
+        self.assertEqual(14, suite.countTestCases())
 
 
     def test_loadAnythingOnString(self):
@@ -382,11 +382,11 @@ class LoaderTest(packages.SysPathManglingTest):
         suite = self.loader.loadPackage(package, recurse=True)
         result = reporter.Reporter()
         suite.run(result)
-        self.failUnlessEqual(False, result.wasSuccessful())
-        self.failUnlessEqual(2, len(result.errors))
+        self.assertEqual(False, result.wasSuccessful())
+        self.assertEqual(2, len(result.errors))
         errors = [test.id() for test, error in result.errors]
         errors.sort()
-        self.failUnlessEqual(errors, ['package.test_bad_module',
+        self.assertEqual(errors, ['package.test_bad_module',
                                       'package.test_import_module'])
 
 
@@ -416,8 +416,8 @@ class LoaderTest(packages.SysPathManglingTest):
         """
         from twisted.trial.test import mockcustomsuite
         suite = self.loader.loadModule(mockcustomsuite)
-        self.failUnlessEqual(0, suite.countTestCases())
-        self.failUnlessEqual("MyCustomSuite", getattr(suite, 'name', None))
+        self.assertEqual(0, suite.countTestCases())
+        self.assertEqual("MyCustomSuite", getattr(suite, 'name', None))
 
 
     def test_loadModuleWith_testSuite(self):
