@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 #
 # Maintainer: Jonathan Lange
@@ -9,6 +9,7 @@ Tests for L{twisted.trial.reporter}.
 
 
 import errno, sys, os, re, StringIO
+from inspect import getmro
 
 from twisted.internet.utils import suppressWarnings
 from twisted.python import log
@@ -17,6 +18,7 @@ from twisted.trial import itrial, unittest, runner, reporter, util
 from twisted.trial.reporter import UncleanWarningsReporterWrapper
 from twisted.trial.test import erroneous
 from twisted.trial.unittest import makeTodo, SkipTest, Todo
+from twisted.trial.test import sample
 
 
 class BrokenStream(object):
@@ -113,35 +115,53 @@ class TestErrorReporting(StringTest):
         suite.run(self.result)
         return self.result
 
-    def testFormatErroredMethod(self):
+    def test_formatErroredMethod(self):
+        """
+        A test method which runs and has an error recorded against it is
+        reported in the output stream with the I{ERROR} tag along with a summary
+        of what error was reported and the ID of the test.
+        """
         suite = self.loader.loadClass(erroneous.TestFailureInSetUp)
         output = self.getOutput(suite).splitlines()
-        match = [self.doubleSeparator,
-                 ('[ERROR]: twisted.trial.test.erroneous.'
-                  'TestFailureInSetUp.test_noop'),
-                 'Traceback (most recent call last):',
-                 re.compile(r'^\s+File .*erroneous\.py., line \d+, in setUp$'),
-                 re.compile(r'^\s+raise FoolishError, '
-                            r'.I am a broken setUp method.$'),
-                 ('twisted.trial.test.erroneous.FoolishError: '
-                  'I am a broken setUp method')]
+        match = [
+            self.doubleSeparator,
+            '[ERROR]',
+            'Traceback (most recent call last):',
+            re.compile(r'^\s+File .*erroneous\.py., line \d+, in setUp$'),
+            re.compile(r'^\s+raise FoolishError, '
+                       r'.I am a broken setUp method.$'),
+            ('twisted.trial.test.erroneous.FoolishError: '
+             'I am a broken setUp method'),
+            'twisted.trial.test.erroneous.TestFailureInSetUp.test_noop']
         self.stringComparison(match, output)
 
-    def testFormatFailedMethod(self):
+
+    def test_formatFailedMethod(self):
+        """
+        A test method which runs and has a failure recorded against it is
+        reported in the output stream with the I{FAIL} tag along with a summary
+        of what failure was reported and the ID of the test.
+        """
         suite = self.loader.loadMethod(erroneous.TestRegularFail.test_fail)
         output = self.getOutput(suite).splitlines()
         match = [
             self.doubleSeparator,
-            '[FAIL]: '
-            'twisted.trial.test.erroneous.TestRegularFail.test_fail',
+            '[FAIL]',
             'Traceback (most recent call last):',
             re.compile(r'^\s+File .*erroneous\.py., line \d+, in test_fail$'),
             re.compile(r'^\s+self\.fail\("I fail"\)$'),
-            'twisted.trial.unittest.FailTest: I fail'
+            'twisted.trial.unittest.FailTest: I fail',
+            'twisted.trial.test.erroneous.TestRegularFail.test_fail',
             ]
         self.stringComparison(match, output)
 
-    def testDoctestError(self):
+
+    def test_doctestError(self):
+        """
+        A problem encountered while running a doctest is reported in the output
+        stream with a I{FAIL} or I{ERROR} tag along with a summary of what
+        problem was encountered and the ID of the test.
+        """
         from twisted.trial.test import erroneous
         suite = unittest.decorate(
             self.loader.loadDoctests(erroneous), itrial.ITestCase)
@@ -154,10 +174,11 @@ class TestErrorReporting(StringTest):
                         "Couldn't match 'Failure in example: ' "
                         "or 'Failed example: '")
         expect = [self.doubleSeparator,
-                  re.compile(r'\[(ERROR|FAIL)\]: .*' + re.escape(path))]
+                  re.compile(r'\[(ERROR|FAIL)\]')]
         self.stringComparison(expect, output.splitlines())
 
-    def testHiddenException(self):
+
+    def test_hiddenException(self):
         """
         Check that errors in C{DelayedCall}s get reported, even if the
         test already has a failure.
@@ -170,8 +191,7 @@ class TestErrorReporting(StringTest):
         output = self.getOutput(test).splitlines()
         match = [
             self.doubleSeparator,
-            '[FAIL]: '
-            'twisted.trial.test.erroneous.DelayedCall.testHiddenException',
+            '[FAIL]',
             'Traceback (most recent call last):',
             re.compile(r'^\s+File .*erroneous\.py., line \d+, in '
                        'testHiddenException$'),
@@ -179,17 +199,17 @@ class TestErrorReporting(StringTest):
                        'hidden exception"\)$'),
             'twisted.trial.unittest.FailTest: '
             'Deliberate failure to mask the hidden exception',
-            self.doubleSeparator,
-            '[ERROR]: '
             'twisted.trial.test.erroneous.DelayedCall.testHiddenException',
+            self.doubleSeparator,
+            '[ERROR]',
             'Traceback (most recent call last):',
             re.compile(r'^\s+File .* in runUntilCurrent'),
             re.compile(r'^\s+.*'),
             re.compile('^\s+File .*erroneous\.py", line \d+, in go'),
             re.compile('^\s+raise RuntimeError\(self.hiddenExceptionMsg\)'),
             'exceptions.RuntimeError: something blew up',
+            'twisted.trial.test.erroneous.DelayedCall.testHiddenException',
             ]
-
         self.stringComparison(match, output)
 
 
@@ -294,13 +314,13 @@ exceptions.TypeError: iterable argument required
 
     def test_mutation(self):
         frames = self.f.frames[:]
-        tb = self.result._formatFailureTraceback(self.f)
+        # The call shouldn't mutate the frames.
+        self.result._formatFailureTraceback(self.f)
         self.assertEqual(self.f.frames, frames)
 
 
 class PyunitTestNames(unittest.TestCase):
     def setUp(self):
-        from twisted.trial.test import sample
         self.stream = StringIO.StringIO()
         self.test = sample.PyunitTest('test_foo')
 
@@ -308,7 +328,7 @@ class PyunitTestNames(unittest.TestCase):
         result = reporter.VerboseTextReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
-        self.failUnlessEqual(
+        self.assertEqual(
             output, 'twisted.trial.test.sample.PyunitTest.test_foo ... ')
 
     def test_treeReporter(self):
@@ -316,12 +336,12 @@ class PyunitTestNames(unittest.TestCase):
         result.startTest(self.test)
         output = self.stream.getvalue()
         output = output.splitlines()[-1].strip()
-        self.failUnlessEqual(output, result.getDescription(self.test) + ' ...')
+        self.assertEqual(output, result.getDescription(self.test) + ' ...')
 
     def test_getDescription(self):
         result = reporter.TreeReporter(self.stream)
         output = result.getDescription(self.test)
-        self.failUnlessEqual(output, 'test_foo')
+        self.assertEqual(output, 'test_foo')
 
 
     def test_minimalReporter(self):
@@ -340,7 +360,7 @@ class PyunitTestNames(unittest.TestCase):
         self.test.run(result)
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
-        self.failUnlessEqual(output[1:], ['1', '1', '0', '0', '0'])
+        self.assertEqual(output[1:], ['1', '1', '0', '0', '0'])
 
 
     def test_minimalReporterTime(self):
@@ -355,7 +375,7 @@ class PyunitTestNames(unittest.TestCase):
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
         timer = output[0]
-        self.assertEquals(timer, "0.7")
+        self.assertEqual(timer, "0.7")
 
 
     def test_emptyMinimalReporter(self):
@@ -366,7 +386,7 @@ class PyunitTestNames(unittest.TestCase):
         result = reporter.MinimalReporter(self.stream)
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
-        self.failUnlessEqual(output, ['0', '0', '0', '0', '0', '0'])
+        self.assertEqual(output, ['0', '0', '0', '0', '0', '0'])
 
 
 
@@ -387,7 +407,7 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_errorByDefault(self):
         """
-        C{DirtyReactorAggregateError}s are reported as errors with the default
+        L{DirtyReactorAggregateError}s are reported as errors with the default
         Reporter.
         """
         result = reporter.Reporter(stream=self.output)
@@ -398,8 +418,8 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_warningsEnabled(self):
         """
-        C{DirtyReactorErrors}s are reported as warnings when using the
-        L{UncleanWarningsReporterWrapper}.
+        L{DirtyReactorAggregateError}s are reported as warnings when using
+        the L{UncleanWarningsReporterWrapper}.
         """
         result = UncleanWarningsReporterWrapper(
             reporter.Reporter(stream=self.output))
@@ -410,7 +430,7 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_warningsMaskErrors(self):
         """
-        C{DirtyReactorErrors}s are I{not} reported as errors if the
+        L{DirtyReactorAggregateError}s are I{not} reported as errors if the
         L{UncleanWarningsReporterWrapper} is used.
         """
         result = UncleanWarningsReporterWrapper(
@@ -418,7 +438,7 @@ class TestDirtyReactor(unittest.TestCase):
         self.assertWarns(UserWarning, self.dirtyError.getErrorMessage(),
                          reporter.__file__,
                          result.addError, self.test, self.dirtyError)
-        self.assertEquals(result._originalReporter.errors, [])
+        self.assertEqual(result._originalReporter.errors, [])
 
 
     def test_dealsWithThreeTuples(self):
@@ -426,17 +446,17 @@ class TestDirtyReactor(unittest.TestCase):
         Some annoying stuff can pass three-tuples to addError instead of
         Failures (like PyUnit). The wrapper, of course, handles this case,
         since it is a part of L{twisted.trial.itrial.IReporter}! But it does
-        not convert L{DirtyReactorError} to warnings in this case, because
-        nobody should be passing those in the form of three-tuples.
+        not convert L{DirtyReactorAggregateError} to warnings in this case,
+        because nobody should be passing those in the form of three-tuples.
         """
         result = UncleanWarningsReporterWrapper(
             reporter.Reporter(stream=self.output))
         result.addError(self.test,
                         (self.dirtyError.type, self.dirtyError.value, None))
         self.assertEqual(len(result._originalReporter.errors), 1)
-        self.assertEquals(result._originalReporter.errors[0][1].type,
+        self.assertEqual(result._originalReporter.errors[0][1].type,
                           self.dirtyError.type)
-        self.assertEquals(result._originalReporter.errors[0][1].value,
+        self.assertEqual(result._originalReporter.errors[0][1].value,
                           self.dirtyError.value)
 
 
@@ -444,7 +464,6 @@ class TestDirtyReactor(unittest.TestCase):
 class TrialTestNames(unittest.TestCase):
 
     def setUp(self):
-        from twisted.trial.test import sample
         self.stream = StringIO.StringIO()
         self.test = sample.FooTest('test_foo')
 
@@ -452,14 +471,14 @@ class TrialTestNames(unittest.TestCase):
         result = reporter.VerboseTextReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
-        self.failUnlessEqual(output, self.test.id() + ' ... ')
+        self.assertEqual(output, self.test.id() + ' ... ')
 
     def test_treeReporter(self):
         result = reporter.TreeReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
         output = output.splitlines()[-1].strip()
-        self.failUnlessEqual(output, result.getDescription(self.test) + ' ...')
+        self.assertEqual(output, result.getDescription(self.test) + ' ...')
 
     def test_treeReporterWithDocstrings(self):
         """A docstring"""
@@ -470,7 +489,7 @@ class TrialTestNames(unittest.TestCase):
     def test_getDescription(self):
         result = reporter.TreeReporter(self.stream)
         output = result.getDescription(self.test)
-        self.failUnlessEqual(output, "test_foo")
+        self.assertEqual(output, "test_foo")
 
 
 class TestSkip(unittest.TestCase):
@@ -478,7 +497,6 @@ class TestSkip(unittest.TestCase):
     Tests for L{reporter.Reporter}'s handling of skips.
     """
     def setUp(self):
-        from twisted.trial.test import sample
         self.stream = StringIO.StringIO()
         self.result = reporter.Reporter(self.stream)
         self.test = sample.FooTest('test_foo')
@@ -495,7 +513,7 @@ class TestSkip(unittest.TestCase):
 
     def test_success(self):
         self.result.addSkip(self.test, 'some reason')
-        self.failUnlessEqual(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_summary(self):
@@ -508,7 +526,7 @@ class TestSkip(unittest.TestCase):
         output = self.stream.getvalue().splitlines()[-1]
         prefix = 'PASSED '
         self.failUnless(output.startswith(prefix))
-        self.failUnlessEqual(output[len(prefix):].strip(), '(skips=1)')
+        self.assertEqual(output[len(prefix):].strip(), '(skips=1)')
 
 
     def test_basicErrors(self):
@@ -518,8 +536,8 @@ class TestSkip(unittest.TestCase):
         """
         self.result.addSkip(self.test, 'some reason')
         self.result.done()
-        output = self.stream.getvalue().splitlines()[4]
-        self.failUnlessEqual(output.strip(), 'some reason')
+        output = self.stream.getvalue().splitlines()[3]
+        self.assertEqual(output.strip(), 'some reason')
 
 
     def test_booleanSkip(self):
@@ -530,8 +548,8 @@ class TestSkip(unittest.TestCase):
         """
         self.result.addSkip(self.test, True)
         self.result.done()
-        output = self.stream.getvalue().splitlines()[4]
-        self.failUnlessEqual(output, 'True')
+        output = self.stream.getvalue().splitlines()[3]
+        self.assertEqual(output, 'True')
 
 
     def test_exceptionSkip(self):
@@ -546,7 +564,7 @@ class TestSkip(unittest.TestCase):
         self.result.addSkip(self.test, error)
         self.result.done()
         output = '\n'.join(self.stream.getvalue().splitlines()[3:5]).strip()
-        self.failUnlessEqual(output, str(e))
+        self.assertEqual(output, str(e))
 
 
 class UncleanWarningSkipTest(TestSkip):
@@ -573,7 +591,6 @@ class TodoTest(unittest.TestCase):
     """
 
     def setUp(self):
-        from twisted.trial.test import sample
         self.stream = StringIO.StringIO()
         self.result = reporter.Reporter(self.stream)
         self.test = sample.FooTest('test_foo')
@@ -646,7 +663,7 @@ class TodoTest(unittest.TestCase):
         self.result.addExpectedFailure(self.test, Failure(Exception()),
                                        makeTodo('some reason'))
         self.result.done()
-        output = self.stream.getvalue().splitlines()[4].strip()
+        output = self.stream.getvalue().splitlines()[3].strip()
         self.assertEqual(output, "Reason: 'some reason'")
 
 
@@ -720,7 +737,6 @@ class MockColorizer:
 
 class TestTreeReporter(unittest.TestCase):
     def setUp(self):
-        from twisted.trial.test import sample
         self.test = sample.FooTest('test_foo')
         self.stream = StringIO.StringIO()
         self.result = reporter.TreeReporter(self.stream)
@@ -775,8 +791,8 @@ class TestTreeReporter(unittest.TestCase):
         """
         self.result.addSuccess(self.test)
         self.result.done()
-        self.assertEquals(self.log[1], (self.result.SUCCESS, 'PASSED'))
-        self.assertEquals(
+        self.assertEqual(self.log[1], (self.result.SUCCESS, 'PASSED'))
+        self.assertEqual(
             self.stream.getvalue().splitlines()[-1].strip(), "(successes=1)")
 
 
@@ -787,11 +803,11 @@ class TestTreeReporter(unittest.TestCase):
         """
         try:
             raise RuntimeError('foo')
-        except RuntimeError, excValue:
+        except RuntimeError:
             self.result.addError(self, sys.exc_info())
         self.result.done()
-        self.assertEquals(self.log[1], (self.result.FAILURE, 'FAILED'))
-        self.assertEquals(
+        self.assertEqual(self.log[1], (self.result.FAILURE, 'FAILED'))
+        self.assertEqual(
             self.stream.getvalue().splitlines()[-1].strip(), "(errors=1)")
 
 
@@ -812,6 +828,60 @@ class TestTreeReporter(unittest.TestCase):
         self.assertEqual([], self.result._getPreludeSegments('foo'))
 
 
+    def test_groupResults(self):
+        """
+        If two different tests have the same error, L{Reporter._groupResults}
+        includes them together in one of the tuples in the list it returns.
+        """
+        try:
+            raise RuntimeError('foo')
+        except RuntimeError:
+            self.result.addError(self, sys.exc_info())
+            self.result.addError(self.test, sys.exc_info())
+        try:
+            raise RuntimeError('bar')
+        except RuntimeError:
+            extra = sample.FooTest('test_bar')
+            self.result.addError(extra, sys.exc_info())
+        self.result.done()
+	grouped = self.result._groupResults(
+	    self.result.errors, self.result._formatFailureTraceback)
+        self.assertEqual(grouped[0][1], [self, self.test])
+        self.assertEqual(grouped[1][1], [extra])
+
+
+    def test_printResults(self):
+        """
+        L{Reporter._printResults} uses the results list and formatter callable
+        passed to it to produce groups of results to write to its output stream.
+        """
+        def formatter(n):
+            return str(n) + '\n'
+        first = sample.FooTest('test_foo')
+        second = sample.FooTest('test_bar')
+        third = sample.PyunitTest('test_foo')
+        self.result._printResults(
+            'FOO', [(first, 1), (second, 1), (third, 2)], formatter)
+        self.assertEqual(
+            self.stream.getvalue(),
+            "%(double separator)s\n"
+            "FOO\n"
+            "1\n"
+            "\n"
+            "%(first)s\n"
+            "%(second)s\n"
+            "%(double separator)s\n"
+            "FOO\n"
+            "2\n"
+            "\n"
+            "%(third)s\n" % {
+                'double separator': self.result._doubleSeparator,
+                'first': first.id(),
+                'second': second.id(),
+                'third': third.id(),
+                })
+
+
 
 class TestReporterInterface(unittest.TestCase):
     """
@@ -828,7 +898,6 @@ class TestReporterInterface(unittest.TestCase):
     resultFactory = reporter.Reporter
 
     def setUp(self):
-        from twisted.trial.test import sample
         self.test = sample.FooTest('test_foo')
         self.stream = StringIO.StringIO()
         self.publisher = log.LogPublisher()
@@ -839,7 +908,7 @@ class TestReporterInterface(unittest.TestCase):
         """
         shouldStop is False to begin with.
         """
-        self.assertEquals(False, self.result.shouldStop)
+        self.assertEqual(False, self.result.shouldStop)
 
 
     def test_shouldStopTrueAfterStop(self):
@@ -847,14 +916,14 @@ class TestReporterInterface(unittest.TestCase):
         shouldStop becomes True soon as someone calls stop().
         """
         self.result.stop()
-        self.assertEquals(True, self.result.shouldStop)
+        self.assertEqual(True, self.result.shouldStop)
 
 
     def test_wasSuccessfulInitiallyTrue(self):
         """
         wasSuccessful() is True when there have been no results reported.
         """
-        self.assertEquals(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulTrueAfterSuccesses(self):
@@ -863,7 +932,7 @@ class TestReporterInterface(unittest.TestCase):
         otherwise.
         """
         self.result.addSuccess(self.test)
-        self.assertEquals(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulFalseAfterErrors(self):
@@ -874,7 +943,7 @@ class TestReporterInterface(unittest.TestCase):
             1 / 0
         except ZeroDivisionError:
             self.result.addError(self.test, sys.exc_info())
-        self.assertEquals(False, self.result.wasSuccessful())
+        self.assertEqual(False, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulFalseAfterFailures(self):
@@ -885,7 +954,7 @@ class TestReporterInterface(unittest.TestCase):
             self.fail("foo")
         except self.failureException:
             self.result.addFailure(self.test, sys.exc_info())
-        self.assertEquals(False, self.result.wasSuccessful())
+        self.assertEqual(False, self.result.wasSuccessful())
 
 
 
@@ -1129,8 +1198,34 @@ class TestSubunitReporter(TestReporterInterface):
         subunitReturn = getattr(subunitClient, methodName)(*args, **kwargs)
         subunitOutput = stream.getvalue()
         reporterReturn = getattr(self.result, methodName)(*args, **kwargs)
-        self.assertEquals(subunitReturn, reporterReturn)
-        self.assertEquals(subunitOutput, self.stream.getvalue())
+        self.assertEqual(subunitReturn, reporterReturn)
+        self.assertEqual(subunitOutput, self.stream.getvalue())
+
+
+    def removeMethod(self, klass, methodName):
+        """
+        Remove 'methodName' from 'klass'.
+
+        If 'klass' does not have a method named 'methodName', then
+        'removeMethod' succeeds silently.
+
+        If 'klass' does have a method named 'methodName', then it is removed
+        using delattr. Also, methods of the same name are removed from all
+        base classes of 'klass', thus removing the method entirely.
+
+        @param klass: The class to remove the method from.
+        @param methodName: The name of the method to remove.
+        """
+        method = getattr(klass, methodName, None)
+        if method is None:
+            return
+        for base in getmro(klass):
+            try:
+                delattr(base, methodName)
+            except (AttributeError, TypeError):
+                break
+            else:
+                self.addCleanup(setattr, base, methodName, method)
 
 
     def test_subunitWithoutAddExpectedFailureInstalled(self):
@@ -1138,17 +1233,7 @@ class TestSubunitReporter(TestReporterInterface):
         Some versions of subunit don't have "addExpectedFailure". For these
         versions, we report expected failures as successes.
         """
-        try:
-            addExpectedFailure = reporter.TestProtocolClient.addExpectedFailure
-        except AttributeError:
-            # Then we've actually got one of those old versions installed, and
-            # the test is immediately applicable.
-            pass
-        else:
-            del reporter.TestProtocolClient.addExpectedFailure
-            self.addCleanup(
-                setattr, reporter.TestProtocolClient, 'addExpectedFailure',
-                addExpectedFailure)
+        self.removeMethod(reporter.TestProtocolClient, 'addExpectedFailure')
         try:
             1 / 0
         except ZeroDivisionError:
@@ -1157,7 +1242,7 @@ class TestSubunitReporter(TestReporterInterface):
         self.stream.truncate(0)
         self.result.addSuccess(self.test)
         successOutput = self.stream.getvalue()
-        self.assertEquals(successOutput, expectedFailureOutput)
+        self.assertEqual(successOutput, expectedFailureOutput)
 
 
     def test_subunitWithoutAddSkipInstalled(self):
@@ -1165,22 +1250,13 @@ class TestSubunitReporter(TestReporterInterface):
         Some versions of subunit don't have "addSkip". For these versions, we
         report skips as successes.
         """
-        try:
-            addSkip = reporter.TestProtocolClient.addSkip
-        except AttributeError:
-            # Then we've actually got one of those old versions installed, and
-            # the test is immediately applicable.
-            pass
-        else:
-            del reporter.TestProtocolClient.addSkip
-            self.addCleanup(
-                setattr, reporter.TestProtocolClient, 'addSkip', addSkip)
+        self.removeMethod(reporter.TestProtocolClient, 'addSkip')
         self.result.addSkip(self.test, "reason")
         skipOutput = self.stream.getvalue()
         self.stream.truncate(0)
         self.result.addSuccess(self.test)
         successOutput = self.stream.getvalue()
-        self.assertEquals(successOutput, skipOutput)
+        self.assertEqual(successOutput, skipOutput)
 
 
     def test_addExpectedFailurePassedThrough(self):
@@ -1201,7 +1277,7 @@ class TestSubunitReporter(TestReporterInterface):
         except ZeroDivisionError:
             exc_info = sys.exc_info()
             self.result.addExpectedFailure(self.test, exc_info, 'todo')
-        self.assertEquals(addExpectedFailureCalls, [(self.test, exc_info)])
+        self.assertEqual(addExpectedFailureCalls, [(self.test, exc_info)])
 
 
     def test_addSkipSendsSubunitAddSkip(self):
@@ -1218,7 +1294,7 @@ class TestSubunitReporter(TestReporterInterface):
         # subunit has addSkip.
         self.result._subunit.addSkip = addSkip
         self.result.addSkip(self.test, 'reason')
-        self.assertEquals(addSkipCalls, [(self.test, 'reason')])
+        self.assertEqual(addSkipCalls, [(self.test, 'reason')])
 
 
     def test_doneDoesNothing(self):
@@ -1227,7 +1303,7 @@ class TestSubunitReporter(TestReporterInterface):
         of results is everything. Thus, done() does nothing.
         """
         self.result.done()
-        self.assertEquals('', self.stream.getvalue())
+        self.assertEqual('', self.stream.getvalue())
 
 
     def test_startTestSendsSubunitStartTest(self):
@@ -1283,7 +1359,19 @@ class TestSubunitReporter(TestReporterInterface):
         subunitClient.addSuccess(self.test)
         subunitOutput = stream.getvalue()
         self.result.addUnexpectedSuccess(self.test, 'todo')
-        self.assertEquals(subunitOutput, self.stream.getvalue())
+        self.assertEqual(subunitOutput, self.stream.getvalue())
+
+
+    def test_loadTimeErrors(self):
+        """
+        Load-time errors are reported like normal errors.
+        """
+        test = runner.TestLoader().loadByName('doesntexist')
+        test.run(self.result)
+        output = self.stream.getvalue()
+        # Just check that 'doesntexist' is in the output, rather than
+        # assembling the expected stack trace.
+        self.assertIn('doesntexist', output)
 
 
 
@@ -1300,7 +1388,7 @@ class TestSubunitReporterNotInstalled(unittest.TestCase):
         stream = StringIO.StringIO()
         self.patch(reporter, 'TestProtocolClient', None)
         e = self.assertRaises(Exception, reporter.SubunitReporter, stream)
-        self.assertEquals("Subunit not available", str(e))
+        self.assertEqual("Subunit not available", str(e))
 
 
 
@@ -1507,7 +1595,7 @@ class AnsiColorizerTests(unittest.TestCase):
         self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
         self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
 
-        self.assertEquals(sys.modules['curses'].setUp, 1)
+        self.assertEqual(sys.modules['curses'].setUp, 1)
 
 
     def test_supportedTigetNumWrongError(self):
@@ -1558,4 +1646,4 @@ class AnsiColorizerTests(unittest.TestCase):
 
         sys.modules['curses'] = fakecurses()
         self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
-        self.assertEquals(sys.modules['curses'].setUp, 1)
+        self.assertEqual(sys.modules['curses'].setUp, 1)
