@@ -1,4 +1,4 @@
-# Copyright (c) 2005-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 #
 # Maintainer: Jonathan Lange
@@ -7,10 +7,12 @@
 
 import StringIO, os, sys
 from zope.interface import implements
+from zope.interface.verify import verifyObject
 
 from twisted.trial.itrial import IReporter, ITestCase
 from twisted.trial import unittest, runner, reporter, util
 from twisted.python import failure, log, reflect, filepath
+from twisted.python.filepath import FilePath
 from twisted.scripts import trial
 from twisted.plugins import twisted_trial
 from twisted import plugin
@@ -132,7 +134,7 @@ class TrialRunnerTestsMixin:
         self.runner._setUpLogFile = setUpLogFile
         self.runner.run(self.test)
         self.runner.run(self.test)
-        self.failUnlessEqual(len(l), 2)
+        self.assertEqual(len(l), 2)
         self.failIf(l[0] is l[1], "Should have created a new file observer")
 
 
@@ -147,7 +149,7 @@ class TrialRunnerTestsMixin:
             l.append(self.runner._logFileObject)
         self.runner._setUpLogFile = setUpLogFile
         self.runner.run(self.test)
-        self.failUnlessEqual(len(l), 1)
+        self.assertEqual(len(l), 1)
         self.failUnless(l[0].closed)
 
 
@@ -297,6 +299,9 @@ class TestRunner(unittest.TestCase):
                                'startTest', 'addSuccess', 'stopTest',
                                'startTest', 'addSuccess', 'stopTest',
                                'startTest', 'addSuccess', 'stopTest',
+                               'startTest', 'addSuccess', 'stopTest',
+                               'startTest', 'addSuccess', 'stopTest',
+                               'startTest', 'addSuccess', 'stopTest',
                                'startTest', 'addSuccess', 'stopTest']
 
 
@@ -366,7 +371,7 @@ class TestRunner(unittest.TestCase):
     def test_runner_working_directory(self):
         self.parseOptions(['--temp-directory', 'some_path'])
         runner = self.getRunner()
-        self.assertEquals(runner.workingDirectory, 'some_path')
+        self.assertEqual(runner.workingDirectory, 'some_path')
 
 
     def test_concurrentImplicitWorkingDirectory(self):
@@ -377,8 +382,16 @@ class TestRunner(unittest.TestCase):
         """
         self.parseOptions([])
 
-        initialDirectory = os.getcwd()
-        self.addCleanup(os.chdir, initialDirectory)
+        # Make sure we end up with the same working directory after this test
+        # as we had before it.
+        self.addCleanup(os.chdir, os.getcwd())
+
+        # Make a new directory and change into it.  This isolates us from state
+        # that other tests might have dumped into this process's temp
+        # directory.
+        runDirectory = FilePath(self.mktemp())
+        runDirectory.makedirs()
+        os.chdir(runDirectory.path)
 
         firstRunner = self.getRunner()
         secondRunner = self.getRunner()
@@ -395,7 +408,7 @@ class TestRunner(unittest.TestCase):
                 # Change the working directory to the value it had before this
                 # test suite was started.
                 where['concurrent'] = subsequentDirectory = os.getcwd()
-                os.chdir(initialDirectory)
+                os.chdir(runDirectory.path)
                 self.addCleanup(os.chdir, subsequentDirectory)
 
                 secondRunner.run(ConcurrentCase('test_second'))
@@ -412,8 +425,8 @@ class TestRunner(unittest.TestCase):
             self.fail(bad[0][1])
         self.assertEqual(
             where, {
-                'concurrent': os.path.join(initialDirectory, '_trial_temp'),
-                'record': os.path.join(initialDirectory, '_trial_temp-1')})
+                'concurrent': runDirectory.child('_trial_temp').path,
+                'record': runDirectory.child('_trial_temp-1').path})
 
 
     def test_concurrentExplicitWorkingDirectory(self):
@@ -436,7 +449,7 @@ class TestRunner(unittest.TestCase):
                 assert that it raises L{_WorkingDirectoryBusy}.
                 """
                 self.assertRaises(
-                    runner._WorkingDirectoryBusy,
+                    util._WorkingDirectoryBusy,
                     secondRunner.run, ConcurrentCase('test_failure'))
 
             def test_failure(self):
@@ -477,6 +490,11 @@ class TestRunner(unittest.TestCase):
         self.assertEqual(['runcall'], debugger._calls)
 
 
+
+class RemoveSafelyTests(unittest.TestCase):
+    """
+    Tests for L{_removeSafely}.
+    """
     def test_removeSafelyNoTrialMarker(self):
         """
         If a path doesn't contain a node named C{"_trial_marker"}, that path is
@@ -486,11 +504,7 @@ class TestRunner(unittest.TestCase):
         directory = self.mktemp()
         os.mkdir(directory)
         dirPath = filepath.FilePath(directory)
-
-        self.parseOptions([])
-        myRunner = self.getRunner()
-        self.assertRaises(runner._NoTrialMarker,
-                          myRunner._removeSafely, dirPath)
+        self.assertRaises(util._NoTrialMarker, util._removeSafely, dirPath)
 
 
     def test_removeSafelyRemoveFailsMoveSucceeds(self):
@@ -508,7 +522,7 @@ class TestRunner(unittest.TestCase):
 
         # Patch stdout so we can check the print statements in _removeSafely
         out = StringIO.StringIO()
-        stdout = self.patch(sys, 'stdout', out)
+        self.patch(sys, 'stdout', out)
 
         # Set up a trial directory with a _trial_marker
         directory = self.mktemp()
@@ -518,9 +532,7 @@ class TestRunner(unittest.TestCase):
         # Ensure that path.remove() raises an OSError
         dirPath.remove = dummyRemove
 
-        self.parseOptions([])
-        myRunner = self.getRunner()
-        myRunner._removeSafely(dirPath)
+        util._removeSafely(dirPath)
         self.assertIn("could not remove FilePath", out.getvalue())
 
 
@@ -546,7 +558,7 @@ class TestRunner(unittest.TestCase):
 
         # Patch stdout so we can check the print statements in _removeSafely
         out = StringIO.StringIO()
-        stdout = self.patch(sys, 'stdout', out)
+        self.patch(sys, 'stdout', out)
 
         # Set up a trial directory with a _trial_marker
         directory = self.mktemp()
@@ -558,10 +570,8 @@ class TestRunner(unittest.TestCase):
         dirPath.remove = dummyRemove
         dirPath.moveTo = dummyMoveTo
 
-        self.parseOptions([])
-        myRunner = self.getRunner()
-        error = self.assertRaises(OSError, myRunner._removeSafely, dirPath)
-        self.assertEquals(str(error), "path movement failed")
+        error = self.assertRaises(OSError, util._removeSafely, dirPath)
+        self.assertEqual(str(error), "path movement failed")
         self.assertIn("could not remove FilePath", out.getvalue())
 
 
@@ -571,8 +581,6 @@ class TestTrialSuite(unittest.TestCase):
     def test_imports(self):
         # FIXME, HTF do you test the reactor can be cleaned up ?!!!
         from twisted.trial.runner import TrialSuite
-        # silence pyflakes warning
-        silencePyflakes = TrialSuite
 
 
 
@@ -602,9 +610,9 @@ class TestUntilFailure(unittest.TestCase):
         a few runs.
         """
         result = self.runner.runUntilFailure(self.test)
-        self.failUnlessEqual(result.testsRun, 1)
+        self.assertEqual(result.testsRun, 1)
         self.failIf(result.wasSuccessful())
-        self.assertEquals(self._getFailures(result), 1)
+        self.assertEqual(self._getFailures(result), 1)
 
 
     def _getFailures(self, result):
@@ -625,10 +633,10 @@ class TestUntilFailure(unittest.TestCase):
             return test
         self.patch(unittest, "decorate", decorate)
         result = self.runner.runUntilFailure(self.test)
-        self.failUnlessEqual(result.testsRun, 1)
+        self.assertEqual(result.testsRun, 1)
 
-        self.assertEquals(len(decorated), 1)
-        self.assertEquals(decorated, [(self.test, ITestCase)])
+        self.assertEqual(len(decorated), 1)
+        self.assertEqual(decorated, [(self.test, ITestCase)])
 
 
     def test_runUntilFailureForceGCDecorate(self):
@@ -643,10 +651,10 @@ class TestUntilFailure(unittest.TestCase):
         self.patch(unittest, "decorate", decorate)
         self.runner._forceGarbageCollection = True
         result = self.runner.runUntilFailure(self.test)
-        self.failUnlessEqual(result.testsRun, 1)
+        self.assertEqual(result.testsRun, 1)
 
-        self.assertEquals(len(decorated), 2)
-        self.assertEquals(decorated,
+        self.assertEqual(len(decorated), 2)
+        self.assertEqual(decorated,
             [(self.test, ITestCase),
              (self.test, unittest._ForceGarbageCollectionDecorator)])
 
@@ -680,7 +688,7 @@ class BreakingSuite(runner.TestSuite):
     def run(self, result):
         try:
             raise RuntimeError("error that occurs outside of a test")
-        except RuntimeError, e:
+        except RuntimeError:
             log.err(failure.Failure())
 
 
@@ -741,22 +749,134 @@ class TestTestHolder(unittest.TestCase):
         L{runner.TestHolder} implements L{ITestCase}.
         """
         self.assertIdentical(self.holder, ITestCase(self.holder))
+        self.assertTrue(
+            verifyObject(ITestCase, self.holder),
+            "%r claims to provide %r but does not do so correctly."
+            % (self.holder, ITestCase))
+
+
+    def test_runsWithStandardResult(self):
+        """
+        A L{runner.TestHolder} can run against the standard Python
+        C{TestResult}.
+        """
+        result = pyunit.TestResult()
+        self.holder.run(result)
+        self.assertTrue(result.wasSuccessful())
+        self.assertEqual(1, result.testsRun)
 
 
 
-class TestErrorHolder(TestTestHolder):
-    """
-    Test L{runner.ErrorHolder} shares behaviour with L{runner.TestHolder}.
+class ErrorHolderTestsMixin(object):
     """
+    This mixin defines test methods which can be applied to a
+    L{runner.ErrorHolder} constructed with either a L{Failure} or a
+    C{exc_info}-style tuple.
 
+    Subclass this and implement C{setUp} to create C{self.holder} referring to a
+    L{runner.ErrorHolder} instance and C{self.error} referring to a L{Failure}
+    which the holder holds.
+    """
+    exceptionForTests = ZeroDivisionError('integer division or modulo by zero')
+
+    class TestResultStub(object):
+        """
+        Stub for L{TestResult}.
+        """
+        def __init__(self):
+            self.errors = []
+
+        def startTest(self, test):
+            pass
+
+        def stopTest(self, test):
+            pass
+
+        def addError(self, test, error):
+            self.errors.append((test, error))
+
+
+    def test_runsWithStandardResult(self):
+        """
+        A L{runner.ErrorHolder} can run against the standard Python
+        C{TestResult}.
+        """
+        result = pyunit.TestResult()
+        self.holder.run(result)
+        self.assertFalse(result.wasSuccessful())
+        self.assertEqual(1, result.testsRun)
+
+
+    def test_run(self):
+        """
+        L{runner.ErrorHolder} adds an error to the result when run.
+        """
+        self.holder.run(self.result)
+        self.assertEqual(
+            self.result.errors,
+            [(self.holder, (self.error.type, self.error.value, self.error.tb))])
+
+
+    def test_call(self):
+        """
+        L{runner.ErrorHolder} adds an error to the result when called.
+        """
+        self.holder(self.result)
+        self.assertEqual(
+            self.result.errors,
+            [(self.holder, (self.error.type, self.error.value, self.error.tb))])
+
+
+    def test_countTestCases(self):
+        """
+        L{runner.ErrorHolder.countTestCases} always returns 0.
+        """
+        self.assertEqual(self.holder.countTestCases(), 0)
+
+
+    def test_repr(self):
+        """
+        L{runner.ErrorHolder.__repr__} returns a string describing the error it
+        holds.
+        """
+        self.assertEqual(repr(self.holder),
+            "<ErrorHolder description='description' "
+            "error=ZeroDivisionError('integer division or modulo by zero',)>")
+
+
+
+class FailureHoldingErrorHolderTests(ErrorHolderTestsMixin, TestTestHolder):
+    """
+    Tests for L{runner.ErrorHolder} behaving similarly to L{runner.TestHolder}
+    when constructed with a L{Failure} representing its error.
+    """
     def setUp(self):
         self.description = "description"
         # make a real Failure so we can construct ErrorHolder()
         try:
-            1/0
+            raise self.exceptionForTests
         except ZeroDivisionError:
-            error = failure.Failure()
-        self.holder = runner.ErrorHolder(self.description, error)
+            self.error = failure.Failure()
+        self.holder = runner.ErrorHolder(self.description, self.error)
+        self.result = self.TestResultStub()
+
+
+
+class ExcInfoHoldingErrorHolderTests(ErrorHolderTestsMixin, TestTestHolder):
+    """
+    Tests for L{runner.ErrorHolder} behaving similarly to L{runner.TestHolder}
+    when constructed with a C{exc_info}-style tuple representing its error.
+    """
+    def setUp(self):
+        self.description = "description"
+        # make a real Failure so we can construct ErrorHolder()
+        try:
+            raise self.exceptionForTests
+        except ZeroDivisionError:
+            exceptionInfo = sys.exc_info()
+            self.error = failure.Failure()
+        self.holder = runner.ErrorHolder(self.description, exceptionInfo)
+        self.result = self.TestResultStub()
 
 
 
@@ -782,9 +902,9 @@ class TestMalformedMethod(unittest.TestCase):
         trialRunner = runner.TrialRunner(reporter.Reporter, stream=stream)
         test = TestMalformedMethod.ContainMalformed(method)
         result = trialRunner.run(test)
-        self.failUnlessEqual(result.testsRun, 1)
+        self.assertEqual(result.testsRun, 1)
         self.failIf(result.wasSuccessful())
-        self.failUnlessEqual(len(result.errors), 1)
+        self.assertEqual(len(result.errors), 1)
 
     def test_extraArg(self):
         """
@@ -822,10 +942,10 @@ class DestructiveTestSuiteTestCase(unittest.TestCase):
         test = MockTest('test_foo')
         result = reporter.TestResult()
         suite = runner.DestructiveTestSuite([test])
-        self.assertEquals(called, [])
+        self.assertEqual(called, [])
         suite.run(result)
-        self.assertEquals(called, [True])
-        self.assertEquals(suite.countTestCases(), 0)
+        self.assertEqual(called, [True])
+        self.assertEqual(suite.countTestCases(), 0)
 
 
     def test_shouldStop(self):
@@ -845,11 +965,11 @@ class DestructiveTestSuiteTestCase(unittest.TestCase):
         loader = runner.TestLoader()
         loader.suiteFactory = runner.DestructiveTestSuite
         suite = loader.loadClass(MockTest)
-        self.assertEquals(called, [])
+        self.assertEqual(called, [])
         suite.run(result)
-        self.assertEquals(called, [1])
+        self.assertEqual(called, [1])
         # The last test shouldn't have been run
-        self.assertEquals(suite.countTestCases(), 1)
+        self.assertEqual(suite.countTestCases(), 1)
 
 
     def test_cleanup(self):
@@ -863,9 +983,9 @@ class DestructiveTestSuiteTestCase(unittest.TestCase):
         test = MockTest('test_foo')
         result = reporter.TestResult()
         suite = runner.DestructiveTestSuite([test])
-        self.assertEquals(suite.countTestCases(), 1)
+        self.assertEqual(suite.countTestCases(), 1)
         suite.run(result)
-        self.assertEquals(suite.countTestCases(), 0)
+        self.assertEqual(suite.countTestCases(), 0)
 
 
 
