@@ -15,19 +15,28 @@ from OpenSSL import SSL
 class IFingerService(Interface):
 
     def getUser(user):
-        """Return a deferred returning a string"""
+        """
+        Return a deferred returning a string.
+        """
 
     def getUsers():
-        """Return a deferred returning a list of strings"""
+        """
+        Return a deferred returning a list of strings.
+        """
+
 
 class IFingerSetterService(Interface):
 
     def setUser(user, status):
-        """Set the user's status to something"""
+        """
+        Set the user's status to something.
+        """
+
 
 def catchError(err):
     return "Internal error in server"
 
+
 class FingerProtocol(basic.LineReceiver):
 
     def lineReceived(self, user):
@@ -42,10 +51,14 @@ class FingerProtocol(basic.LineReceiver):
 class IFingerFactory(Interface):
 
     def getUser(user):
-        """Return a deferred returning a string"""
+        """
+        Return a deferred returning a string.
+        """
 
     def buildProtocol(addr):
-        """Return a protocol returning a string"""
+        """
+        Return a protocol returning a string.
+        """
 
 
 class FingerFactoryFromService(protocol.ServerFactory):
@@ -63,6 +76,7 @@ components.registerAdapter(FingerFactoryFromService,
                            IFingerService,
                            IFingerFactory)
 
+
 class FingerSetterProtocol(basic.LineReceiver):
 
     def connectionMade(self):
@@ -79,10 +93,14 @@ class FingerSetterProtocol(basic.LineReceiver):
 class IFingerSetterFactory(Interface):
 
     def setUser(user, status):
-        """Return a deferred returning a string"""
+        """
+        Return a deferred returning a string.
+        """
 
     def buildProtocol(addr):
-        """Return a protocol returning a string"""
+        """
+        Return a protocol returning a string.
+        """
 
 
 class FingerSetterFactoryFromService(protocol.ServerFactory):
@@ -102,6 +120,7 @@ components.registerAdapter(FingerSetterFactoryFromService,
                            IFingerSetterService,
                            IFingerSetterFactory)
 
+
 class IRCReplyBot(irc.IRCClient):
 
     def connectionMade(self):
@@ -124,10 +143,14 @@ class IIRCClientFactory(Interface):
     """
 
     def getUser(user):
-        """Return a deferred returning a string"""
+        """
+        Return a deferred returning a string.
+        """
 
     def buildProtocol(addr):
-        """Return a protocol"""
+        """
+        Return a protocol.
+        """
 
 
 class IRCClientFactoryFromService(protocol.ClientFactory):
@@ -147,6 +170,7 @@ components.registerAdapter(IRCClientFactoryFromService,
                            IFingerService,
                            IIRCClientFactory)
 
+
 class UserStatusTree(resource.Resource):
 
     template = """<html><head><title>Users</title></head><body>
@@ -229,10 +253,15 @@ class UserStatusXR(xmlrpc.XMLRPC):
 class IPerspectiveFinger(Interface):
 
     def remote_getUser(username):
-        """return a user's status"""
+        """
+        Return a user's status.
+        """
 
     def remote_getUsers():
-        """return a user's status"""
+        """
+        Return a user's status.
+        """
+
 
 class PerspectiveFingerFromService(pb.Root):
 
@@ -258,7 +287,6 @@ class FingerService(service.Service):
 
     def __init__(self, filename):
         self.filename = filename
-        self._read()
 
     def _read(self):
         self.users = {}
@@ -275,14 +303,24 @@ class FingerService(service.Service):
     def getUsers(self):
         return defer.succeed(self.users.keys())
 
+    def startService(self):
+        self._read()
+        service.Service.startService(self)
+
+    def stopService(self):
+        service.Service.stopService(self)
+        self.call.cancel()
+
 
 class ServerContextFactory:
 
     def getContext(self):
-        """Create an SSL context.
+        """
+        Create an SSL context.
 
         This is a sample implementation that loads a certificate from a file
-        called 'server.pem'."""
+        called 'server.pem'.
+        """
         ctx = SSL.Context(SSL.SSLv23_METHOD)
         ctx.use_certificate_file('server.pem')
         ctx.use_privatekey_file('server.pem')
@@ -290,7 +328,6 @@ class ServerContextFactory:
 
 
 
-
 # Easy configuration
 
 def makeService(config):
